*-- Author :    Jenny Ivarsson   11/09/96
*--
*-- modified:   Thomas Lohse  May 9, 1999
*             -> random selection of special particle from event record
*                implemented to avoid biases)
*             -> treatment of generic heavy quark hadron as special
*                particle added
*
      SUBROUTINE ALSPEC

************************************************************************
*
* Description: Treat the special decay (for example golden J/psi K-short)
*              Before PYEVNT the selected particle (for example B0) and
*              its antiparticle were not allowed to decay.
*              Now, tables are changed and the decays are pursued properly
*              (taking mixing into account). Tables are then changed back
*              to the default values
*
* Called from: ALPYTH
*
* External references:
*
* Argument in/out description
*
************************************************************************

      IMPLICIT NONE

#include "arte/ZDECAY.fi"

      INTEGER I,particle,channel,gKC, special,
     +        idc(maxfin), oMDCY(maxfin), LUCOMP, oMDME(maxfin,5)
      INTEGER nspec, spelis(10)
      INTEGER ii, jj, ipm, ipd, gneric
      REAL RLU

#include "arte/Arte.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "arte/Arte.ff"

c----------------------------------------------------
C Find the selected particle and give it the flag "decayed",
C although it has not. (This is a legal trick to fool the program)

C  search for up to 10 special particles in the particle list
      nspec = 0
      DO I = 1, N
         IF(K(I,2).eq.gKF(abs(meson)).and.K(I,1).eq.1) THEN
	   nspec         = nspec + 1
           spelis(nspec) = I
C  Stop searching if more than 9 specials are  found
           if (nspec.gt.9) goto 10
	 ENDIF
      ENDDO

 10   continue

C  Select a special particle randomly from the list
      if (nspec.eq.0) then
        special = 0
      else
        I = nspec*RLU(1)
        I = I + 1
        I = min(nspec,I)
        special = spelis(I)
        K(special,1) = 11
      endif

C  If the special particle is a generic b/c-hadron, find the
C  last b/c-hadron in the event record, define it to be the
C  special particle, and remove all its daughters
      gneric = 0
      if (abs(gKF(abs(meson))).eq.85) then
        gneric = 5
      elseif (abs(gKF(abs(meson))).eq.84) then
        gneric = 4
      endif
      if (gneric.ne.0) then
        do  I = N, 1, -1
          if (K(I,1).ne.0.and.K(I,1).lt.21) then
            ii = abs(K(I,2))/100
            jj = ii/10
            if (mod(ii,10).eq.gneric .or. mod(jj,10).eq.gneric) then
              if (K(I,2)*gKF(abs(meson)).gt.0) then
C  last b/c-hadron in the chain found --> special
                special = I
                K(special,1) = 11
                K(special,4) = 0
                K(special,5) = 0
C  now remove all daughters of the special particle
                do  ipd = N, I+1, -1
                  if (K(I,1).ne.0.and.K(I,1).lt.21) then
                    ipm = K(ipd,3)
 15                 continue
                    if (ipm.gt.I) then
                      ipm = K(ipm,3)
                      goto 15
                    endif
                    if (ipm.eq.I) then
                      K(ipd,1) = 0
                    endif
                  endif
                enddo
                goto 20
              endif
            endif
          endif
        enddo
 20     continue
        call luedit(12)
      endif

C Allow all particles to decay, including selected particles.
C Particles with flag "decayed" will of course not decay...

      gKC = LUCOMP(gKF(abs(meson)))
      MDCY(gKC,1) = 1

      CALL LUEXEC

C Put back the selected particle to its undecayed state

      IF(special.ne.0) THEN
         K(special,1) = 1
      ENDIF

C--> Change the decay table for the selected particle and its decay products

C Loop over particles for which special decay channels have been requested

      do particle = 1, maxfin

         if(gMDCY(particle).ne.0) then

    	   gKC = LUCOMP(gKF(particle))		! Save:
	   idc(particle)   = MDCY(gKC,2)	! Entry in the decay table
	   oMDCY(particle) = MDCY(gKC,3)	! Number of decay channels

c Loop over number of selected decay channels

           do channel = 1, gMDCY(particle)

c Save the old information on the channel which will become "special"

              oMDME(particle,channel) = MDME(idc(particle)-1+channel,1)

c Switch on the special channel

              MDME(idc(particle)-1+channel,1) = 1

           enddo

c Switch off all other channels

           do channel = gMDCY(particle),oMDCY(particle)-1
             MDME(idc(particle)+channel,1) = 0
           enddo

         endif

      enddo

c--> Execute LUEXEC for the selected particle only

      call LUEXEC

c--> Change back decay tables.

      do particle = 1, maxfin

         if(gMDCY(particle).ne.0) then

         do channel = 1, gMDCY(particle)
            MDME(idc(particle)-1+channel,1) = oMDME(particle,channel)
         enddo

         do channel = gMDCY(particle),oMDCY(particle)-1
            MDME(idc(particle)+channel,1) = 1
         enddo

         endif

      enddo

c      call lulist(1)

      END

************************************************************************
* End of ALSPEC
************************************************************************
