//                              -*- Mode: C++ -*-
//: ESPosStrategy.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef ESPOSSTRATEGY_HH_
#define ESPOSSTRATEGY_HH_

#include "care/EAClusterAnalyzer.hh"
#include "care/ESPosition.hh"
/**
class ESPosStrategy.
  The most general algorithm to evaluate a point of a shower.
  Pure abstract class.

Usage:  
       <PRE>
       // Suppose you have an EMCluster* initialized somehow
       const EMCluster * cluster = ...
       // Create a position strategy with the StrategyFactory class
       StrategyFactory SF;
       // Ask for a position strategy
       ESPosStrategy* posStra = SF.PositionStrategy()
       // now evaluate the position of the cluster
       posStra->evaluatePos( cluster );
       // you can now access the data with
       ESPosition data = posStra->getData();
       </PRE>
      
*/

class ESPosStrategy : public EAClusterAnalyzer {
public:
 
  /// Constructors
  ESPosStrategy();
  //-- 
  /// Destructor: currently void but needs to be virtual
  virtual ~ESPosStrategy();
  
  //-- Behavioural operations
  ///   evaluate the position for a cluster. To be defined by subclasses.
  virtual void evaluatePos(const EMCluster* clus) = 0;
  
  //   Access functions 
  //--
  ///   Inquire for the data stored
  inline const ESPosition & getData() const{ return  posData;};

protected:
  //              Data Part 
  //--
  /// The position data 
  ESPosition posData; 

};

#endif  //   ESPOSSTRATEGY_HH_
