using namespace std;

#include "cats/Class_GEDE.hh"


  Gede::Gede(int gede, int side, float z) { 
    g_gede = gede; g_side = side; g_z = z; 
    
    g_index  = gede;
    g_pair   = 0;
    g_status = 0;
    g_sign   = 0;
    g_sl     = 0;
    g_lay    = 0;
    g_sec    = 0;
    g_alpha  = 0.;
    g_pitch  = 0.;
    g_sxmn   = 0.;
    g_sxmx   = 0.;
    g_symn   = 0.;
    g_symx   = 0.;
    g_d0     = 0.;
    g_ms     = 0.;
    g_da     = 0.;
    g_dd     = 0.;
    g_dz     = 0.;
  }

  //==============================================================================
  // Constructor Leda
  //==============================================================================
  Gede::Gede(LedaTableIterator<class VDS_Side> side, int index) {

    // determine orientation of detector
    double deltax, deltay;
    if ( side->sec == 2 || side->sec == 4 ) {  // 3 and 9 o'clock
      deltax = 5.;
      deltay = 7.;
    }
    else {                                     // 6 and 12 o'clock
      deltax = 7.;
      deltay = 5.;
    }
 
    g_gede   = index;          // ARTE gede #
    g_index  = index;          // use gede id as index
    g_side   = side->id;
    g_pair   = side->backside;
    g_status = static_cast<int>(side->status);
    // g_pair   = (int) ((signed char) side->status);
    //    g_sign   = (float) ((signed char) side->polarity);
    g_sign   = static_cast<double>(side->polarity);
    g_sl     = atoi(side->slname+2);
    g_lay    = side->lay;
    g_sec    = side->sec;
    g_z      = side->z;
    g_alpha  = side->alpha;
    g_pitch  = side->pit;
    g_sxmn   = side->sxmn;
    g_sxmx   = side->sxmn + deltax;
    g_symn   = side->symn;
    g_symx   = side->symn + deltay;
    g_d0     = side->d0;
    g_ms     = 0.;

    plane_view();

    // set the dmin (independent from rotations!) :
    g_dmin  = g_d0 
      + side->sxmn * cos( side->alpha * 3.1415926 / 180. ) 
      - side->symn * sin( side->alpha * 3.1415926 / 180. );

    // alignment parameters (contain rotation only)
    g_dd = 0;
    g_da = vdsleda_get_zrot( side->rot );
    g_dz = 0;
    g_grad  = 0;
    g_hesse = 0;

    // cos,sin of 'real' stereo angle
    g_c = cos( ( side->alpha + g_da ) * 3.1415926 / 180. );
    g_s = sin( ( side->alpha + g_da ) * 3.1415926 / 180. );
     
    strcpy( g_text, side->name );

    fill_projectors();
    lminlmax();
  }

  //==============================================================================
  // Constructor Gede
  //==============================================================================
  Gede::Gede(ArtePointer<GEDE> gede) {

    g_gede  = gede.index();
    g_index = gede.index();
    g_side  = gede.index();
    gpair_from_GEDE(gede);
    g_status  = 1;            // all MC sides are perfect of course
    g_sign    = gede->xfw[2];
 
    char slname[5];
    strncpy( slname, gede->gesl->name, 4 );
    slname[4] = '\0';
    g_sl      = atoi( slname+2 );
    g_lay     = gede->lay;
    g_sec     = gede->sec;
    g_z       = gede->zfw[0];

    // TG: Make Holmes compatible to all kind of geometries
    double gcos = gede->cos  * gede->rot[0] - gede->sin * gede->rot[1];
    double gsin = gede->sin  * gede->rot[4] - gede->cos * gede->rot[3];
    double sxmn = gede->sxmn * gede->rot[0] +
                  gede->symn * gede->rot[1];
    double sxmx = gede->sxmx * gede->rot[0] +
                  gede->symx * gede->rot[1];
    double symn = gede->sxmn * gede->rot[3] +
                  gede->symn * gede->rot[4];
    double symx = gede->sxmx * gede->rot[3] +
                  gede->symx * gede->rot[4];

    double tmp;
    if( sxmn > sxmx ){
      tmp  = sxmx;
      sxmx = sxmn;
      sxmn = tmp;
    }
    if( symn > symx ){
      tmp  = symx;
      symx = symn;
      symn = tmp;
    }

    g_alpha  = 180. / 3.1415926 * atan2( gsin, gcos );
    if ( g_alpha < -10 ) g_alpha += 360.;
    g_pitch   = gede->pit;
    g_sxmn    = sxmn;
    g_sxmx    = sxmx;
    g_symn    = symn;
    g_symx    = symx;
    g_ms      = 0.;
    plane_view();
    g_dmin    = gede->xfw[0];
    g_dd      = 0.;
    g_da      = 0.;
    g_dz      = 0.;
    g_d0      = - gede->cos * gede->sxmn
                + gede->sin * gede->symn
                + gede->xfw[0];
    g_grad    = 0.;
    g_hesse   = 0.;
    g_s       = gsin;
    g_c       = gcos;
    g_text[0] = '\0';
 
    fill_projectors();
    lminlmax();
  }

  //==============================================================================
  // plane_view()
  //==============================================================================
  void Gede::plane_view(void) {

    // determine the type of view of the current plane
    if( (-10 < g_alpha && g_alpha <  10) ||
        (170 < g_alpha && g_alpha < 190) ||
        (350 < g_alpha && g_alpha < 370)   )
      g_view = 0;
    else
      g_view = 1;
  }

  //==============================================================================
  // fill_projectors()
  //==============================================================================
  void Gede::fill_projectors(void) {
    g_v[0]  =  g_c;
    g_v[1]  = -g_s;
    g_v[2]  =  g_c * g_z;
    g_v[3]  = -g_s * g_z;
    for (int i=0; i<4; i++)
      for (int j=0; j<4; j++)
        g_vv[4*i+j] = g_v[i] * g_v[j];
  }

  //==============================================================================
  // gpair_from_GEDE()
  //==============================================================================
  void Gede::gpair_from_GEDE(ArtePointer<GEDE> gede) {

    // Determine back side of the current detector
    //
    // !!!                                          !!!
    // !!!  here we use the assumption that layers  !!!
    // !!!  1 <-> 2 and 3 <-> 4 build pairs         !!!
    // !!!                                          !!!

    for ( ArteTable<GEDE>::iterator igedeback = ArteTable<GEDE>::begin();
	  igedeback != ArteTable<GEDE>::end(); igedeback++ ) {
      ArtePointer<GEDE> gedeback = *igedeback;
      if ( gedeback->cmp == gede->cmp	                         &&
	   !strncmp( gedeback->gesl->name, gede->gesl->name, 4 ) &&
	   gedeback->sec == gede->sec                            &&
	   ( gedeback->lay == 2 && gede->lay == 1     ||
	     gedeback->lay == 1 && gede->lay == 2     ||
	     gedeback->lay == 4 && gede->lay == 3     ||
	     gedeback->lay == 3 && gede->lay == 4 )
	  ) {
	g_pair = gedeback.index();
	break;
      }
    }
    
    
  }
  
  //==============================================================================
  // lminlmax(): used in project2D()
  //==============================================================================
  void Gede::lminlmax() {
   
    g_lmin =  1000000.;
    g_lmax = -1000000.;

    double l = g_sxmn * g_s + g_symn * g_c; 
    if(l < g_lmin) g_lmin = l; 
    if(l > g_lmax) g_lmax = l;

    l = g_sxmn * g_s + g_symx * g_c; 
    if(l < g_lmin) g_lmin = l; 
    if(l > g_lmax) g_lmax = l;

    l = g_sxmx * g_s + g_symn * g_c; 
    if(l < g_lmin) g_lmin = l;
    if(l > g_lmax) g_lmax = l;

    l = g_sxmx * g_s + g_symx * g_c; 
    if(l < g_lmin) g_lmin = l;
    if(l > g_lmax) g_lmax = l;
  }

  //==============================================================================
  // Access functions
  //==============================================================================
  int    Gede::gede()    const { return g_gede;   }
  int    Gede::index()   const { return g_index;  }
  int    Gede::side()    const { return g_side;   }
  int    Gede::pair()    const { return g_pair;   }
  int    Gede::status()  const { return g_status; }
  double Gede::sign()    const { return g_sign;   }
  int    Gede::sl()      const { return g_sl;     }
  int    Gede::lay()     const { return g_lay;    }
  int    Gede::sec()     const { return g_sec;    }
  double Gede::z()       const { return g_z;      }
  double Gede::d0()      const { return g_d0;     }
  double Gede::alpha()   const { return g_alpha;  }
  double Gede::c()       const { return g_c;      }
  double Gede::s()       const { return g_s;      }
  int    Gede::view()    const { return g_view;   }
  double Gede::pitch()   const { return g_pitch;  }
  double Gede::dmin()    const { return g_dmin;   }
  double Gede::sxmn()    const { return g_sxmn;   }
  double Gede::sxmx()    const { return g_sxmx;   }
  double Gede::symn()    const { return g_symn;   }
  double Gede::symx()    const { return g_symx;   }
  double Gede::lmin()    const { return g_lmin;   }
  double Gede::lmax()    const { return g_lmax;   }
  double Gede::v(int i)  const { return g_v[i];   }
  double Gede::vv(int i) const { return g_vv[i];  }
