#ifndef VDS_CLASS_GEDE_HH
#define VDS_CLASS_GEDE_HH

#include <iostream.h>
#include <iomanip.h>
#include <cmath>
#include "vdsleda.h"
#include "arte/arte.hh"

  class Gede {
    int    g_gede;     // ARTE gede-number
    int    g_index;    // index=gede for ARTE, index=side for standalone
    int    g_side;     // side-number
    int    g_pair;     // side-number of the other side (!=0 only for double sided mdules)
    int    g_status;   // status of the side 1=normal, 0=dead, -1=ok but dont align
    double g_sign;     // sign of good cluster charge (n=+1, p=-1)
    int    g_sl;       // superlayer
    int    g_lay;      // layer
    int    g_sec;      // sector
    double g_z;        // z-position
    double g_alpha;    // stereo angle (in degrees)
    double g_pitch;    // strip pitch
    double g_sxmn;     // xmin of sensitive area
    double g_sxmx;     // xmax of sensitive area
    double g_symn;     // ymin of sensitive area
    double g_symx;     // ymax of sensitive area
    double g_d0;       // internal offset of first strip
    double g_ms;       // multiple scattering angle, left empty
    int    g_view;     // 0 = 0+-10 or 180+-10 deg, 1 = 90+-10 or 270+-10 deg
    double g_dmin;     // offset of first strip
    double g_dd;       // alignment correction for dmin
    double g_da;       // alignment correction for alpha
    double g_dz;       // alignment correction for z
    double g_grad;     // gradient in alignment fit
    double g_hesse;    // second derivative for error estimate
    double g_c;        // cosine of stereo angle
    double g_s;        // sine of stereo angle
    char   g_text[50]; // descriptive comment (trailer of input line)
    double g_v[4];     // projector for track vectors on local coordinate
    double g_vv[16];   // projector on v
    double g_lmin;     // minimum u of plane
    double g_lmax;     // maximum u of plane

    // internal functions
    void plane_view(void);
    void fill_projectors(void);
    void gpair_from_GEDE(ArtePointer<GEDE> gede);
    void lminlmax(void);

  public:
    Gede(int gede, int side, float z);
    Gede(LedaTableIterator<class VDS_Side> side, int index);
    Gede(ArtePointer<GEDE> gede);

    int    gede()    const;  
    int    index()   const; 
    int    side()    const;  
    int    pair()    const;  
    int    status()  const;
    double sign()    const;  
    int    sl()      const;    
    int    lay()     const;   
    int    sec()     const;   
    double z()       const;     
    double d0()      const;    
    double alpha()   const; 
    double c()       const;     
    double s()       const;
    int    view()    const;
    double pitch()   const;
    double dmin()    const;  
    double sxmn()    const;  
    double sxmx()    const;  
    double symn()    const;  
    double symx()    const;
    double lmin()    const;
    double lmax()    const;
    double v(int i)  const;
    double vv(int i) const;

    //
    // Overloaded operators
    //
    friend ostream& operator<<(ostream& s, Gede a) {
      cout.setf(ios::right,ios::adjustfield);
      cout.setf(ios::fixed,ios::floatfield);
      return s << a.g_text                                 << endl
	       << "gede   : "     << setw(4) << a.g_gede   << endl
	       << "index  : "     << setw(4) << a.g_index  << endl
	       << "side   : "     << setw(4) << a.g_side   << endl
	       << "pair   : "     << setw(4) << a.g_pair   << endl
	       << "status : "     << setw(4) << a.g_status << endl
	       << "sign   : "     << setw(4) << a.g_sign   << endl
	       << "sl     : "     << setw(4) << a.g_sl     << endl
	       << "lay    : "     << setw(4) << a.g_lay    << endl
	       << "sec    : "     << setw(4) << a.g_sec    << endl
	       << "view   : "     << setw(4) << a.g_view   << endl
	       << "z      : "     << setw(8) << setprecision(4) << a.g_z << endl
	       << "alpha  : "     << setw(8) << a.g_alpha  << endl
	       << "pitch  : "     << setw(8) << a.g_pitch  << endl
	       << "cos    : "     << setw(8) << a.g_c      << endl
	       << "sin    : "     << setw(8) << a.g_s      << endl
	       << "dmin   : "     << setw(8) << a.g_dmin   << endl
	       << "d0     : "     << setw(8) << a.g_d0     << endl
	       << "sxmn   : "     << setw(8) << a.g_sxmn   << endl
	       << "sxmx   : "     << setw(8) << a.g_sxmx   << endl
	       << "symn   : "     << setw(8) << a.g_symn   << endl
	       << "symx   : "     << setw(8) << a.g_symx   << endl
	       << "dd     : "     << setw(8) << a.g_dd     << endl
	       << "da     : "     << setw(8) << a.g_da     << endl
	       << "dz     : "     << setw(8) << a.g_dz     << endl;
	}
  }; // class GEDE

  //==============================================================================
  // operator < (sort according to view)
  //==============================================================================
  inline bool operator<(const Gede& lhs, const Gede& rhs) {
    return lhs.view() < rhs.view();
  }
#endif
