#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"

extern int DbLedaCompatibility;

void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
char *ku_gets(void);
char *ku_getc(void);
int ku_geti(void);
void DbToUpper( char *Csym, int len );


void dbopenact_(void) {

  char Name[128];
  char *Csym;
  char mess[80];
  char ropt[32];
  int lun, var;
  int ierr;
  
  Csym = ku_gets();
  strcpy(ropt,ku_gets());
  ropt[2]=0;
  DbToUpper(ropt,2);
  
  DbToUpper(Csym,strlen(Csym));
  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess,
       "dbopen: symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  } 

  if(MEIO(lun).concat!=0)
  {
    static int errPtr;
    sprintf(mess,
       "dbopen: do not concatenate database on %s",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  } 

  strncpy(Name,MEIO(lun).name,120);
  for(var=0;var<120 && Name[var] && Name[var]!=' ';var++);
    Name[var]=0;
    
  DbOpen( lun, Name, strcmp(ropt,"RW")==0, &ierr );

  return;
}

void dbcloseact_(void) {

  char *Csym;
  char mess[80];
  int lun;
  int ierr;
  
  Csym = ku_gets();
  
  DbToUpper(Csym,strlen(Csym));
  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess,
       "dbclose: symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbClose( lun, &ierr );

  return;
}

void dbwriteact_(void) {

  char *Csym, Name[DB_MAX_NAME_SIZE+1], mess[80]="DBWRITEACT: ";
  int  Major, Minor, n, nlink;
  char curName[DB_MAX_NAME_SIZE+1];
  int var;
  int lun, ierr;
  
  Csym   =  ku_gets();
  strcpy(Name,ku_gets());
  Major  =  ku_geti();
  Minor  =  ku_geti();

  DbToUpper( Name, strlen(Name) );
  DbToUpper( Csym, strlen(Csym) );
  
  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       " symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }
  
  /* Look for bank name in MEMO bank. */

  for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) {
    var = DB_MAX_NAME_SIZE/4;
    chhtot_(MEMO(n).name, curName, &var );
    if( strcmp( Name, curName ) == 0 ) {
      nlink = n; 
      break; 
    } 
  }

  if( nlink == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not booked in MEMO bank",Name );
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbWriteHeader(lun, (int *)((artelink_.larln1 + nlink) * 4), 
		 Major, Minor, &ierr );

  return;
}


void dbreadact_(void) {

  char *Csym, Name[DB_MAX_NAME_SIZE+1], mess[128]="DBREADACT: ";
  int Major, Minor, n, nlink;
  char Opt[16];
  char curName[DB_MAX_NAME_SIZE+1];
  int var, lun, ierr;

  Csym   =  ku_gets();
  strcpy(Name,ku_gets());
  Major  =  ku_geti();
  Minor  =  ku_geti();
  strcpy(Opt,ku_gets());
  
  DbToUpper( Csym, strlen(Csym) );

  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess,
       "DBREADACT: symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }
  
  DbToUpper( Name, strlen(Name) );

  /* Look for Bank name in MEMO bank. */

  for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) {
    var = DB_MAX_NAME_SIZE/4;
    chhtot_( MEMO(n).name, curName, &var );
    if( strcmp( Name, curName ) == 0 ) {
      nlink = n; 
      break; 
    } 
  }

  if( nlink == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not booked in MEMO bank", Name );
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbReadHeader( lun, (int *)((artelink_.larln1 + nlink) * 4), 
		Major, Minor, Opt, &ierr );

  return;
}

void dblfilact_(void) {
  char *Csym;

  Csym   =  ku_gets();

  DbListFile( Csym );

  return;
}

void dbstorrel_(void)
{
  char *Csym, Name[DB_MAX_NAME_SIZE+1], mess[80]="DBSTOREREL: ";
  int  Major, Minor, n, nlink;
  char curName[DB_MAX_NAME_SIZE+1];
  int var;
  int lun, ierr;
  
  Csym   =  ku_gets();
  strcpy(Name,ku_gets());
  Major  =  ku_geti();
  Minor  =  ku_geti();

  DbToUpper( Name, strlen(Name) );
  DbToUpper( Csym, strlen(Csym) );

  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       " symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }
  
  /* Look for bank name in MEMO bank. */

  for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) {
    var = DB_MAX_NAME_SIZE/4;
    chhtot_(MEMO(n).name, curName, &var );
    if( strcmp( Name, curName ) == 0 ) {
      nlink = n; 
      break; 
    } 
  }

  if( nlink == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not booked in MEMO bank",Name );
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbStoreLedaRel(lun, nlink, Major, Minor, &ierr );

  return;
}
	
void dbloadrel_(void)
{
  char *Csym, Name[DB_MAX_NAME_SIZE+1], mess[80]="DBLOADREL: ";
  int  Major, Minor, n, nlink;
  char curName[DB_MAX_NAME_SIZE+1];
  char Opt[16];
  int var;
  int lun, ierr;
  
  Csym   =  ku_gets();
  strcpy(Name,ku_gets());
  Major  =  ku_geti();
  Minor  =  ku_geti();
  strcpy(Opt,ku_gets());

  DbToUpper( Name, strlen(Name) );
  DbToUpper( Csym, strlen(Csym) );

  iounit_( Csym, &lun, strlen(Csym) );
  if( lun == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       " symbolic name %s unknown",Csym);
    mecErrorCount( &errPtr, mess );
    return;
  }
  
  /* Look for bank name in MEMO bank. */

  for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) {
    var = DB_MAX_NAME_SIZE/4;
    chhtot_(MEMO(n).name, curName, &var );
    if( strcmp( Name, curName ) == 0 ) {
      nlink = n; 
      break; 
    } 
  }

  if( nlink == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not booked in MEMO bank",Name );
    mecErrorCount( &errPtr, mess );
    return;
  }

  DbLoadLedaRel(lun, nlink, Major, Minor, Opt, &ierr );

  return;
}

void dbslcact_(void) {


  DbLedaCompatibility  =  ku_geti();

  return;
}


void dbsetkey_(void)
{
	
	char *tName, *fName;
	int tab, i, j, var;
	
	tName=ku_gets();
	DbToUpper(tName,strlen(tName));
	fName=ku_gets();

	
	
	{ /* Lookup table index */
	int n,nlink,var;
	char mess[80]="DbSetKey: ";
	char curName[DB_MAX_NAME_SIZE+1];

	/* Look for bank name in MEMO bank. */

	for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ )
	{
		var = DB_MAX_NAME_SIZE/4;
		chhtot_(MEMO(n).name, curName, &var );
		if( strcmp( tName, curName ) == 0 )
		{
			nlink = n;
			break;
		}
	}

	if( nlink == 0 )
	{
		static int errPtr;
		sprintf(mess+strlen(mess),
			"bank %s is not booked in MEMO bank",tName);
		mecErrorCount( &errPtr, mess );
		return;
	}

	tab=nlink;
	}
	
	j=0;
	for(i=0;i<MEMO(tab).ocol;i++)
	{
		char curName[DB_MAX_NAME_SIZE+1];
		var = DB_MAX_NAME_SIZE/4;
		chhtot_(MEMA(MEMO(tab).mema+i).name, curName, &var );
		if( strcmp( fName, curName ) == 0 )
		{
			MEMA(MEMO(tab).mema+i).key=1;
			j++;
		}
	}
	
	if(j==0)
		printf("DbSetKey: field %s not found in table %s.\n",fName,tName);
	else if(j==1)
		printf("DbSetKey: field %s in table %s defined to be a key.\n",
				fName,tName);
	else
		printf("DbSetKey: array %s[%d] in table %s defined to be a key.\n",
				fName,j,tName);
		
}


void DbToUpper( char *Csym, int len ) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Convert text to upper case.
   *
   * Called from: anywhere.
   *
   * External references: toupper function.
   *
   * Argument in/out description
   *  Csym    in/out the text to be converted
   *  len     in     lenght of Csym.
   *
   ************************************************************************/

  while ( len-- != 0 ) 
    *Csym++ = toupper(*Csym);

  return;

}

