/***********************************************************************
* Author :    Hartwig Albrecht   16/11/95
*             Siegmund Nowak     11/01/00
* Description: read geometry input for MC from DB or simple return,
*               if a command 'mmread geomin' was executed before.
* Called from: MMREVT
*
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteUtil.h"
#include "arte/ArteIOlun.h"
#include "arte/ArteMessage.hh"

extern "C" {
  void dggeomread_();
  void iounit_(char *, int &, int);
  void dggehd_();
  void dggept_();
  void mmread_(int &, char *, const int);
  void DbOpen(int,char*,int,int*);
  void DbClose(int,int*);
  void DbReadHeader(int, int *, int, int, char *,int *);
}

void dggeomread_() { 
  int lun=50, ierr=0; 
  
  if(arteflag_.geaninit != 0) return;    // for gean no input

  // if no MC -> return
  if(!ArteTable<MCIN>::booked() || !ArteTable<MCIN>::size()) return;

  ArtePointer<MCIN> min(1);

  iounit_( "GEOMIN", lun, 6 );          //  check whether GEOMIN exit before 
  int next = lun;
  ArtePointer<MEIO> mg;
  mg =ArtePointer<MEIO>(lun);
  for (;;) { 
     if(!mg->concat) break; 
     next = mg->concat;
     mg = ArtePointer<MEIO>(next);
  }

  if(ArteTable<NGHD>::booked() && ArteTable<NGHD>::size()) {  //NGHD exists  
     ArtePointer<NGHD> nhd(1);
     if(lun != next && mg->flag >2) {   // NGHD obtained via GEOMIN
       if(min->vgeo != nhd->geom) {     // take it even if different versions
           arteMsg << "dggeomread difference: mcin->vgeo " << min->vgeo 
                   << " nghd->geom " << nhd->geom;
           static int Warn = 0x00c0ead0;
           arteMsg.warn(Warn);
	} 
        return;
     }
     else {        //NGHD obtained via DB, take it if it is the same version  
        if (min->vgeo == nhd->geom) {
           if(!ArteTable<GEHD>::booked() || !ArteTable<GEHD>::size())dggehd_();
           return;
        }
        else {  // drop old NGHD if different versions - be ready for new NGHD
           mmdrop_(ArtePointer<NGHD>::getLink().getRef());
           if(ArteTable<GEHD>::booked()) mmdrop_(ArtePointer<GEHD>::getLink().getRef());
	}
     }
  }						   

  // NGHD doesn't exist, but GEHD exists   
  else if(ArteTable<GEHD>::booked() && ArteTable<GEHD>::size()) {
     ArtePointer<GEHD> ghd(1);
     if(lun != next && mg->flag >2) { // GEHD obtained via GEOMIN
       if(min->vgeo != ghd->geom) {   //if different versions -> message 
           arteMsg << "dggeomread difference: mcin->vgeo " << min->vgeo 
                   << " gehd->geom " << ghd->geom;
           static int Warn = 0x00c0ead1;
           arteMsg.warn(Warn);
           return;
	} 
     }
     else {  // take it if same version or drop old  if different versions 
        if (min->vgeo == ghd->geom) return;
        else mmdrop_(ArtePointer<GEHD>::getLink().getRef()); 
     }
  }


  // take from DB    
  iounit_( "GEODB", lun, 5 );
  ArtePointer<MEIO> me(lun);
  if(me->flag < 1) {
     arteMsg << "dggeomread: IOFILE GEODB not declared";
     static int Warn = 0x00c0ead2;      
     arteMsg.warn(Warn);
  }

  int var; char Name[128];
  strncpy(Name,me->name,120);  
  for(var=0;var<120 && Name[var] && Name[var]!=' ';var++);
  Name[var]=0;
  DbOpen( lun, Name, 0, &ierr );

  int Major = int(min->vgeo); 
  int Minor=int(10000.*(min->vgeo+1.e-6 -float(Major))); 

  int nlink=0;
  for( ArteTableIterator<MEMO>mo; mo.valid(); mo.next()) {
     if(!strncmp("NGHD", mo->name,4 ) ) {
           nlink = mo.index(); 
           break; 
     } 
  }
  if( nlink == 0 ) {
     arteMsg <<  "bank " << Name << " is not booked in MEMO bank";
     arteMsg.error();
  }

  DbReadHeader( lun, (int *)((artelink_.larln1.getRef() + nlink)*4), 
                   Major, Minor, "EQ", &ierr );

  if(ierr) {
     arteMsg << "dggeomread: detector geometry not found";
     static int Warn = 0x00c0ead1;      
     arteMsg.warn(Warn);
  }

  if(!ArteTable<GEHD>::booked() || !ArteTable<GEHD>::size()) dggehd_();
  dggept_();

  DbClose( lun, &ierr );
  return;

}


