************************************************************************
* DGMSGC
* written by Siegmund Nowak 08/93
*
* This routines translates GEANT output MIMP into ZEBRA bank structures
* JDMSG and JHITB for MSGC
*
*       called from DGEVNT
*
* Arguments  kind         description
*-----------------------------------------------------------------------
*  IPL        in          Detector number in bank JGEDE
************************************************************************

      SUBROUTINE DGMSGC(IPL)
C-----------------------------

      IMPLICIT NONE
#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "auto/MC.fi"
#include "auto/DIGB.fi"
#include "auto/HITB.fi"
#include "gean/ZGESL.fi"
      CHARACTER*4 DGMOD
      COMMON /DIGMOD/ DGMOD(20)

      INTEGER NHITDE,NMIM,NDIGDE,NMX_MERG,MULTI,ISTRIP,NN,NN1,ID
      INTEGER I,J,K,MAX,IH,N,NV,NH,NF,NL,ISN,IFMIMP,IPL,IFLAG,NWIRPL
      REAL PITCH,COSX,SINX,XT,YT,XTV,XTH,x,XLENG,DOD,SUM,SUM2
      REAL ZWIR,GASGAP,DELTAZ,RNDM,TGTRX,TGTRY,F,DELX,XTU,XTO,YVT
      REAL DIGLIM,PROBEX(4),PR13,CHG,PADH(4),SIG,EXPR,XHIT,DELT,PROX
      LOGICAL LBIGR

      INTEGER MXHIT,MXDIG,IWIR,IND0,IND1,IXMIMP,IDIGX,IDGOR,IND2,IDUMMY
      REAL XMIMP,DIGX,HTOR,DGOR,WIR
      PARAMETER (MXHIT=100,MXDIG=1000)
      COMMON /WRKSP1/ WIR(1000,2),XMIMP(MXHIT),IXMIMP(MXHIT),
     +                DIGX(4,MXDIG),HTOR(MXHIT),DGOR(MXDIG),IND0(MXHIT),
     +                IND1(MXDIG),IND2(MXHIT),IDUMMY(3400)
      DIMENSION IDIGX(4,MXDIG),IDGOR(MXDIG),IWIR(1000,2)
      EQUIVALENCE (DIGX,IDIGX),(DGOR,IDGOR),(IWIR,WIR)

      DATA     PROBEX /0.23, 0.5, 0.22, .05/, NMX_MERG /20/

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/MC.ff"
#include "auto/DIGB.ff"
#include "auto/HITB.ff"

      NMIM = 0
      IFMIMP  = IGEDE_MIM1(IPL)
      IF(IFMIMP.NE.0) NMIM  = IGEDE_MIML(IPL) - IFMIMP + 1
      IF(NMIM.LE.0) GO TO 998
      IF(NMIM.GT.MXHIT) THEN
         C78 = 'DGMSGC: NMIM > MAXHIT for IPL '
         CALL CH78AI(NMIM)
         CALL CH78AC(' for ')
         CALL CH78AI(IPL)
         CALL ME78WC(18282806)
         NMIM = MXHIT
      ENDIF

      PITCH = FGEDE_PIT(IPL)     ! wire distance
      DELT =  .5 * PITCH
      COSX =  FGEDE_ROT(1,IPL)   ! cosine of the rotation angle
      SINX =  FGEDE_ROT(2,IPL)   ! sine of the rotation angle
      DOD   = FGEDE_DOD(IPL)     ! double hit resolution
      NWIRPL= IGEDE_NWI(1,IPL)   ! number of wires in that plane
      ZWIR =  FGEDE_ZFW(1,IPL)   ! Z-position of the first wire -
      GASGAP = FGEDE_ZFW(2,IPL)-ZWIR

      IF(NWIRPL.GT.1000) THEN
         C78 = 'DGMSGC:: NWIRPL > 1000 '
         CALL CH78AI(NWIRPL)
         CALL ME78ER
      ENDIF

      LBIGR = .FALSE.
      IF(ABS(FGEDE_XMAX(IPL)).LE.ABS(FGEDE_XMIN(IPL))) LBIGR = .TRUE.

      DIGLIM = 0.
      PR13 = PROBEX(1)+PROBEX(3)

C--   first loop over all  MIMP - check whether the mimps are really
C--   inside the detector -  rotate MIMP coordinates

      DO 10 I=1,NMIM
      IH = I + IFMIMP -1
      IXMIMP(I)=  IH
      DELTAZ = ZWIR-FMIMP_Z(IH)
      XT = FMIMP_X(IH) + FMIMP_TX(IH)*DELTAZ
      YT = FMIMP_Y(IH) + FMIMP_TY(IH)*DELTAZ
      XMIMP(I) =  COSX*XT - SINX*YT
      YVT =  SINX*XT + COSX*YT
      IF(XMIMP(I).LE.FGEDE_SXMN(IPL).OR.XMIMP(I).GT.FGEDE_SXMX(IPL))THEN
         C78 = 'DGMSGC: hit outside x'
         CALL ME78MC(310267,0,ILARGEST)
         IXMIMP(I) = 0
         XMIMP(I) = -100000.
         IF(LBIGR) XMIMP(I) = 100000.
      ENDIF
      IF(YVT.LE.FGEDE_SYMN(IPL).OR.YVT.GT.FGEDE_SYMX(IPL)) THEN
         C78 = 'DGMSGC: hit outside y'
         CALL ME78MC(310268,0,ILARGEST)
         IXMIMP(I) = 0
         XMIMP(I) = -100000.
         IF(LBIGR) XMIMP(I) = 100000.
      ENDIF
      CALL GRNDM(RNDM,1)
      IF(RNDM.GT.FGEDE_EFF(IPL)) THEN
         IXMIMP(I) = 0
         XMIMP(I) = -100000.
         IF(LBIGR) XMIMP(I) = 100000.
      ENDIF
      HTOR(I) = XMIMP(I)
   10 CONTINUE

C--     sort the XMIMP matrix due to increasing x

      IF(LBIGR) THEN
         CALL SORTZV(HTOR,IND0,NMIM,1,0,0)
      ELSE
         CALL SORTZV(HTOR,IND0,NMIM,1,1,0)
      ENDIF

C--    the next loop translates the mimps into digits
C*     (not yet cleaned up)

      NDIGDE = 0
      CALL VZERO(IWIR(1,2),1000)
      DO 20 IH=1,NMIM
      I = IND0(IH)
      IF(IXMIMP(I).EQ.0) GO TO 20
      ID = IXMIMP(I)
      TGTRX = COSX*FMIMP_TX(ID) - SINX*FMIMP_TY(ID)
      TGTRY = SINX*FMIMP_TX(ID) + COSX*FMIMP_TY(ID)
      XTV = XMIMP(I) - (FGEDE_XFW(1,IPL) - DELT)
      XTH = XTV + TGTRX * GASGAP
      XLENG = SQRT(1.+TGTRX**2 + TGTRY**2) * GASGAP
      DELX = XTH - XTV
      IF(XTV.GE.0.) THEN
         NV = INT(XTV/PITCH) + 1
      ELSE
         NV = INT(XTV/PITCH)
      ENDIF
      IF(XTH.GE.0.) THEN
         NH = INT(XTH/PITCH) + 1
      ELSE
         NH = INT(XTH/PITCH)
      ENDIF
      IF(NV.GT.NH) THEN
         ISN=-1
         NF = NH
         NL = NV
         XTO = XTV
         XTU = AMAX1(XTH,PITCH*(NV-1))
      ELSE
         ISN = 1
         NF = NV
         NL = NH
         XTU = XTV
         XTO = AMIN1(XTH,PITCH*NV)
      ENDIF
      IF(DGMOD(LGESL_MSG).EQ.'MTDL') THEN
      C78 = 'MTDL in digitization not yet ready'
      CALL ME78MC(94750908,3,ILARGEST)
      ENDIF
      PROX = 1.
      IF(ABS(DELX).GT.1.E-6) PROX = (XTO-XTU)/DELX

C*    loop over strips hidden by this particle
      CALL VZERO(IWIR(MAX0(NF-2,1),1),MIN0(NL-NF+5,NWIRPL))
      DO 25 N=NV,NH,ISN
      IF(PROX.EQ.0.) GO TO 29
      CHG = ABS(XLENG * PROX)
      XHIT = .5*(XTO+XTU) - (N-.5)*PITCH

      CALL GRNDM(RNDM,1)
      IF(ABS(XHIT).LT.DELT*PR13) THEN
C         Cluster is likely to lie around the center of a strip
C         Now decide whether the multiplicity will be 1 or 3
         IF(RNDM.LE.PROBEX(1)/PR13) THEN
            PADH(1) = CHG
            ISTRIP = N
            MULTI = 1
         ELSE
            MULTI = 3
            ISTRIP = N - 1
            SIG = .424 * PITCH
         ENDIF
      ELSE
         IF(RNDM.LE.PROBEX(2)/(1.-PR13)) THEN
            MULTI = 2
            IF(XHIT.LE.0.) THEN
               ISTRIP = N-1
               PADH(2) = 0.5 * CHG * (2.-PR13- ABS(XHIT)/DELT)/(1-PR13)
               PADH(1) = CHG - PADH(2)
            ELSE
               ISTRIP = N
               PADH(1) = 0.5 * CHG * (2.-PR13- ABS(XHIT)/DELT)/(1-PR13)
               PADH(2) = CHG - PADH(1)
            ENDIF
         ELSE
            MULTI = 4
            ISTRIP = N-1
            IF(XHIT.LE.0.) ISTRIP = N-2
            SIG = .566 * PITCH
            ENDIF
      ENDIF

      DO 28 J=ISTRIP,ISTRIP+MULTI-1
      IF(J.LT.1 .OR. J.GT.NWIRPL) GO TO 28
      IF(MULTI.GT.2) THEN
         EXPR = AMIN1(160.,(((J-N)*PITCH-XHIT)/SIG) **2)
         PADH(J-ISTRIP+1) = 0.4*CHG*PITCH/SIG * EXP(-0.5*EXPR)
      ENDIF
      IF(PADH(J-ISTRIP+1).LE.0.)print*,'MTLSTR:PADH= ',PADH(J-ISTRIP+1)
      IF(IWIR(J,1).EQ.0) THEN
         NDIGDE = NDIGDE+1
         IF(NDIGDE.GT.MXDIG) THEN
            C78 = 'DGMSGC: NDIGDE>MXDIG '
            CALL ME78MC(300167,0,ILARGEST)
            NDIGDE = MXDIG
            GO TO 28
         ENDIF
         IDIGX(1,NDIGDE) = J
         DIGX(2,NDIGDE) = PADH(J-ISTRIP+1)
         IDIGX(3,NDIGDE) = ID
         IDIGX(4,NDIGDE) = OMIMP_MTRA(ID)
         IDGOR(NDIGDE) = J
         IWIR(J,1) = NDIGDE
      ELSE
         DIGX(2,IWIR(J,1)) = DIGX(2,IWIR(J,1)) + PADH(J-ISTRIP+1)
      ENDIF
      WIR(J,2) = WIR(J,2) +  PADH(J-ISTRIP+1)
   28 CONTINUE
 29   IF(NH.GT.NV) THEN
         XTU = XTO
         XTO = AMIN1(XTH,PITCH*(N+1))
         PROX = (XTO-XTU)/DELX
      ELSE IF(NH.LT.NV) THEN
         XTO = XTU
         XTU = AMAX1(XTH,PITCH*(N-2))
         PROX = (XTO-XTU)/DELX
      ENDIF
   25 CONTINUE
   20 CONTINUE

      IF(NDIGDE.LE.0) GO TO 998

C--   SORT THE DIGIT MATRIX DUE TO INCREASING X
C
      IF(LBIGR) THEN
         CALL SORTZV(IDGOR,IND1,NDIGDE,-1,0,0)
      ELSE
         CALL SORTZV(IDGOR,IND1,NDIGDE,-1,1,0)
      ENDIF

C--    The third loop cleans the digits and fills DMSG bank and relations

      IFLAG = 0
      CALL VZERO (IWIR(1,1),1000)
      DO 30 IH = 1,NDIGDE
      I = IND1(IH)
      N = IDIGX(1,I)
      IF(WIR(N,2).LT.DIGLIM) GO TO 30
      IF(IWIR(N,1).EQ.0) THEN
         CALL MMBOOK(JDIGB,1)
         NN1 = NDIGB()
         IF(IFLAG.EQ.0) THEN
            IQ(IQGEDE_DIG1(IPL)) = NN1
            IFLAG = 1
         ENDIF
         IQ(IQDIGB_ID_L()) = 10000*N + IPL
         Q (QDIGB_Q_L())   = WIR(N,2)
      ENDIF
      CALL MMBREL(JMIDI,IDIGX(3,I),NN1)
      IWIR(N,1) = IWIR(N,1) + 1
 30   CONTINUE

C*    The fourth loop creates hits from XMIMP  (new algorithm /J.Rieling)

      NHITDE = 0
      CALL VZERO(IND2,NMIM)
      IH = 1
      DO WHILE (IH.LE.NMIM)
      I = IND0(IH)
      IF(IXMIMP(I).EQ.0) GO TO 50
      IF(IND2(IH).LT.0) GO TO 50
      IF(IH.EQ.NMIM) GO TO 45           ! check whether last mimp

c     check if next hits are far enough away
c     if not define the number of hits to be merged and mark them
      MAX = 0
      XT = XMIMP(I)
      DO J = 1,NMIM
       K=IND0(J)
       IF(K.NE.I.AND.IXMIMP(K).NE.0) THEN
          IF(ABS(XMIMP(K)-XT).LT.DOD) THEN
             MAX = MAX + 1
             IND2(J) = -I
          ENDIF
       ENDIF
      ENDDO
      IF(MAX.NE.0) GO TO 40

c     last mimp or next mimp far enough, take it and translate into a hit
 45   CALL MMBOOK(JHITB,1)
      NN = NHITB()
      IF(NHITDE.EQ.0) IQ(IQGEDE_HIT1(IPL))=NN
      NHITDE = NHITDE + 1
      IQ(IQHITB_GEDE_L()) = IPL
      Q (QHITB_Z_L()) = ZWIR
      Q (QHITB_Q_L()) = 1.
      Q (QHITB_R_L()) =  FGEDE_ACC(IPL)
      CALL GRANOR(F,X)
      Q(QHITB_X_L()) = XMIMP(I) +  FGEDE_ACC(IPL)*F
      CALL MMBREL(JMIHI,IXMIMP(I),NN)
      GO TO 50

c        merge a yet unknown number of hits
c        define the number of hits to be merged

 40   IF(MAX+1.GT.NMX_MERG) GO TO 50

      IND2(IH) = -I
C     you have to merge mimps from I to I+MAX
      CALL MMBOOK(JHITB,1)
      NN = NHITB()
      IF(NHITDE.EQ.0) IQ(IQGEDE_HIT1(IPL))=NN
      NHITDE = NHITDE + 1
      SUM = 0.
      SUM2 = 0.
      MAX = MAX + 1
      DO J = 1,NMIM
       K=IND0(J)
       IF(IND2(J).EQ.-I) THEN
          SUM = SUM +XMIMP(K)
          SUM2 = SUM2 + XMIMP(K)**2
          CALL MMBREL(JMIHI,IXMIMP(K),NN)
       ENDIF
      ENDDO
      SUM = SUM /MAX
      SUM2 = SUM2/MAX
      IQ(IQHITB_GEDE_L()) = IPL
      Q (QHITB_Z_L()) = ZWIR
      Q (QHITB_Q_L()) = MAX
      Q (QHITB_R_L()) = SQRT((SUM2 - SUM**2)**2 +  FGEDE_ACC(IPL)**2)
      CALL GRANOR(F,X)
      Q (QHITB_X_L()) = SUM + Q(QHITB_R_L())*F

 50   IH = IH + 1
      ENDDO                    ! WHILE

c     fill GEDE table for hits and digits
      IQ(IQGEDE_HITL(IPL)) = NN
      IQ(IQGEDE_DIGL(IPL)) = NN1
 999  RETURN

 998  IQ(IQGEDE_DIG1(IPL)) = 0
      IQ(IQGEDE_HIT1(IPL)) = 0
      IQ(IQGEDE_DIGL(IPL)) = -1
      IQ(IQGEDE_HITL(IPL)) = -1
      RETURN
      END

