      SUBROUTINE DGPTO( IPL , iMIMF , iMIML )
***********************************************************************
*                                                                     *
* DDDDDDDDD      GGGGGGGGGG   PPPPPPPPPP   TTTTTTTTTTTT  OOOOOOOOOOOO *
* DDDDDDDDDD    GGGGGGGGGGGG  PPPPPPPPPPP  TTTTTTTTTTTT  OOOOOOOOOOOO *
* DD       DD   GG        GG  PP       PP       TT       OO        OO *
* DD        DD  GG            PP       PP       TT       OO        OO *
* DD        DD  GG            PP       PP       TT       OO        OO *
* DD        DD  GG            PPPPPPPPPPP       TT       OO        OO *
* DD        DD  GG     GGGGG  PPPPPPPPPP        TT       OO        OO *
* DD        DD  GG     GGGGG  PP                TT       OO        OO *
* DD        DD  GG        GG  PP                TT       OO        OO *
* DD       DD   GG        GG  PP                TT       OO        OO *
* DDDDDDDDDD    GGGGGGGGGGGG  PP                TT       OOOOOOOOOOOO *
* DDDDDDDDD      GGGGGGGGGG   PP                TT       OOOOOOOOOOOO *
*                                                                     *
***********************************************************************
*   Original version by I. Belyaev                May, 1997           *
*   Rewritten by A. Schwartz/A. Tumanov           Aug, 1997           *
*   Add ORTEC correction factor,                                      *
*     new gas gain factors   A. Schwartz          Sept,1997           *
*   Some impovements and correction due to new DIGB bank meaning      *
*                            I.Belyaev            18.09.97            *
*   Fix y-position indexing bug  A. Schwartz       5.12.97            *
*                                                                     *
*   Description: this routine translates mimps from GEANT output      *
*                into zebra bank DIGB for OUTER part of high-pt       *
*                system                                               *
*                                                                     *
*   Called from: DGPTL                                                *
*                                                                     *
*   Arguments:   IPL      detector plane                              *
*                iMIMF    first mimp                                  *
*                iMIML    last mimp                                   *
***********************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "auto/GEO.fi"
#include "auto/EV.fi"
#include "auto/TOPP.fi"
#include "gean/ZGESL.fi"
#include "gean/ZGMOTH.fi"

#include "digi/TMPDIG.fi"

      INTEGER    iPL, iMIMF, iMIML, iMIM

      INTEGER    I, IX_PAD, IX_STR_TOP, IX_STR_BOT, IY, IY_PAD,
     &           NX, NY, NTUB, iCELL, IDIR, ICHA 

      REAL       DIST_TOP, DIST_BOT, DIST_Y
      REAL       XPMIN , XPMAX , YPMIN , YPMAX , ZPMIN , ZPMAX 
      REAL       XSTART, YSTART, ZSTART, DXDZ, DYDZ, XEND, YEND
      REAL       PMIM, PMAS 
      REAL       XPAD, YPAD, ZPAD, XSTRAW

      REAL       QQ1, QQ2 , QQ( 10 ) 
      REAL       FACTOR1(-1:1), FACTOR2(-1:1), QDEP(-1:1)
         
      INTEGER    NNINCR
      PARAMETER (NNINCR=10) 
      REAL      YFACTOR_L(NNINCR)
      REAL      YFACTOR_C(NNINCR)
      REAL      YFACTOR_U(NNINCR)
      SAVE      YFACTOR_L
      SAVE      YFACTOR_C
      SAVE      YFACTOR_U

      REAL      YFACT_L 
      REAL      YFACT_C
      REAL      YFACT_U
      
      REAL      DTUB
      PARAMETER(DTUB=0.5)  ! Diameter of straw tube at PT1;
                                    ! this is the only hardwired number.

      LOGICAL   LCR_TALK
      PARAMETER(LCR_TALK=.TRUE.)

      LOGICAL   LFIRST 
      SAVE      LFIRST 

      INTEGER   LENOCC 
      EXTERNAL  LENOCC 

      
C     * Data statements

      DATA     LFIRST   / .TRUE. /
      
C     --y-spread values from integrating function for
C       induced signal parallel to anode wires (see NIM paper)
      DATA YFACTOR_L / 
     &  0.50000 ,
     &  0.31199 ,
     &  0.17652 ,
     &  0.09593 ,
     &  0.05146 ,
     &  0.02749 ,
     &  0.01467 ,
     &  0.00783 ,
     &  0.00418 ,
     &  0.00223 /
      DATA YFACTOR_C /
     &           0.50000 ,
     &           0.68803 ,
     &           0.82334 ,
     &           0.90406 ,
     &           0.94856 ,
     &           0.97253 ,
     &           0.98526 ,
     &           0.99221 ,
     &           0.99586 ,
     &           0.99770 /
      DATA YFACTOR_U / 
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 ,
     &                    0.00000 /
 

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/EV.ff"
#include "auto/TOPP.ff"

#include "digi/TMPDIG.ff"


****************************************************************************
*                                                                          *
*    ssssssssss   tttttttttttt   aaaaaaaaaa   rrrrrrrrrrr   tttttttttttt   *
*   ssssssssssss  tttttttttttt  aaaaaaaaaaaa  rrrrrrrrrrrr  tttttttttttt   *
*   ss        ss       tt       aa        aa  rr        rr       tt        *
*   ss                 tt       aa        aa  rr        rr       tt        *
*   sss                tt       aa        aa  rr        rr       tt        *
*    sssssssss         tt       aaaaaaaaaaaa  rrrrrrrrrrrr       tt        *
*     sssssssss        tt       aaaaaaaaaaaa  rrrrrrrrrrr        tt        *
*            sss       tt       aa        aa  rr    rr           tt        *
*             ss       tt       aa        aa  rr     rr          tt        *
*   ss        ss       tt       aa        aa  rr      rr         tt        *
*   ssssssssssss       tt       aa        aa  rr       rr        tt        *
*    ssssssssss        tt       aa        aa  rr        rr       tt        *
*                                                                          *
****************************************************************************

      
C     --some checks
      IF(  iPL   .LE. 0 .OR. iPL   .GT. NGEDE() .OR.  
     &     iMIMF .LE. 0 .OR. iMIMF .GT. NMIMP() .OR. 
     &     iMIML .LE. 0 .OR. iMIML .GT. NMIMP() )THEN
         C78='DGPTO Error! Wrong Parameters:'
         CALL CH78BC( 'iPL='   ) 
         CALL CH78AI(  iPL     )
         CALL CH78CC( 'iMIMF=' ) 
         CALL CH78AI(  iMIMF   )  
         CALL CH78CC( 'iMIML=' ) 
         CALL CH78AI(  iMIML   ) 
         CALL CH78AC( '! Skip It!' )
         CALL MEMESC(2439876,1,ILARGEST,C78(1:LENOCC(C78)))
         RETURN  
      ENDIF 
      
C     --geometry information for this Plane 
      XPMIN   = FGEDE_SXMN(     iPL )       
      XPMAX   = FGEDE_SXMX(     iPL )       
      YPMIN   = FGEDE_SYMN(     iPL )       
      YPMAX   = FGEDE_SYMX(     iPL )       
      ZPMIN   = FGEDE_ZMIN(     iPL )       
      ZPMAX   = FGEDE_ZMAX(     iPL )      
      
      NX      = IGEDE_NWI ( 1 , iPL )        
      NY      = IGEDE_NWI ( 2 , iPL )        
      
      XPAD    = ( XPMAX - XPMIN ) / FLOAT( NX )
      YPAD    = ( YPMAX - YPMIN ) / FLOAT( NY )
      ZPAD    = ( ZPMAX - ZPMIN ) 
      
      NTUB    = NINT(XPAD * (460./ZPMIN)/DTUB)
      XSTRAW  = XPAD/FLOAT(NTUB)
      
      
C     --this section can be used to read in constants via Arte
C       IO* and IF* routines or via C/C++ fopen function. 
C       Originally used to read in YFACTOR_x arrays.
C
C      IF( LFIRST )THEN
C         LFIRST = .FALSE. 
C      ENDIF 


C     * check for a temporary booking table:
      IF( JTEMP .EQ. 0 )THEN
         C78='DGPTLI warning!temporary table is not booked yet!'
         CALL ME78WC( 9238 ) 
         CALL MMBWRK( JTEMP , 0 , JTMPD_LEN , 50 ) 
      ENDIF 
      IF( N_C( JTEMP ) .NE. JTMPD_LEN )THEN
         C78='DGPTLI warning!temporary table has wrong format!'
         CALL ME78WC( 92382345 ) 
         CALL MMBWRK( JTEMP , 0 , JTMPD_LEN , 50 ) 
      ENDIF 
      IF( N_U( JTEMP ) .NE. 0 )THEN
         C78='DGPTLI warning!temporary table has wrong size'
         CALL ME78WC( 923825 ) 
         CALL MMBWRK( JTEMP , 0 , JTMPD_LEN , 50 ) 
      ENDIF 
      

C     --Main loop over mimps
      
      DO 100 iMIM = iMIMF , iMIML 
         
         XSTART =  FMIMP_X ( iMIM ) 
         YSTART =  FMIMP_Y ( iMIM )
         ZSTART =  FMIMP_Z ( iMIM ) 
         
C     --require that mimp be inside sensitive volume of plane
         IF( ZSTART .GT. ZPMAX .OR. ZSTART .LT. ZPMIN .OR. 
     &        XSTART .GT. XPMAX .OR. XSTART .LT. XPMIN .OR. 
     &        YSTART .GT. YPMAX .OR. YSTART .LT. YPMIN )THEN
            C78='DGPTO warning! MIMP is outside the sensitive volume'
            CALL MEMESC(922345,1,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100  
         ENDIF 
         
C     --require only charged particle
         IF(  OMIMP_MTRA(iMIM) .GT. 0  .AND. 
     &        OMIMP_MTRA(iMIM) .LE. NMTRA() )THEN 
            IF( OMTRA_TOPP(OMIMP_MTRA(iMIM)) .GT. 0 .AND. 
     &           OMTRA_TOPP(OMIMP_MTRA(iMIM)) .LE. NTOPP())THEN
               ICHA       = ITOPP_CHRG(OMTRA_TOPP(OMIMP_MTRA(iMIM)))/3 
               PMAS       = FTOPP_MASS(OMTRA_TOPP(OMIMP_MTRA(iMIM)))
            ELSE 
               ICHA       = 0 
            ENDIF 
         ELSE 
            ICHA = 0 
         ENDIF 
         IF( ICHA .EQ. 0 ) GO TO 100 
         
C     --obtain track information
         PMIM  = ABS( FMIMP_P ( iMIM ) ) 
         DXDZ  =      FMIMP_TX( iMIM )  
         DYDZ  =      FMIMP_TX( iMIM )
         
C     --project track to second straw tube layer assuming track 
C       is straight (since straw tube diameter is so small) 
         XEND   =  XSTART + DXDZ * XSTRAW
         YEND   =  YSTART + DYDZ * XSTRAW
         
C     --particle direction
         IDIR       = 1
         IF( FMIMP_P( iMIM ) .LT. 0 )IDIR = -1 
         
C     --determine pad intersected by track
         IX_PAD = INT( (XSTART-XPMIN)/XPAD ) + 1
         IY_PAD = INT( (YSTART-YPMIN)/YPAD ) + 1 
         
         IF( IX_PAD .LE. 0 .OR. IX_PAD .GT. NX ) GO TO 100 
         IF( IY_PAD .LE. 0 .OR. IY_PAD .GT. NY ) GO TO 100 
         
C     --determine top-layer straw tube intersected by track 
C     (pads mounted upstream)
         DIST_TOP   = AMOD( (XSTART-XPMIN),XPAD )
         IX_STR_TOP = INT( DIST_TOP/XSTRAW ) + 1
         IF (IX_STR_TOP.LT.1 .OR. IX_STR_TOP.GT.NTUB) THEN
            C78='DGPTO warning! MIMP top straw'//
     &           'tube number out of bounds'
            CALL MEMESC(922345,1,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100  
	 ENDIF
         IF (IX_STR_TOP.EQ.1) THEN
            IX_STR_TOP = 13
         ELSEIF (IX_STR_TOP.EQ.NTUB)  THEN
            IX_STR_TOP = 17
         ELSE
            IX_STR_TOP = 15
         ENDIF
         
C     --determine bottom-layer straw tube intersected by track 
         DIST_BOT   = AMOD( (XEND-XPMIN),XPAD )
         IX_STR_BOT = INT( DIST_BOT/XSTRAW ) + 1
         IF (IX_STR_BOT.LT.1 .OR. IX_STR_BOT.GT.NTUB) THEN
            C78='DGPTO warning! MIMP bott straw'//
     &           'tube number out of bounds'
            CALL MEMESC(922345,1,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100  
	 ENDIF
         IF (IX_STR_BOT.EQ.1) THEN
            IX_STR_BOT = 14
         ELSEIF (IX_STR_BOT.EQ.NTUB)  THEN
            IX_STR_BOT = 18
         ELSE
            IX_STR_BOT = 16
         ENDIF
         
C     --get y-sharing factors based on track y position
         YFACT_U = 0.
         YFACT_C = 1.
         YFACT_L = 0.
         IF (LCR_TALK) THEN
            DIST_Y  = AMOD( (YSTART-YPMIN),YPAD)
            IF (DIST_Y.LT.1.0) THEN       
              IY      = INT(DIST_Y * 10) + 1
              YFACT_U = YFACTOR_U(IY)
              YFACT_C = YFACTOR_C(IY)
              YFACT_L = YFACTOR_L(IY)
            ELSEIF ((YPAD-DIST_Y).LT.1.0) THEN       
              IY      = INT((YPAD-DIST_Y) * 10) + 1
              YFACT_U = YFACTOR_L(IY)
              YFACT_C = YFACTOR_C(IY)
              YFACT_L = YFACTOR_U(IY)
            ENDIF
	 ENDIF
         
C     --sample charge 
         CALL DGPTQ( QQ , 2 )
         QQ1 = QQ( 1 ) 
         QQ2 = QQ( 2 ) 
         
C     --divide by factor 0.52 to account for 0.5 us ORTEC integration time
         QQ1 = QQ1/0.52
         QQ2 = QQ2/0.52
         
C     --rescale to account for running at higher gain (1350 V vs 1300 V)
         QQ1 = QQ1 * 82794./44463.
         QQ2 = QQ2 * 82794./44463.
         
C     --signal spread in direction perpendicular to straw tubes
         FACTOR1( -1 ) = 0.
         FACTOR1(  0 ) = 1.
         FACTOR1( +1 ) = 0.
         
         FACTOR2( -1 ) = 0.
         FACTOR2(  0 ) = 0.2437
         FACTOR2( +1 ) = 0.
         
         IF (LCR_TALK) THEN
            IF (IX_STR_TOP.EQ.13) THEN
               IF (IX_STR_BOT.EQ.14) THEN
                  FACTOR1( -1 ) = 0.331660 
                  FACTOR1(  0 ) = 0.947600 
                  FACTOR1( +1 ) = 0.331660
                  
                  FACTOR2( -1 ) = 0.011360
                  FACTOR2(  0 ) = 0.214200
                  FACTOR2( +1 ) = 0.000910
               ENDIF
            ENDIF
            
            IF (IX_STR_TOP.EQ.15) THEN
               IF (IX_STR_BOT.EQ.14) THEN
                  FACTOR1( -1 ) = 0.355355
                  FACTOR1(  0 ) = 1.015300
                  FACTOR1( +1 ) = 0.355355
                  
                  FACTOR2( -1 ) = 0.011360
                  FACTOR2(  0 ) = 0.214200   
                  FACTOR2( +1 ) = 0.000910
               ELSEIF (IX_STR_BOT.EQ.16) THEN
                  FACTOR1( -1 ) = 0.355355
                  FACTOR1(  0 ) = 1.015300
                  FACTOR1( +1 ) = 0.355355
                  
                  FACTOR2( -1 ) = 0.002000
                  FACTOR2(  0 ) = 0.243700 
                  FACTOR2( +1 ) = 0.002000
               ENDIF
            ENDIF
            
            IF (IX_STR_TOP.EQ.17) THEN
               IF (IX_STR_BOT.EQ.16) THEN
                  FACTOR1( -1 ) = 0.331660
                  FACTOR1(  0 ) = 0.947600
                  FACTOR1( +1 ) = 0.331660
                  
                  FACTOR2( -1 ) = 0.002000
                  FACTOR2(  0 ) = 0.243700   
                  FACTOR2( +1 ) = 0.002000
               ELSEIF (IX_STR_BOT.EQ.18) THEN
                  FACTOR1( -1 ) = 0.331660
                  FACTOR1(  0 ) = 0.947600
                  FACTOR1( +1 ) = 0.331660
                  
                  FACTOR2( -1 ) = 0.000910 
                  FACTOR2(  0 ) = 0.214200
                  FACTOR2( +1 ) = 0.011360
               ENDIF
            ENDIF
	 ENDIF
         
C     --fill temorary digits Arte table for central pads in y
         DO I = -1, 1
            IF ((IX_PAD + i).GT.0 .AND. (IX_PAD + i).LE.NX) THEN
               iCELL = 1000000 * ( IX_PAD + i )  + 
     &              10000 * IY_PAD + iPL
               QDEP(I) = (FACTOR1(I) * QQ1 +  
     &              FACTOR2(I) * QQ2) * YFACT_C
               CALL MMBOOK( JTEMP , 1 ) 
               IQ(IQTMPD_ID_L   ()) = iCELL   
                Q( QTMPD_Q_L    ()) = QDEP(I)
               IQ(IQTMPD_MIMP_L ()) = iMIM
               IQ(IQTMPD_USED_L ()) = 0 
            ENDIF
         ENDDO
         
C     --fill DIGB zebra bank for upper pads in y
         IF ((IY_PAD + 1).LE.NY) THEN
            DO I = -1, 1
               IF ((IX_PAD + i).GT.0 .AND. (IX_PAD + i).LE.NX) THEN
                  
                  iCELL = 1000000 * (IX_PAD + i) 
     &                 + 10000 * (IY_PAD + 1) + iPL
                  QDEP(I) = (FACTOR1(I) * QQ1 
     &                 +  FACTOR2(I) * QQ2) * YFACT_U
                  CALL MMBOOK( JTEMP , 1 ) 
                  IQ(IQTMPD_ID_L   ()) = iCELL   
                   Q( QTMPD_Q_L    ()) = QDEP(I)
                  IQ(IQTMPD_MIMP_L ()) = iMIM
                  IQ(IQTMPD_USED_L ()) = 0 
               ENDIF
            ENDDO
         ENDIF
         
C     --fill DIGB zebra bank for lower pads in y
         IF ((IY_PAD - 1).GT.0) THEN
            DO I = -1, 1
               IF ((IX_PAD + i).GT.0 .AND. (IX_PAD + i).LE.NX) THEN
                  iCELL = 1000000 * (IX_PAD + i) 
     &                 + 10000 * (IY_PAD - 1) + iPL
                  QDEP(I) = (FACTOR1(I) * QQ1 +  
     &                 FACTOR2(I) * QQ2) * YFACT_L
                  CALL MMBOOK( JTEMP , 1 ) 
                  IQ(IQTMPD_ID_L   ()) = iCELL   
                   Q( QTMPD_Q_L    ()) = QDEP(I)
                  IQ(IQTMPD_MIMP_L ()) = iMIM
                  IQ(IQTMPD_USED_L ()) = 0 
               ENDIF
            ENDDO
         ENDIF
         
         
 100  CONTINUE                  ! Go to next MIMP 
      
      RETURN 
      END
****************************************************************************
*                    end of routine(function) dgpto                        * 
****************************************************************************
*                                                                          *
*            EEEEEEEEEEEE  NN        NN  DDDDDDDDD                         *
*            EEEEEEEEEEEE  NNN       NN  DDDDDDDDDD                        *
*            EE            NNNN      NN  DD       DD                       *
*            EE            NN NN     NN  DD        DD                      *
*            EE            NN  NN    NN  DD        DD                      *
*            EEEEEEEE      NN   NN   NN  DD        DD                      *
*            EEEEEEEE      NN    NN  NN  DD        DD                      *
*            EE            NN     NN NN  DD        DD                      *
*            EE            NN      NNNN  DD        DD                      *
*            EE            NN       NNN  DD       DD                       *  
*            EEEEEEEEEEEE  NN        NN  DDDDDDDDDD                        *   
*            EEEEEEEEEEEE  NN         N  DDDDDDDDD                         *
*                                                                          *
****************************************************************************
*                    end of routine(function) dgpto                        * 
****************************************************************************

