 
// *****************************************************************************
//
// source:    DQ_FARM_rec_evnt.C
//
// type:      source code
//
// created:   17 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ_event()
//
// hints:
//            components: COMP_SVD
//                        COMP_ITR
//                        COMP_HIPT
//                        COMP_OTR
//                        COMP_RICH
//                        COMP_TRD
//                        COMP_ECAL
//                        COMP_MUON
//                        COMP_FLT
//                        COMP_SLT
//                        COMP_TLT
//                        COMP_FARM
//
//            levels:     LVL_RAW
//                        LVL_HIT
//                        LVL_REC
//
// changes:
// 17 Nov 1999 (AG) creation
// 13 Mar 2000 (AG) monitor RTRA, RSEG, RCCL, RVER
// 08 May 2000 (AG) event classification flag
// 12 May 2000 (AG) histogram 0 if no class bit set
// 18 May 2000 (AG) timings of reco modules
// 22 May 2000 (AG) comment 2-d histograms; polish printout
// 24 May 2000 (AG) histogram bytes
// 26 May 2000 (AG) histogram time_rcevnt here
//
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte includes
#include "arte/arte.hh"
#include "dq/DQ.h"
#include "dq/DQ_FARM.h"

// externals
extern int FATIME; // flag for printout of timing

// time from rcevnt
extern float time_rcevnt;

// times from rc99AStandAlone()
extern float time_rc99AVDS;
extern float time_rc99AITR;
extern float time_rc99AHPT;
extern float time_rc99AOTR;
extern float time_rc99ARICH;
extern float time_rc99ATRD;
extern float time_rc99AECAL;
extern float time_rc99AMUON;

// times from rc99Aevnt()
extern float time_rc99AStandAlone;
extern float time_rc99Match;
extern float time_rc99SelectTracks;
extern float time_rc99Arefit;
extern float time_rcSetTrackPoints;
extern float time_rc99AParticleID;
extern float time_rc99Avertex;
extern float time_rcSelectDecays;
extern float time_rcClassify;

// times from rc99Arefit()
extern float time_rcrefitOTR;

// times from rc99AParticleID()
extern float time_rc99AidRICH;
extern float time_rc99AidECAL;

//
// function code
//
int
DQ_FARM_rec_evnt()
{
  // DST flag
  ArtePointer<EVHD> evhd( 1 );
  int flagdst = evhd->flagdst;

  // l4t event classification
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 1 ), 0, 1 );
  for( int bit=0; bit<32; bit++ ){
    bool j = ( flagdst >> bit ) & 1;
    if( j ) rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 1 ), bit + 1, 1 );
  }

  // reco tables
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  10 ), ArteTable<RTRA>::size(), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  11 ), ArteTable<RSEG>::size(), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  12 ), ArteTable<RCCL>::size(), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  13 ), ArteTable<RVER>::size(), 1 );

  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  20 ), ArteTable<RTRA>::size() * sizeof( RTRA ), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  21 ), ArteTable<RSEG>::size() * sizeof( RSEG ), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  22 ), ArteTable<RCCL>::size() * sizeof( RCCL ), 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC,  23 ), ArteTable<RVER>::size() * sizeof( RVER ), 1 );

  // reco standalone modules
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 100 ), time_rcevnt,    1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 101 ), time_rc99AVDS,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 102 ), time_rc99AITR,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 103 ), time_rc99AHPT,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 104 ), time_rc99AOTR,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 105 ), time_rc99ARICH, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 106 ), time_rc99ATRD,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 107 ), time_rc99AECAL, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 108 ), time_rc99AMUON, 1 );

  // correlations
  //rhp_hfill( DQ_ID( COMP_FARM, LVL_REC, 1101 ), float( ArteTable<DSVD>::size() ), time_rc99AVDS, 1 );
  //rhp_hfill( DQ_ID( COMP_FARM, LVL_REC, 1102 ), float( ArteTable<DITR>::size() ), time_rc99AITR, 1 );
  //rhp_hfill( DQ_ID( COMP_FARM, LVL_REC, 1104 ), float( ArteTable<DOTR>::size() ), time_rc99AOTR, 1 );
  
  // reco modules
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 111 ), time_rc99AStandAlone,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 112 ), time_rc99Match,        1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 113 ), time_rc99SelectTracks, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 114 ), time_rc99Arefit,       1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 115 ), time_rcSetTrackPoints, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 116 ), time_rc99AParticleID,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 117 ), time_rc99Avertex,      1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 118 ), time_rcSelectDecays,   1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 119 ), time_rcClassify,       1 );

  // refit modules
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 204 ), time_rcrefitOTR, 1 );

  // Particle ID modules
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 305 ), time_rc99AidRICH,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_REC, 307 ), time_rc99AidECAL,  1 );

  // reco standalone modules
  if( FATIME ){
    cout << "TIME: Reconstruction=" << time_rcevnt           << " sec" << endl;
    cout << "      StandAlone=    " << time_rc99AStandAlone  << " sec" << endl;
    cout << "             VDS=    " << time_rc99AVDS         << " sec" << endl;
    cout << "             ITR=    " << time_rc99AITR         << " sec" << endl;
    cout << "             HPT=    " << time_rc99AHPT         << " sec" << endl;
    cout << "             OTR=    " << time_rc99AOTR         << " sec" << endl;
    cout << "            RICH=    " << time_rc99ARICH        << " sec" << endl;
    cout << "             TRD=    " << time_rc99ATRD         << " sec" << endl;
    cout << "            ECAL=    " << time_rc99AECAL        << " sec" << endl;
    cout << "            MUON=    " << time_rc99AMUON        << " sec" << endl;
    cout << "      Match=         " << time_rc99Match        << " sec" << endl;
    cout << "      SelectTracks=  " << time_rc99SelectTracks << " sec" << endl;
    cout << "      Refit=         " << time_rc99Arefit       << " sec" << endl;
    cout << "        OTR=         " << time_rcrefitOTR       << " sec" << endl;
    cout << "      SetTrackPoints=" << time_rcSetTrackPoints << " sec" << endl;
    cout << "      ParticleID=    " << time_rc99AParticleID  << " sec" << endl;
    cout << "            RICH=    " << time_rc99AidRICH      << " sec" << endl;
    cout << "            ECAL=    " << time_rc99AidECAL      << " sec" << endl;
    cout << "      Vertex=        " << time_rc99Avertex      << " sec" << endl;
    cout << "      SelectDecays=  " << time_rcSelectDecays   << " sec" << endl;
    cout << "      Classify=      " << time_rcClassify       << " sec" << endl;
  }

  return 0;
}
