 
// *****************************************************************************
//
// source:    DQ_FARM_rec_init.C
//
// type:      source code
//
// created:   17 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ_init()
//
// hints:
//            components: COMP_SVD
//                        COMP_ITR
//                        COMP_HIPT
//                        COMP_OTR
//                        COMP_RICH
//                        COMP_TRD
//                        COMP_ECAL
//                        COMP_MUON
//                        COMP_FLT
//                        COMP_SLT
//                        COMP_TLT
//                        COMP_FARM
//
//            levels:     LVL_RAW
//                        LVL_HIT
//                        LVL_REC
//
// changes:
// 17 Nov 1999 (AG) creation
// 13 Mar 2000 (AG) monitor RTRA, RSEG, RCCL, RVER
// 08 May 2000 (AG) event classification flag
// 12 May 2000 (AG) increase scale for ECLASS by 1
// 17 May 2000 (AG) add timing histograms for reco modules
// 22 May 2000 (AG) comment 2-d histograms
// 24 May 2000 (AG) histogram bytes
// 26 May 2000 (AG) histogram time_rcevnt here
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte includes
#include "arte/arte.hh"
#include "dq/DQ.h"
#include "dq/DQ_FARM.h"

//
// function code
//
int
DQ_FARM_rec_init()
{
  cout << "DQ_FARM_rec_init()" << endl;

  // 4lt trigger
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 1 ), "FARM: ECLASS [bits]", 33, -0.5, 32.5 );

  // reco tables
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 10 ), "FARM: RTRA [rows]", 100, 0, 1000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 11 ), "FARM: RSEG [rows]", 100, 0, 1000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 12 ), "FARM: RCCL [rows]", 100, 0, 1000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 13 ), "FARM: RVER [rows]", 100, 0,  100 );

  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 20 ), "FARM: RTRA [B]", 100, 0, 100000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 21 ), "FARM: RSEG [B]", 100, 0, 500000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 22 ), "FARM: RCCL [B]", 100, 0, 100000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 23 ), "FARM: RVER [B]", 100, 0,   2000 );

  // reco standalone timings
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 100 ), "FARM: RECO [sec]", 100, 0, 20.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 101 ), "FARM: VDS [sec]",  100, 0,  1.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 102 ), "FARM: ITR [sec]",  100, 0,  2.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 103 ), "FARM: HPT [sec]",  100, 0,  1.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 104 ), "FARM: OTR [sec]",  100, 0, 20.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 105 ), "FARM: RICH [sec]", 100, 0,  5.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 106 ), "FARM: TRD [sec]",  100, 0,  1.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 107 ), "FARM: ECAL [sec]", 100, 0,  2.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 108 ), "FARM: MUON [sec]", 100, 0,  2.0 );

  // correlations
  //rhp_hbook2( DQ_ID( COMP_FARM, LVL_REC, 1101 ), "FARM: SVD vs. DSVD", 100, 0, 20000, 100, 0,  1.0 );
  //rhp_hbook2( DQ_ID( COMP_FARM, LVL_REC, 1102 ), "FARM: ITR vs. DITR", 100, 0, 50000, 100, 0,  2.0 );
  //rhp_hbook2( DQ_ID( COMP_FARM, LVL_REC, 1104 ), "FARM: OTR vs. DOTR", 100, 0, 50000, 100, 0, 20.0 );

  // general reco module timings
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 111 ), "FARM: Standalone [sec]",     100, 0, 20.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 112 ), "FARM: Match [sec]",          100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 113 ), "FARM: SelectTracks [sec]",   100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 114 ), "FARM: Refit [sec]",          100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 115 ), "FARM: SetTrackPoints [sec]", 100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 116 ), "FARM: ParticleID [sec]",     100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 117 ), "FARM: Vertex [sec]",         100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 118 ), "FARM: SelectDecays [sec]",   100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 119 ), "FARM: Classify [sec]",       100, 0, 10.0 );

  // refit timings
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 204 ), "FARM: Refit OTR [sec]",  100, 0, 10.0 );

  // ParticleID timings
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 305 ), "FARM: ParticleID RICH [sec]",  100, 0, 10.0 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 307 ), "FARM: ParticleID ECAL [sec]",  100, 0,  1.0 );

  return 0;
}
