
// ******************************************************************
//
// 
// M. Braeuer / D. Samtleben
// Filling of histograms for VDS
//
// changes:
// 28 Oct 1999 (AG)  use defines from DQ.h
// 01 Nov 1999 (AG)  get rid of dq_hid.h
// 03 Nov 1999 (AG)  repair hist id
// 10 Dez 1999 (MBr) update/major revision
// 19 Apr 2000 (AG)  polish
// 16 May 2000 (GS)  vertexing added
// 25 May 2000 (GS)  bug fix in filling of residuals histograms
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte include
#include "arte/arte.hh"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"

// from Alignment stuff:
inline int fUnsin(int sl,int sec,int lay) { 
  int i = (lay-1)+(4*(sec-1))+(16*(sl-1));
  return(i); 
}

extern int RoverHidden;
extern int RoverNver;
extern int rver_per_wnr[10];
extern int ntr_per_rver[10];

extern "C"{
  void prima_love(void);
}

int DQ_SVD_vertex_evnt();

//
// function code
//
int
DQ_SVD_rec_evnt()
{
  //cout << "DQ_SVD_rec_evnt()" << endl;
  
  int nrSVDtracks=0;
  int nrVDStrackhits=0; // no of track associated hits/event
  
  // loop over all tracks, track=pointer to RSEG information
  for(ArteTableIterator<RSEG> track;track.valid();track.next()){
    float tx=track->txf;  // x-slope at first hit (smallest z)
    float ty=track->tyf;
    
    /* VDS Modified !!================================== */
    // VDS segments only 65537=CATS  1=HOLMES
    if(track->cmp ==1||track->cmp==65537){ 
      int VDShitstontrack = 0;
      float VDSxp1,VDSyp1,VDSxp2,VDSyp2,VDSres, VDSocc, VDSusid;
      VDSxp1 = VDSyp1 = VDSxp2 = VDSyp2 = VDSres = VDSocc = VDSusid = 0;
      float VDSx = track->xf;
      float VDSy = track->yf;
            
      // loop over the hits of track      
      for ( ArteRelationIterator<HITB> hitbIt = track->hitb.begin();
	    hitbIt != track->hitb.end(); ++hitbIt ) {
	ArtePointer<HITB> hitb(hitbIt);     // hitb = pointer to HITB 
	ArtePointer<GEDE> gede(hitb->gede); // gede = pointer to GEDE
	
	// calculate the plane ID:
	VDSusid = float(fUnsin(gede->gesl->nsl,gede->sec,gede->lay) ); 

	// increase number of hits per track:
	++VDShitstontrack;
	++nrVDStrackhits;
	
	// book residuals: VDSxp1 = extrapolated track position at 
	// the plane of hit, VDSxp2 taking rotation from alignment into account
	VDSxp1 = VDSx + tx*( (gede->zmax+gede->zmin)/2 - track->zf ); 
	VDSyp1 = VDSy + ty*( (gede->zmax+gede->zmin)/2 - track->zf );
	VDSxp2 = VDSxp1*gede->rot[0] + VDSyp1*gede->rot[1];
	VDSyp2 = VDSxp1*gede->rot[3] + VDSyp1*gede->rot[4];
	VDSres = 1000 * (hitb->x - ((VDSxp2*gede->cos) - (VDSyp2*gede->sin) ));

// printf(">>>>>>>> VDSres:%f\n",VDSres);
//cout << "VDSres" << VDSres <<endl;

	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,204),VDSres,1); // to total residual

// printf(">>>>>>>> VDSusid:%f\n", (float) VDSusid);
// printf(">>>>>>>> VDSres:%f\n",VDSres);
//cout << "VDSusid" << VDSusid <<endl;
//cout << "VDSres" << VDSres <<endl;

	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,203),VDSusid,VDSres,1); // to residual
	
	// the occupancy:
	VDSocc = (hitb->x - gede->xfw[0]) / gede->pit;

// printf(">>>>>>>> VDSusid:%f\n", (float) VDSusid);
//cout << "VDSusid" << VDSusid <<endl;

	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,202),VDSusid,VDSocc,1); // to residual
	
      } // end of loop
      VDSxp1 = VDSx + (tx * (100 - track->zf));
      VDSyp1 = VDSy + (ty * (100 - track->zf));
      rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,205),10*VDSxp1,10*VDSyp1,1); // to refplane
      rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,201),VDSusid,1);
      nrSVDtracks++;
    } // if VDS-track (rseg->cmp==1)
    
    /* End of VDS modified ============================= */
  }
  rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,200),nrSVDtracks,1);
  if ( nrVDStrackhits > 0 )
	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,206),nrVDStrackhits,1);
  
  /* Do the Vertexing stuff ============================ */
  DQ_SVD_vertex_evnt();

  return 0; // return error code (0 = okay)
}

int
DQ_SVD_vertex_evnt()
{ 

  RoverHidden = 1;
  
  prima_love();

  int nwires = 0;   
  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,300), float(RoverNver), 0.0, 1.0);
  //cout<<"DQ rover => "<<RoverNver<<" : ";
  for(int itar=1;itar<=8;itar++){
    rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,300+itar), float(rver_per_wnr[itar]), 0.0, 1.0);
    //cout<<rver_per_wnr[itar]<<" ";
    if(rver_per_wnr[itar] != 0){
      nwires++;   
    }
  }
  //cout << "Total wires used: " <<nwires<<endl;
  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,310),float(RoverNver),nwires,1.0);
  //cout<<endl;

  //cout<<"Tracks per vertex : ";
  for(int iver=1;iver<=RoverNver;iver++){
    rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,309), float(ntr_per_rver[iver]), 0.0, 1.0);
    //cout<<ntr_per_rver[iver]<<" ";
  }
  //cout<<endl;

  RoverHidden = 0;

  return 0; // return error code (0 = okay)
}






