/***********************************************************************
*
* CRMIBK
* written by Siegmund Nowak, 27/10/94 13.00.07
*
* This routines transforms the temporary gean banks for MIMPs, MTCPs, MCALs
* into the ARTE MIMP/MTCP/MCAL  banks according to increasing
* detector numbers.
*
* called from GUOUT
***********************************************************************/
#include <vector.h>
#include <stdio.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/ZGMOTH.h"
#include "gean/zgkine.hh"
#include "gean/zgmimp.hh"

extern "C" {
      void crmibk_();
      void sortzv_(int[], int[], int*, int*, int*, int*); 
      void mmirel_(ArteLink *);}

void crmibk_(){     
      extern vector<hgmtra> *hg_mtra;
      extern vector<hgmimp> *normal_mimp;
      extern vector<hgmimp> *modul_mimp;
      extern vector<hgmimp> *TRD_mimp;
      extern vector<hgmcal> *TR_mimp;
      extern vector<hgmimp> *RICH_mimp;
      extern vector<hgmtcp> *ECA_ep;
      extern vector<hgmcal> *ECA_mimp;

      MIMP mimp;
      vector<hgmtra>::iterator mt;
      vector<hgmimp>::iterator mm;
      vector<hgmtcp>::iterator mp;
      vector<hgmcal>::iterator me, mee; 
      int m1 = -1; int i0 = 0;

//    old fashioned mimps
      vector<hgmimp>::size_type np = normal_mimp->size();
      int n = int(np); 
      if(n) {
         int *srt_plane = 0;              int *ind = 0;
         srt_plane = new int[n];          ind = new int[n];
         for(int i = 0; i<n ; i++) {  
             mm = normal_mimp->begin() +i;         srt_plane[i] = mm->gede;} 
         sortzv_(srt_plane,ind,&n,&m1,&i0,&i0);
         for (int i = 0;i< n; i++){
            mm = normal_mimp->begin() + ind[i] -1;
            mt = hg_mtra->begin() +  mm->itra -1;
            mimp.z    = mm->z;      mimp.x    = mm->x;     mimp.y    = mm->y;
            mimp.tx   = mm->tx;     mimp.ty   = mm->ty;    mimp.p    = mm->p;
            mimp.time = mm->time;   mimp.mtra = mt->mtra;  mimp.gede = mm->gede;
            ArteTable<MIMP>::push_back(mimp); }
         delete [] srt_plane;  srt_plane =0;  delete [] ind;   ind = 0; }  

//    modular mimps 
      np = modul_mimp->size();
      n = int(np);
      if(n) {
         int *srt_plane = 0;              int *ind = 0;
         srt_plane = new int[n];          ind = new int[n];
         for (int i =0; i<n; i++) {  
              mm = modul_mimp->begin() + i;     srt_plane[i] = mm->gede; 
         } 
         sortzv_(srt_plane,ind,&n,&m1,&i0,&i0);
         int inumold=0;  int islold=0;  int m=0;  int ifpl=0;  int nipl=0;
         for (int i = 0;i< n; i++){
            mm = (modul_mimp->begin() + ind[i] -1);
            mt = hg_mtra->begin() +  mm->itra -1;
            mimp.z    = mm->z;      mimp.x    = mm->x;     mimp.y    = mm->y;
            mimp.tx   = mm->tx;     mimp.ty   = mm->ty;    mimp.p    = mm->p;
            mimp.time = mm->time;   mimp.mtra = mt->mtra;
            int inum = mm->gede;
            int found;
            if(inum != inumold) {
               int isl = inum >>24;
               inumold = inum;
               if(isl != islold) {
                  ArtePointer<GESL> gesl(isl);  
                  ifpl = gesl->fpl; nipl = gesl->npl;
                  islold = isl; 
               }
               int icopy = inum&4095;  
               int modul = inum >> 12;    modul = modul&4095; 
               found = 0;
               for(m=ifpl;m<ifpl+nipl; m++) { 
                  ArtePointer<GEDE> gede(m); 
                  int gmodul= gede->gcop >>16 ; int gcopy = gede->gcop&65535;
                  if(modul == gmodul && icopy == gcopy) {
                     found = 1;  break;
                  }
               }
               if(!found) { arteMsg << "crmibk:gede not found for mimp "
                           << i << " - contact S.Nowak"; 
                           int id = 65748389; arteMsg.allp(id,3);
               }
            }          
            mimp.gede = m; 
            ArteTable<MIMP>::push_back(mimp); 
        }
        delete [] srt_plane; srt_plane = 0;  delete [] ind; ind = 0; 
      }
   
//    TRD mimps (long version)       
      np = TRD_mimp->size();    int np1 = TR_mimp->size();
      int n0 = int(np);      int n1 =  int(np1);      
      if(n0) {
         int *srt_plane = NULL;            int *ind = NULL;
         srt_plane = new int[n0];          ind = new int[n0]; 
         if(srt_plane == NULL || ind ==NULL) {
	    arteMsg << "crmibk: not enough storage for TRD sorting";
            arteMsg.error();
         }  
         for (int i=0; i<n0 ; i++) {  
           mm = TRD_mimp->begin() + i;       srt_plane[i] = mm->gede&65535;
         }

         int *srt_plane1 = 0;              int *ind1 = 0;
         srt_plane1 = new int[n1];          ind1 = new int[n1];
         if(srt_plane1 == NULL || ind1 ==NULL) {
	    arteMsg << "crmibk: not enough storage for TR sorting";
            arteMsg.error();
         } 
         for (int i=0; i<n1 ; i++) {  
           me = TR_mimp->begin() + i;        srt_plane1[i] = me->gcal&65535;
         } 

         sortzv_(srt_plane,  ind,&n0,&m1,&i0,&i0);
         sortzv_(srt_plane1,ind1,&n1,&m1,&i0,&i0);

         for (int i = 0;i< n0 ; i++){
            mm = TRD_mimp->begin() + ind[i] -1;
            mt = hg_mtra->begin() +  mm->itra -1;
            mimp.z    = mm->z;     mimp.x    = mm->x;     mimp.y    = mm->y;
            mimp.tx   = mm->tx;    mimp.ty   = mm->ty;    mimp.p    = mm->p;
            mimp.time = mm->time;  mimp.mtra = mt->mtra;  mimp.gede = mm->gede;
            int gdlast = mm->gede;
            mimp.q = 0.;
            for(int j=0; j<n1; j++) {
	       me =  TR_mimp->begin()  + ind1[j] -1;
               if(me->gcal == gdlast) {
                  mimp.q = me->de;
                  break; 
	       }
            }

            ArteTable<MIMP>::push_back(mimp); 
         }
         if(srt_plane)  delete [] srt_plane;      srt_plane = NULL;  
         if(ind)        delete [] ind;            ind = NULL;
         if(srt_plane1) delete [] srt_plane1;     srt_plane1 = NULL; 
         if(ind1)       delete [] ind1;           ind1 =NULL;
      }


//    RICH mimps
      np = RICH_mimp->size();
      n = int(np);
      if(n) {
         for (int i = 0;i< n; i++){
           mm = RICH_mimp->begin() + i;
           mt = hg_mtra->begin() +  mm->itra -1;
           mimp.z    = mm->z;    mimp.x    = mm->x;    mimp.y    = mm->y;
           mimp.tx   = mm->tx;   mimp.ty   = mm->ty;   mimp.p    = mm->p;
           mimp.time = mm->time; mimp.mtra = mt->mtra; mimp.gede = mm->gede;
	   mimp.q   = mm->q;
           ArteTable<MIMP>::push_back(mimp); 
         } 
      }
      int nmimp = ArteTable<MIMP>::size();
      if(nmimp) mmirel_(&artelink_.jmtmi);
      else { arteMsg << "no MIMPs "; 
             int id = 65748390; arteMsg.allp(id,3);
      }          

//    calorimeter end points
      MTCP mtcp;
      np = ECA_ep->size();
      n = int(np);
      if(n) {
         for (int i = 0;i< n; i++){
            mp = (ECA_ep->begin() + i);
            mt = hg_mtra->begin() +  mp->itra -1;
            mtcp.z    = mp->z;    mtcp.x  = mp->x;     mtcp.y = mp->y;
            mtcp.tx   = mp->tx;   mtcp.ty = mp->ty;    mtcp.p = mp->p;
            mtcp.mtra = mt->mtra;
            ArteTable<MTCP>::push_back(mtcp); 
         } 
      }
      int nmtcp = ArteTable<MTCP>::size();
      if(nmtcp)   mmirel_(&artelink_.jmtmp);
      else { arteMsg << "no calorimeter endpoints "; 
             int id = 65748391; arteMsg.allp(id,3);
      }          

//    calorimeter and scint hodoscope mcals
      MCAL mcal;
      np = ECA_mimp->size();
      n = int(np);
      if(n) { 
	int *srt_plane = 0;      int *ind = 0;    
	srt_plane = new int[n];  ind = new int[n];
         for (int i =0; i<n; i++) {  
            me = ECA_mimp->begin() + i;        srt_plane[i] = me->gcal; 
         } 
         sortzv_(srt_plane,ind,&n,&m1,&i0,&i0);
         int new_plane =0;    int index_new_plane = 0;
         for (int i = 0;i< n; i++){
	   srt_plane[i] =0;
            me = (ECA_mimp->begin() + ind[i] -1);
            mt = hg_mtra->begin() +  me->itra -1;
            if(me->gcal != new_plane) {
               new_plane = me->gcal;  index_new_plane =i;
            }   
            else {
              int found = 0;
              for (int k = index_new_plane; k<i; k++) {
                 mee = (ECA_mimp->begin() + ind[k] -1);
		 if(me->itra == mee->itra) {
                    ArtePointer<MCAL> gmm(srt_plane[k]);
                    gmm->de = gmm->de + me->de; found =1;  break;
                 }  
              }
              if(found)  continue;             
            }
            mcal.de = me->de;    mcal.gcal = me->gcal;    mcal.mtra = mt->mtra;
            ArteTable<MCAL>::push_back(mcal);
            srt_plane[i] = ArteTable<MCAL>::size(); 
         }
         delete [] srt_plane;  srt_plane = 0;  delete [] ind;  ind = 0; 
      }
      int nmcal = ArteTable<MCAL>::size();
      if (nmcal) mmirel_(&artelink_.jmtmc);
      else { arteMsg << "no calorimeter entries MCAL"; 
             int id = 65748392; arteMsg.allp(id,3);
      }          
}


