*CMZ :  0.01/02 22/08/94  18.23.57  by  Dominik Ressing
*-- Author :    Dominik Ressing   22/08/94
************************************************************************
*
* L1HICRO
* written by Dominik Ressing, 22/08/94 18.04.54
*
* This routines action:
* calculates the cross-section of the regions of the 3 hits given.
* If there is now overlap a -1. as z value for the region is returned.
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* iDetP       in          Detector specifier for +alpha layer
* iP          in          Cell number for +alpha layer
* nP          in          number of set Cells in a row
* iDetZ       in          Detector specifier for axial layer
* iZ          in          Cell number for axial layer
* nZ          in          number of set Cells in a row
* iDetN       in          Detector specifier for -alpha layer
* iN          in          Cell number for -alpha layer
* nN          in          number of set Cells in a row
* Geom        out         Geometry of cross section (type of GeoInf)
*
************************************************************************
c
      SUBROUTINE L1HICRO(iDetP,iP,nP,iDetZ,iZ,nZ,iDetN,iN,nN,Geom)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      INTEGER iDetP,iP,nP,iDetZ,iZ,nZ,iDetN,iN,nN
      REAL    Geom(L1GEOINFlen),Scr(L1GEOINFlen),Scr1(L1GEOINFlen)
c
      GEOM(L1GEOINFz) = -1.
c
      IF ( iP.eq.0 .or. iZ.eq.0 .or. iN.eq.0 .or.
     &     nP.eq.0 .or. nZ.eq.0 .or. nN.eq.0 ) RETURN
c
      CALL l1hitif(iDetP,iP,Geom)                 ! +alpha geometry into Geom
      IF ( nP.gt.1 ) THEN                         ! or neighbours
         CALL l1hitif(iDetP,iP+nP-1,Scr)
         Geom(L1GEOINFsx) = Geom(L1GEOINFsx) +
     &        Scr(L1GEOINFx0) - Geom(L1GEOINFx0)
      END IF
c
      CALL l1hitif(iDetN,iN,Scr1)                 ! -alpha geometry into Scr1
      IF ( nN.gt.1 ) THEN                         ! or neighbours
         CALL l1hitif(iDetN,iN+nN-1,Scr)
         Scr1(L1GEOINFsx) = Scr1(L1GEOINFsx) +
     &        Scr(L1GEOINFx0) - Scr1(L1GEOINFx0)
      END IF
      CALL l1detis(Geom,Scr1,Geom)                ! intersection of +alpha and -alpha
      IF ( GEOM(L1GEOINFz).lt.0. ) RETURN
c
      CALL l1hitif(iDetZ,iN,Scr1)                 ! axial geometry in Scr1
      IF ( nZ.gt.1 ) THEN                         ! or neighbours
         CALL l1hitif(iDetZ,iZ+nZ-1,Scr)
         Scr1(L1GEOINFsx) = Scr1(L1GEOINFsx) +
     &        Scr(L1GEOINFx0) - Scr1(L1GEOINFx0)
      END IF
      CALL l1detis(Geom,Scr1,Geom)                ! Intersection of all hits
c
      RETURN
c
      END
