
// *****************************************************************************
//
// source:    l4tEvnt.C
//
// type:      source code
//
// created:   14 Jan 1998
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    l4tEvnt
//
// changes:
// 25 Nov 1999 (AG) move to l4tEvnt()
// 22 Feb 2000 (AG) handle various settings
// 13 Mar 2000 (AG) remove some printout
// 04 Apr 2000 (AG) return trigger result
// 27 Apr 2000 (AG) use ARTE_mark()
// 05 May 2000 (AG) set EDIR flag accoring to flagdst
// 08 May 2000 (AG) only flagdst needs to be set
// 16 May 2000 (AG) set flag = flagdst
// 24 May 2000 (AG) set classification bit for random triggers (8 + 9)
// 25 May 2000 (AG) set classification bit for FLT trigger (0 + 8)
// 29 May 2000 (AG) comment some printout
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// Arte includes
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "farm/farm.hh"
#include "farm/eclass.hh"
#include "l4trig/l4t.hh"

// externals
extern int FATRG;

//
// program code
//
int
l4tEvnt()
{
  // mark
  ARTE_mark( "l4tEvnt", 0 );

  // EVHD table and trigger code
  ArtePointer<EVHD> evhd( 1 );
  int tcode = faDDAQFCS_tcode();
  cout << "l4tEvnt(): tcode=" << tcode << endl;

  // RNDTRG
  if( ( tcode == 8 ) ||
      ( tcode == 9 )    ) evhd->flagdst |= ECLASS_RNDTRG; // bit-wise AND

  // FLTTRG
  if( ( tcode == 0 ) ||
      ( tcode == 8 )    ) evhd->flagdst |= ECLASS_FLTTRG; // bit-wise AND
  
  // set flag
  evhd->flag = evhd->flagdst;  // NEEDED!
  //cout << "l4tEvnt(): flagdst=0x" << hex << evhd->flagdst << dec << endl;

  // counters
  static int nAll = 0;
  static int nDST = 0;
  static int nOUT = 0;

  // output flag
  //cout << "l4tEvnt(): FATRG=" << FATRG << endl;
  arteutil_.eventout = 0;
  switch( FATRG ){
  case L4_OFF:
    arteutil_.eventout = 1;
    break;
  case L4_ON:
    if( evhd->flagdst ) arteutil_.eventout = 1;
    break;
  case L4_KEEP:
    arteutil_.eventout = 1;
    break;
  case L4_REJECT:
    arteutil_.eventout = 0;
    break;
  default:
    arteutil_.eventout = 1;
  }

  // print
  cout << " TRIGGER: "
       << "TRG="            << FATRG                     << " "
       << "CLASS=0x" << hex << evhd->flagdst      << dec << " "
       << "OUT="            << arteutil_.eventout        << endl;

  // statistics
  nAll++;
  if( evhd->flagdst )      nDST++;
  if( arteutil_.eventout ) nOUT++;
  cout << " TRIGGER: Summary: "
       << "EVT="   << nAll << " "
       << "CLASS=" << nDST << " "
       << "OUT="   << nOUT << endl;

  // return
  return arteutil_.eventout; // return flag to write event
}
