/* see interactive.hh for details */

// Author V.Balagura (balagura@mail.desy.de)   (10.04.98).

#include "ptset/interactive.hh"

void InteractiveCommand::operator()(list<string>& args, istream* is) {
  if ( !args.empty() ) {
    string s = "wrong command: " + *(args.begin());
    if ( args.size() > 1 ) s += " ...";
    global_error()(s);
  }
  else operator()(is);
}

void InteractiveMenu::operator()(list<string>& args, istream* is) {
  if ( args.empty() ) {
    global_error()("end of input where command was expected");
    return;
  }
  iterator iter_found;
  if ( (iter_found=find(args.front())) == end() ) {
    iterator iter_match;
    int n_matches = 0;
    for ( iter_match=begin(); iter_match!=end(); ++iter_match ) {
      if ( (*iter_match).first.compare(args.front(), 0,
				       args.front().size()) == 0 ) {
	if ( n_matches > 0 ) {
	  global_error()("ambiguous command: " + args.front());
	  return;
	}
	iter_found = iter_match;
	++n_matches;
      }
    }
    if ( n_matches == 0 ) {
      global_error()("unrecognized command: " + args.front());
      return;
    }
  }
  InteractiveItem& ii = *((*iter_found).second)();
  args.pop_front();
  ii(args, is);
  if ( global_error() ) global_error() << (*iter_found).first;
}
PtrWrapper<InteractiveItem>& InteractiveMenu::operator()(const string& s) {
  return 
    map<string, PtrWrapper<InteractiveItem>, less<string> >::operator[](s);
}
InteractiveMenu& InteractiveMenu::operator[](const string& s) {
  PtrWrapper<InteractiveItem>& pi = (*this)(s);
  if ( pi() == 0 ) pi = new InteractiveMenu();
  else if ( ! pi()->is_menu() ) {
    global_error()("error in program: InteractiveItem corresponding to "
		   "the key \"" +
		   s + "\"\n is not an InteractiveMenu, but it is used in"
		   "\nexpression (some_interactive_menu)[\"" + 
		   s + "\"]\n"
		   "which returns InteractiveMenu& of this item.\n"
		   "   Look at the codes in your program.");
    return *((InteractiveMenu*)NULL);
  }
  return *((InteractiveMenu*)pi());
}



