// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include "arte/arte.hh"
#include "rang/Node.hh"
#include "rang/Hit.hh"
#include "rang/HitPad.hh"
#include "rang/Geom.hh"
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
#include <math.h>

//-- Constructor
rangerHitPad::rangerHitPad() {};
rangerHitPad::rangerHitPad(ArtePointer<RHIT> rhit,rangerFitMode fm) 
: rangerHit(rhit,fm) {
  m = new HepVector(2);
  (*m)(1) = rhit->hitb->x;
  (*m)(2) = rhit->hitb->y;
  V = new HepSymMatrix(2);
  ArtePointer<GEDE> gede = getHitb()->gede;
  if (gede->nwi[0]>0 && gede->nwi[1]>0);
  else {
    cout << "rangerHitPad::rangerHitPad(ArtePointer<RHIT>,rangerFitMode):"
	 << " gede->nwi[0]=" << gede->nwi[0]
	 << " gede->nwi[1]=" << gede->nwi[1] << endl;
  }
  float dx = 0.2887 * (gede->sxmx - gede->sxmn) / gede->nwi[0];
  float dy = 0.2887 * (gede->symx - gede->symn) / gede->nwi[1];
//   cout << "rangerHitPad::rangerHitPad(ArtePointer<RHIT>,rangerFitMode):"
// 	 << " dx=" << dx
// 	 << " dy=" << dy << endl;
  (*V)(1,1) = dx * dx;
  (*V)(1,2) = 0;
  (*V)(2,1) = 0;
  (*V)(2,2) = dy * dy;
//  cout << "This is the rangerHitPad constructor" << endl;
};

rangerHitPad::rangerHitPad(ArtePointer<HITB> hb,rangerFitMode fm) 
: rangerHit(hb,fm) {
  m = new HepVector(2);
  (*m)(1) = hb->x;
  (*m)(2) = hb->y;
  V = new HepSymMatrix(2);
  ArtePointer<GEDE> gede = getHitb()->gede;
  if (gede->nwi[0]>0 && gede->nwi[1]>0);
  else {
    cout << "rangerHitPad::rangerHitPad(ArtePointer<RHIT>,rangerFitMode):"
	 << " gede->nwi[0]=" << gede->nwi[0]
	 << " gede->nwi[1]=" << gede->nwi[1] << endl;
  }
  float dx = 0.2887 * (gede->sxmx - gede->sxmn) / gede->nwi[0];
  float dy = 0.2887 * (gede->symx - gede->symn) / gede->nwi[1];
//   cout << "rangerHitPad::rangerHitPad(ArtePointer<RHIT>,rangerFitMode):"
// 	 << " dx=" << dx
// 	 << " dy=" << dy << endl;
  (*V)(1,1) = dx * dx;
  (*V)(1,2) = 0;
  (*V)(2,1) = 0;
  (*V)(2,2) = dy * dy;
//  cout << "This is the rangerHitPad constructor" << endl;
};

rangerHitPad::~rangerHitPad() {
  delete m;
  delete V;
//  cout << "The HitPad destructor deletes m and V" << endl;
}

void rangerHitPad::projectP(HepVector& resid,
			      HepMatrix& HH,rangerStateP* s) {

#ifdef FULL
  cout << "This is rangerHitPad::projectP" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                      yhitb=" << getHitb()->y << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  resid(1) = (*m)(1) - (s->x() * cosTot - s->y() * sinTot);
  resid(2) = (*m)(2) - (s->x() * sinTot + s->y() * cosTot);

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) =   sinTot;
  HH(2,2) =   cosTot;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;

}
  
void rangerHitPad::projectPI(HepVector& resid,
			       HepMatrix& HH,rangerStatePI* s) {

#ifdef FULL
  cout << "This is rangerHitPad::projectPI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                      xhitb=" << getHitb()->y << endl;
  cout << "                       gede=" << gede << endl;
#endif

  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  resid(1) = (*m)(1) - (s->x() * cosTot - s->y() * sinTot);
  resid(2) = (*m)(2) - (s->x() * sinTot + s->y() * cosTot);

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;
  HH(1,6) = 0;

  HH(2,1) =   sinTot;
  HH(2,2) =   cosTot;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;
  HH(2,6) = 0;

}


  
void rangerHitPad::projectL(HepVector& resid,
			      HepMatrix& HH,rangerStateL* s) {

#ifdef FULL
  cout << "This is rangerHitPad::projectL" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  resid(1) = (*m)(1) - (s->x() * cosTot - s->y() * sinTot);
  resid(2) = (*m)(2) - (s->x() * sinTot + s->y() * cosTot);

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;

  HH(2,1) =   sinTot;
  HH(2,2) =   cosTot;
  HH(2,3) = 0;
  HH(2,4) = 0;

}
  
void rangerHitPad::projectLI(HepVector& resid,
			       HepMatrix& HH,rangerStateLI* s) {

#ifdef FULL
  cout << "This is rangerHitPad::projectLI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  resid(1) = (*m)(1) - (s->x() * cosTot - s->y() * sinTot);
  resid(2) = (*m)(2) - (s->x() * sinTot + s->y() * cosTot);

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) =   sinTot;
  HH(2,2) =   cosTot;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;

}
  
