// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include "arte/arte.hh"
#include "rang/Node.hh"
#include "rang/Hit.hh"
#include "rang/HitWire.hh"
#include "rang/Geom.hh"
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
//#include "rang/zfconst.hh"
#include <math.h>

//-- Constructor
rangerHitWire::rangerHitWire() {
  LRSign = 1;
};
rangerHitWire::rangerHitWire(ArtePointer<RHIT> rhit,rangerFitMode fm) 
: rangerHit(rhit,fm) {
//                                LRSign = (rhit->hitb->q)>0 ? 1 : -1;      
  LRSign = (rhit->q)>0 ? 1 : -1;
  m = new HepVector(1);
  (*m)(1) = LRSign * fabs(rhit->hitb->q);
  //(*m)(1) = LRSign * (fabs(rhit->hitb->q) - zfconst_.dOff);
  V = new HepSymMatrix(1);
  (*V)(1,1) = (rhit->hitb->r) * (rhit->hitb->r);
//  cout << "This is the rangerHitWire constructor" << endl;
};

rangerHitWire::rangerHitWire(ArtePointer<HITB> hb,rangerFitMode fm) 
: rangerHit(hb,fm) {
  LRSign = 0;
  m = new HepVector(1);
  (*m)(1) = fabs(hb->q);
  //(*m)(1) = LRSign * (fabs(rhit->hitb->q) - zfconst_.dOff);
  V = new HepSymMatrix(1);
  (*V)(1,1) = (hb->r) * (hb->r);
//  cout << "This is the rangerHitWire constructor" << endl;
};

rangerHitWire::~rangerHitWire() {
  delete m;
  delete V;
//  cout << "The HitWire destructor deletes m and V" << endl;
}

void rangerHitWire::projectP(HepVector& resid,
			     HepMatrix& HH,rangerStateP* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectP" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  double Hx = deltaU * cosThetaU;
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitWire::projectP: the stereo angle is " << gede->sin << endl;
  cout << "rangerHitWire::projectP: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;
  HH(1,5) = 0;

}

void rangerHitWire::projectPI(HepVector& resid,
			      HepMatrix& HH,rangerStatePI* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectPI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  // assume that the drift velocity is 0.01 cm/ns
  double Hx = deltaU * cosThetaU - LRSign * 0.01 * s->I();
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitWire::projectPI: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitWire::projectPI: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;
  HH(1,5) = 0;
  // assume that the drift velocity is 0.01 cm/ns
  HH(1,6) = -LRSign * 0.01;
}


void rangerHitWire::projectL(HepVector& resid,
			     HepMatrix& HH,rangerStateL* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectL" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  double Hx = deltaU * cosThetaU;
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitWire::projectL: the stereo angle is " << gede->sin << endl;
  cout << "rangerHitWire::projectL: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;

}


void rangerHitWire::projectLI(HepVector& resid,
			      HepMatrix& HH,rangerStateLI* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectLI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  // assume that the drift velocity is 0.01 cm/ns
  double Hx = deltaU * cosThetaU - LRSign * 0.01 * s->I();
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitWire::projectLI: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitWire::projectLI: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;
  // assume that the drift velocity is 0.01 cm/ns
  HH(1,5) = -LRSign * 0.01;
}


void rangerHitWire::updInternal(){
  if (!r) return;
  if (LRSign) {
    if ( (LRSign * ( (*m)(1) - (*r)(1) )) < 0 ) {
      cout << "HitWire::updInternal(): flip sign for hitb=" << getHitb() 
	   << " from " << LRSign
	   << " resid=" << (*r)(1);
      LRSign = -LRSign;
      (*r)(1) = (*r)(1) - 2 * (*m)(1);
      (*m)(1) = -(*m)(1);
      cout << " to " << LRSign << " resid=" << (*r)(1) << endl;
    }
  }
  else {
    double wirDist = - (*r)(1);
    if (wirDist > 0) LRSign = 1;
    else LRSign = -1;
    
    (*m)(1) = LRSign * fabs(getHitb()->q);
    cout << "HitWire::updInternal(): set sign for hitb=" << getHitb() 
	 << " to   " << LRSign
	 << " resid=" << (*r)(1);
  }
}

