*-- Author :    Rainer Mankel   30/07/96
*-- changed:    A.Spiridonov    05/05/98

      subroutine tftrans(dz,izf,jzf,f,ecorr,iter,istat)

*     **** construct transport matrix for electrons & non-electrons

*     **** takes track parameters,cov matrix,z position, plane and
*     **** hit numbers from candidate izf and fills prediction in jzf

      implicit none
      integer ndimmag
      parameter (ndimmag=5)

      double precision dz ! z difference for transport (in)
      integer izf,        ! row # of start candidate   (in)
     +        jzf         ! row # of target candidate  (in)
      double precision f (ndimmag,ndimmag)
                          ! transport matrix          (out)
      logical ecorr       ! energy correction flag     (in)
      integer iter,       ! iteration #                (in)
     +        istat       ! return status             (out)

#include "arte/Arte.fi"
      integer npoints,ifstart,ifend,ii,jj,kk,ll,mm,iii,jjj,ipl,imc,
     +             mimp,nnmimp
      integer message_id_1, message_id_2 , message_id_3,
     +     message_id_4, message_id_5
      double precision z_target, rrstart, x_in(5), x_out(5)
      real entry_pt(3), exit_pt(3), pnoise, dnoise, sqxy, tfwall,
     +     rad_thick, sum_wall, zmean
      double precision entry_z, exit_z, sum, cnoise(5,5)
#include "rang/zf.fi"
#include "rang/zfmode.fi"
#include "rang/zfpar.fi"

#include "rang/zma.fi"
      double precision f1(ndimmag,ndimmag),f2(ndimmag,ndimmag)
      double precision x_border(ndimmag)

#include "auto/HITB.fi"
#include "auto/GEO.fi"
#include "auto/MIMP.fi"

      data message_id_1 /1/,message_id_2 /2/,message_id_3 /3/,
     +     message_id_4 /4/,message_id_5 /5/

#include "arte/Arte.ff"
#include "auto/HITB.ff"
#include "auto/GEO.ff"
#include "auto/MIMP.ff"

*     **** local statement functions
      entry_z() = dble( entry_pt(3) )
      exit_z()  = dble( exit_pt (3) )

      istat = 0

      if(dabs(dz).le.1.e-4) then
        do ii=1,5
          xp_zf(ii,jzf)=xx_zf(ii,izf) 
          do jj=1,5
            cp_zf(ii,jj,jzf)=cx_zf(ii,jj,izf)
            if(ii.eq.jj) then 
              f(ii,ii) = 1.
            else
              f(ii,jj) = 0.
            end if
          end do
        end do
        go to 999
      end if

      ipl = plane_zf(izf)

      if (min0(izf,jzf).le.0.or.max0(izf,jzf).gt.nzfmx) then
         print *,' () tftrans: invalid izf,jzf=',izf,jzf
         istat = -1
         go to 999
      endif

      z_target = z_zf(izf) + dz

*     **** step 1: determine case flags
*            ifstart=0 : Runge-Kutta transport
*            ifstart=1 : starting point outside magnet
*                    2 :    ||          inside    ||
*            ifend  =1 : exit     point outside   ||
*                    2 :    ||          inside    ||


      if (.not.l_homog) then    ! inhomogeneous field
         if (amax1(z_zf(izf),sngl(z_target)).lt.zmin_mag_zfpar
     +        .or.amin1(z_zf(izf),sngl(z_target)).gt.zmax_mag_zfpar)
     +                                                            then
            ifstart = 1
            ifend   = 1
         else
            ifstart = 0
            ifend = 0
         endif

      else                      ! homogeneous field
         if (z_zf(izf).gt.(ffeld_z(1)+ffeld_r(1))
     +        .and.sngl(z_target).gt.(ffeld_z(1)+ffeld_r(1))) then
            ifstart = 1
            ifend   = 1
         else
*           **** check more precisely if starting point is inside magnet
            rrstart = (z_zf(izf) - ffeld_z(1))**2 + xx_zf(1,izf)**2
*           **** start flag
            if (rrstart.gt.dble(ffeld_r(1)**2)) then
               ifstart = 1
            else
               ifstart = 2
            endif
*           **** search intersection
            call tfmgl(ifstart,izf,npoints,entry_pt,exit_pt)
*           **** check endpoint
            if (npoints.gt.0) then
               if (ifstart.eq.1) then
                  if (z_target.gt.entry_z()
     +                 .or.z_zf(izf).lt.entry_z()) then
                     ifend= 1
                  else
                     ifend = 2
                  endif
               else
                  if (z_target.lt.exit_z()) then
                     ifend= 1
                  else
                     ifend = 2
                  endif
               endif
            else
*              **** no further intersection with magnet path
*              **** assume track stays in medium
               ifend = ifstart
            endif
         endif
      endif

*     **** step 2: transport vector & fill derivative matrix f
      if (ifstart.eq.0) then
         if (iter.eq.0) then
            do ii=1,5
               x_in(ii) = xx_zf(ii,izf)
            enddo
         else
            do ii=1,5
               x_in(ii) = xs_zf(ii,izf)
            enddo
         endif

         if (abs(sngl(dz)).gt.dzpara_mag_zfpar) then
            call tftrkut(ndimmag,x_in,dble(z_zf(izf)),z_target,
     +           xp_zf(1,jzf),f,istat)
         else
            call tftpara(ndimmag,x_in,dble(z_zf(izf)),z_target,
     +           xp_zf(1,jzf),f,istat)
         endif

         if (istat.lt.0) then
            print *,' tftrans: transport impossible (rk) '

            call mepric(message_id_1,1,
     +           'tftrans: transport impossible (rk)')
            go to 999
         endif

         if (iter.gt.0) then
            do ii=1,5           ! overwrite xp_zf with linear expansion
               sum = 0.
               do jj=1,5
                  sum = sum + f(ii,jj) * (xx_zf(jj,izf)-xs_zf(jj,izf))
               enddo
               xp_zf(ii,jzf) = xp_zf(ii,jzf) + sum
            enddo
         endif
      elseif (ifstart.eq.1.and.ifend.eq.1) then
*        **** start/end outside magnet
*        **** ignore possibility of WHOLE MAGNET in between
         xp_zf(1,jzf) = xx_zf(1,izf) + dz * xx_zf(3,izf)
         xp_zf(2,jzf) = xx_zf(2,izf) + dz * xx_zf(4,izf)
         xp_zf(3,jzf) = xx_zf(3,izf)
         xp_zf(4,jzf) = xx_zf(4,izf)
         xp_zf(5,jzf) = xx_zf(5,izf)

*        **** f matrix
         do ii=1,5
            do jj=1,5
               if (ii.eq.jj) then
                  f(ii,jj) = 1.
               elseif ((ii.eq.1.and.jj.eq.3).or.(ii.eq.2.and.jj.eq.4))
     +                       then
                  f(ii,jj) = dz
               else
                  f(ii,jj) = 0.
               endif
            enddo
         enddo
      elseif (ifstart.eq.2.and.ifend.eq.2) then
*        **** track fully contained in magnet
         if (iter.eq.0) then
            call tfthel(ndimmag,xx_zf(1,izf),dz,xp_zf(1,jzf),f,istat)
            if (istat.lt.0) then
               call mepric(message_id_2,1,
     +              'tftrans: transport impossible (hel)')
               go to 999
            endif
         else
            call tfthel(ndimmag,xs_zf(1,izf),dz,x_out,f,istat)
            if (istat.lt.0) then
               print *,' tftrans: transport impossible (hel) '
               call mepric(message_id_3,1,
     +              'tftrans: transport impossible (hel)')
               go to 999
            endif
            do ii=1,5           ! overwrite xp_zf with linear expansion
               sum = 0.
               do jj=1,5
                  sum = sum + f(ii,jj) * (xx_zf(jj,izf)-xs_zf(jj,izf))
               enddo
               xp_zf(ii,jzf) = x_out(ii) + sum
            enddo
         endif
      else
*        **** split transport into inside & outside pieces
         if (iter.eq.0) then
            do ii=1,5
               x_in(ii) = xx_zf(ii,izf)
            enddo
         else
            do ii=1,5
               x_in(ii) = xs_zf(ii,izf)
            enddo
         endif

         if (ifstart.eq.1.and.ifend.eq.2) then
*           **** from outside to inside
            call tftlin(ndimmag,x_in,entry_z()-z_zf(izf),
     +           x_border,f1)
            call tfthel(ndimmag,x_border,z_target-entry_z(),
     +           xp_zf(1,jzf),f2,istat)
            if (istat.lt.0) then
               call mepric(message_id_4,1,
     +              'tftrans: transport impossible (lin/hel)')
               go to 999
            endif

         elseif (ifstart.eq.2.and.ifend.eq.1) then
*           **** from inside to outside
            call tfthel(ndimmag,x_in,exit_z()-z_zf(izf),
     +           x_border,f1,istat)
            if (istat.lt.0) then
               call mepric(message_id_5,1,
     +              'tftrans: transport impossible (hel/lin)')
               go to 999
            endif

            call tftlin(ndimmag,x_border,z_target-exit_z(),
     +           xp_zf(1,jzf),f2)
         endif

         do ii=1,ndimmag        ! combine derivative matrices
            do jj=1,ndimmag
               sum = 0.
               do kk=1,ndimmag
                  sum = sum + f2(ii,kk) * f1(kk,jj)
               enddo
               f(ii,jj) = sum
            enddo
         enddo

         if (iter.gt.0) then
            do ii=1,5           ! overwrite xp_zf with linear expansion
               sum = 0.
               do jj=1,5
                  sum = sum + f(ii,jj) * (xx_zf(jj,izf)-xs_zf(jj,izf))
               enddo
               xp_zf(ii,jzf) = xp_zf(ii,jzf) + sum
            enddo
         endif
      endif

*     **** step 3: propagate cov matrix
      call tuhmimp(hit_zf(izf),imc,mimp,nnmimp)
      pnoise = amax1(abs( fmimp_p(mimp) ),1.)
*     **** get radiation thickness
      sum_wall = tfwall(ipl,sngl(z_target),
     +     sngl(xx_zf(1,izf)),
     +     sngl(xx_zf(2,izf)),
     +     sngl(xx_zf(3,izf)),
     +     sngl(xx_zf(4,izf)))
      dnoise = f_moliere_zfpar * sum_wall / pnoise**2


*     **** correct for angle
      sqxy = dsqrt(1.d0 + xp_zf(3,jzf)**2 + xp_zf(4,jzf)**2)
      dnoise = dnoise * sqxy
      rad_thick = sum_wall * sqxy / 0.0136**2

      do ii=1,5
         do jj=1,5
            cnoise(ii,jj) = 0.
         enddo
      enddo
      cnoise(3,3) = (1.+xp_zf(3,jzf)**2) * sqxy**2 * dnoise
      cnoise(4,4) = (1.+xp_zf(4,jzf)**2) * sqxy**2 * dnoise
      cnoise(3,4) = -xp_zf(3,jzf) * xp_zf(4,jzf) * sqxy**2 * dnoise
      cnoise(4,3) = cnoise(3,4)

      do ii=1,5
         do jj=1,ii
            sum = 0.
            do ll=1,5
               do mm=1,5
                  sum = sum
     +                 + f(ii,ll) * (cx_zf(ll,mm,izf)+cnoise(ll,mm))
     +                 * f(jj,mm)
               enddo
            enddo
            cp_zf(ii,jj,jzf) = sum
            cp_zf(jj,ii,jzf) = sum
         enddo
      enddo

*     **** electron energy loss correction
      if (ecorr) then
         zmean = 0.5 * (z_zf(izf) + z_target)
         if ( zmean.gt.zmin_ec_zfpar.and.
     +        zmean.lt.zmax_ec_zfpar) then

            if(dz.lt.0.) then
              xp_zf(5,jzf) = xp_zf(5,jzf) * exp(- rad_thick)
            else
              xp_zf(5,jzf) = xp_zf(5,jzf) * exp(  rad_thick)
            end if

            cp_zf(5,5,jzf) = cp_zf(5,5,jzf)
     +           + xp_zf(5,jzf)**2
     +              * (exp(-rad_thick * alog(3.)/alog(2.))
     +           - exp(-2.*rad_thick))

         endif
      endif

      if (diag_level.ge.6) then
         print *,' tftrans: dz, izf=',dz,izf
         print *,' tftrans: xx_zf(,izf)=',(xx_zf(iii,izf),iii=1,5)
         print *,' tftrans: cx_zf(,,izf)',
     +        ((cx_zf(iii,jjj,izf),iii=1,5),jjj=1,5)
         print *,' tftrans: xp_zf=',(xp_zf(iii,jzf),iii=1,5)
         print *,' tftrans: cp_zf=',
     +        ((cp_zf(iii,jjj,jzf),iii=1,5),jjj=1,5)
      endif

 999  end
