
// ********************************************************************
//
//  Routine:      rc99AParticleID
//  Author:       T.Oest/D.Samtleben
//
//  Description:  stand alone reconstruction
//
//  Modification:
//   --  RICH particle ID added                        14/09/99 T.Oest
//   --  bug correction (rc99AidRICH was never called) 16/04/00 T.Oest
//   --  introduce timers                              18/05/00 AG
//     -- creation of chrono jobs only once per job    24/05/00 D.Samtleben
//
// ********************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ARTE includes
#include "reco/rc99Areco.hh"
#include "reco/rcflags.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "farm/farm.hh"

// globals
float time_rc99AidRICH = -1;
float time_rc99AidECAL = -1;

// prototypes
int rc99AidRICH(int);
int rc99AidECAL();

//********************************************************************
// rcParticleID
//********************************************************************
  int rc99AParticleID() {

  int ECALidFlag = 0;
  int RICHidFlag = 0;

  // timers
  static Chrono *cron_rc99AidRICH = 0;
  static Chrono *cron_rc99AidECAL = 0;
  
  static bool first = 1;
  if( first ){
    first = 0;
    ChronoManager CM;
    
    cron_rc99AidRICH = CM.createChrono( "cron_rc99AidRICH"  );
    cron_rc99AidECAL = CM.createChrono( "cron_rc99AidECAL"  );
  }
  
//===================================================================
//  RICH particle ID for electrons
//===================================================================

    if (recoPidRICH){
      ARTE_mark( "rc99AidRICH", 0 );
      cron_rc99AidRICH->start();
      RICHidFlag = rc99AidRICH(recoPidRICH) ;
      cron_rc99AidRICH->stop();
      time_rc99AidRICH = cron_rc99AidRICH->lastTime();        
    }

//===================================================================
//  ECAL particle ID for electrons & gammas
//===================================================================

    if (recoPidECAL){
      ARTE_mark( "rc99AidECAL", 0 );
      cron_rc99AidECAL->start();
      ECALidFlag = rc99AidECAL();
      cron_rc99AidECAL->stop();
      time_rc99AidECAL = cron_rc99AidECAL->lastTime();        
    }

//===================================================================
//  flag for particle ID
//===================================================================

    return 0;

  }


