
// ********************************************************************
//
//  Routine:      rc99Aevnt
//  Author:       T.Oest/D.Samtleben
//
//  Description:  event reconstruction
//
//  Modification:
//    -- matching added                         03/06/99 T.Oest
//    -- vertexing added                        06/09/99 T.Oest
//    -- particle ID added                      16/09/99 T.Oest
//    -- event classification added,
//       select decay candidates                29/10/99 T.Oest
//    -- track points RPNT filled               07/01/00 T.Oest
//    -- fill EVHI bank                         09/02/00 T.Oest
//    -- introduce MODULE                       11/04/00 A.Gellrich
//    -- introduce timers                       17/05/00 AG
//    -- times to globals                       18/05/00 AG
//
// ********************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>

// ARTE includes
#include "arte/arte.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "farm/farm.hh"
#include "reco/rc99Areco.hh"

// globals
static int firstcall = 1;

float time_rc99AStandAlone  = -1;
float time_rc99Match        = -1;
float time_rc99SelectTracks = -1;
float time_rc99Arefit       = -1;
float time_rcSetTrackPoints = -1;
float time_rc99AParticleID  = -1;
float time_rc99Avertex      = -1;
float time_rcSelectDecays   = -1;
float time_rcClassify       = -1;

// prototypes
int  rc99Arefit();
int  rc99SelectTracks();
int  rcSelectDecays();
int  rcSetTrackPoints();
int  rcClassify();
void rc99Avertex();
void rcfillEVHI();

//*****************************************************************
// function   no action for MC
//*****************************************************************

void rc99Aevnt() {

//==================================================================
// 1. initialization, return for MC
//==================================================================
  ARTE_mark( "rc99Aevnt", 0 );
  
  ArtePointer<EVHD> header(1);
  
  if (ArteTable<MTRA>::begin() != ArteTable<MTRA>::end()){
    if (header->exp > 0 )         header->exp = -1;
  }

  // Timers
  static Chrono *cron_rc99AStandAlone  = 0;
  static Chrono *cron_rc99Match        = 0;
  static Chrono *cron_rc99SelectTracks = 0;
  static Chrono *cron_rc99Arefit       = 0;
  static Chrono *cron_rcSetTrackPoints = 0;
  static Chrono *cron_rc99AParticleID  = 0;
  static Chrono *cron_rc99Avertex      = 0;
  static Chrono *cron_rcSelectDecays   = 0;
  static Chrono *cron_rcClassify       = 0;

  // inits
  if (firstcall == 1 ) {
    firstcall = 0;

    // rc99Ainit
    rc99Ainit();

    // timers    
    ChronoManager CM;
    cron_rc99AStandAlone  = CM.createChrono( "cron_rc99AStandAlone" );
    cron_rc99Match        = CM.createChrono( "cron_rc99Match" );
    cron_rc99SelectTracks = CM.createChrono( "cron_rc99SelectTracks" );
    cron_rc99Arefit       = CM.createChrono( "cron_rc99Arefit" );
    cron_rcSetTrackPoints = CM.createChrono( "cron_rcSetTrackPoints" );
    cron_rc99AParticleID  = CM.createChrono( "cron_rc99AParticleID" );
    cron_rc99Avertex      = CM.createChrono( "cron_rc99Avertex" );
    cron_rcSelectDecays   = CM.createChrono( "cron_rcSelectDecays" );
    cron_rcClassify       = CM.createChrono( "cron_rcClassify" );
  }
  
//==================================================================
// 2. stand alone reconstruction
//==================================================================

  ARTE_mark( "rc99AStandAlone", 0 );
  cron_rc99AStandAlone->start();
  int rcStandAloneFlag = rc99AStandAlone();
  cron_rc99AStandAlone->stop();
  time_rc99AStandAlone = cron_rc99AStandAlone->lastTime();
  
//==================================================================
// 3. matching, track selection
//==================================================================

  ARTE_mark( "rc99Match", 0 );
  cron_rc99Match->start();
  int MatchingFlag = rc99Match();
  cron_rc99Match->stop();
  time_rc99Match = cron_rc99Match->lastTime();
  
  ARTE_mark( "rc99SelectTracks", 0 );
  cron_rc99SelectTracks->start();
  int RTRAFlag = rc99SelectTracks();
  cron_rc99SelectTracks->stop();
  time_rc99SelectTracks = cron_rc99SelectTracks->lastTime();
  
//==================================================================
// 4. refit of tracks, fill RPNT bank
//==================================================================

  ARTE_mark( "rc99Arefit", 0 );
  cron_rc99Arefit->start();
  int refit = rc99Arefit();
  cron_rc99Arefit->stop();
  time_rc99Arefit = cron_rc99Arefit->lastTime();
  
  ARTE_mark( "rcSetTrackPoints", 0 );
  cron_rcSetTrackPoints->start();
  int RPNTFlag = rcSetTrackPoints();
  cron_rcSetTrackPoints->stop();
  time_rcSetTrackPoints = cron_rcSetTrackPoints->lastTime();

//==================================================================
// 5. Particle ID
//==================================================================

  ARTE_mark( "rc99AParticleID", 0 );
  cron_rc99AParticleID->start();
  int ParticleID = rc99AParticleID();
  cron_rc99AParticleID->stop();
  time_rc99AParticleID = cron_rc99AParticleID->lastTime();

//==================================================================
// 6. vertexing
//==================================================================

  ARTE_mark( "rc99Avertex", 0 );
  cron_rc99Avertex->start();
  rc99Avertex();
  cron_rc99Avertex->stop();
  time_rc99Avertex = cron_rc99Avertex->lastTime();
  
//==================================================================
// 7. reconstruction of decays
//==================================================================

  ARTE_mark( "rcSelectDecays", 0 );
  cron_rcSelectDecays->start();
  int SelectDecays = rcSelectDecays();
  cron_rcSelectDecays->stop();
  time_rcSelectDecays = cron_rcSelectDecays->lastTime();

//==================================================================
// 8. set event classification flag
//==================================================================

  ARTE_mark( "rcClassify", 0 );
  cron_rcClassify->start();
  rcClassify();
  cron_rcClassify->stop();
  time_rcClassify = cron_rcClassify->lastTime();
  
//==================================================================
// 9. fill EVHI bank
//==================================================================

  ARTE_mark( "rcfillEVHI", 0 );
  rcfillEVHI();
}
