
// ********************************************************************
//
//  Routine:      rc99Arefit 
//  Author:       T.Oest
//
//  Description:  refit for RTRA tracks
//
//  Modification:
//   -- call the refit only if RHITs are available  9/09/99 T.Oest
//   -- call global alignment transformation       29/10/99 T.Oest
//   -- no global alignment transformation, this 
//      is done now for RHITs                      13/01/00 T.Oest
//   -- call rcSetRTRArelations                    18/01/00 T.Oest
//   -- new refit for 00 RTRA selection            17/02/00 T.Oest
//   -- OTR refit includes TC segments             17/05/00 T.Oest
//   -- include timing                             18/05/00 AG
//   -- creation of chrono jobs only once per job  24/05/00 D.Samtleben
//   -- fill RHITs before refit                    26/05/00 T.Oest
//
// ********************************************************************

#include <algo.h>
#include <arte/arte.hh>
#include <arte/artefunc.h>
#include <rang/Track.hh>
#include <rang/Node.hh>
#include <rang/Geom.hh>
#include <rang/rgfit.hh>
#include "rang/Fit.hh"
#include "reco/rcflags.hh"
#include "arte/RSEG_cmp.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "farm/farm.hh"

extern int rcOTRrefit;

float time_rcrefitOTR = -1;

void acRSEGinGlobalCoordinates(ArtePointer<RSEG> rseg, int flag );
void rcSetRTRArelations(ArtePointer<RTRA>, int);

void rcrefitOTR();

//****************************************************************
// function
//****************************************************************
void rc99Arefit(){

  // timers
  static Chrono *cron_rcrefitOTR = 0;
  
  static bool first = 1;
  if( first ){
    first = 0;
    ChronoManager CM;
    
    cron_rcrefitOTR  = CM.createChrono( "cron_rcrefitOTR"  );
  }
  
//------------------------------------------------------------------
// loop over all tracks
//------------------------------------------------------------------

  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); ++rtra ) {
    if( (*rtra)->fit == 0 || (*rtra)->fit == 11) {


     rcSetRTRArelations(*rtra, 2);

//------------------------------------------------------------------
// refit for 00 track selection
//------------------------------------------------------------------

   if (rcSelectTracksStatus == 2) rangerFit(*rtra);

//------------------------------------------------------------------
// get the silicon segment and do a refit
//------------------------------------------------------------------
  if( rcSelectTracksStatus == 1 && (*rtra)->hsi>0) {

  for( ArteRelationIterator<RSEG> rseg = (*rtra)->rseg.begin();
       rseg != (*rtra)->rseg.end(); ++rseg ) {
    if (((*rseg)->cmp == 1 || (*rseg)->cmp == 65537) &&
        (*rseg)->rhit.begin() != (*rseg)->rhit.end()){

       //==== keep the information on the number of hits in rseg->time/dtim
           float np=(*rseg)->time;
           float nn=(*rseg)->dtim;
 
 for( ArteRelationIterator<RHIT> rhit = (*rseg)->rhit.begin();
            rhit != (*rseg)->rhit.end(); ++rhit ) {
        (*rhit)->p = (*rtra)->pf;
       };

      (*rseg)->pf = (*rtra)->pf;
      (*rseg)->cvf[14] = (*rtra)->cvf[14];



      int rsegFit   = (*rseg)->fit;
      int rs        = *rseg - ArtePointer<RSEG>(1) + 1;
      int fitstatus = rgfit_(rs,0);
      (*rseg)->fit  =  rsegFit;


          (*rseg)->time=np;
          (*rseg)->dtim=nn;


      if( fitstatus >= 0 ) {

        (*rtra)->lth  =  (*rseg)->chi;
        (*rtra)->zf   =  (*rseg)->zf;
        (*rtra)->xf   =  (*rseg)->xf;
        (*rtra)->yf   =  (*rseg)->yf;
        (*rtra)->txf  =  (*rseg)->txf;
        (*rtra)->tyf  =  (*rseg)->tyf;

        (*rtra)->cvf[10] = (*rtra)->cvf[10] * sqrt((*rseg)->cvf[0]/(*rtra)->cvf[0]);
        (*rtra)->cvf[11] = (*rtra)->cvf[11] * sqrt((*rseg)->cvf[2]/(*rtra)->cvf[2]);
        (*rtra)->cvf[12] = (*rtra)->cvf[12] * sqrt((*rseg)->cvf[5]/(*rtra)->cvf[5]);
        (*rtra)->cvf[13] = (*rtra)->cvf[13] * sqrt((*rseg)->cvf[9]/(*rtra)->cvf[9]);

        for (int i=0; i<=9 ; i++) (*rtra)->cvf[i] = (*rseg)->cvf[i];
      } else {
        (*rtra)->fit  = 1000+(*rtra)->fit; 
      };
  };
  };
  };
};
};
//------------------------------------------------------------------
// do an OTR refit
//------------------------------------------------------------------

  if (rcOTRrefit == 1){
    ARTE_mark( "rcrefitOTR", 0 );
    cron_rcrefitOTR->start();
    rcrefitOTR();
    cron_rcrefitOTR->stop();
    time_rcrefitOTR = cron_rcrefitOTR->lastTime();   
  }
  

};


//****************************************************************
// function
//****************************************************************


void rcrefitOTR(){

  if (rcOTRrefit == 0) return;

//------------------------------------------------------------------
// loop over all tracks
//------------------------------------------------------------------

  rangerParticleType Type = MUON;

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

    if( (*rseg)->fit == 0 && ((*rseg)->cmp & Rsegc::patt) &&
        (*rseg)->zf < 900.) {

      ArtePointer<RSEG>  tcfseg = 0;
      vector<ArtePointer<RSEG> > rsegVec;
      rsegVec.push_back(*rseg);

      for( ArteRelationIterator<RSEG> tcseg = (*rseg)->rend.begin();
           tcseg != (*rseg)->rend.end(); ++tcseg ) {
       if ((*tcseg)->fit == 0 && ((*tcseg)->cmp & Rsegc::wch) &&
           (*tcseg)->zf > 900.) {
        rsegVec.push_back(*tcseg);
        tcfseg = *tcseg;
       };
      };

      int fitstatus = rangerFit(rsegVec,Type);


  };
 };
};
