////////////////////////////////////////////////////////////////
//
// Author: D.Dujmic
//
// Particle ID using RICH stand-alone ring reconstruction. 
// - rc99idRISE(float ip, float rad, float *dd) returns deviations
//   in sigma^2
// - rc99idRISE(ArtePointer<RTRA> t) fills RTRA->lre..lrp with
//   probabilities normalized to 1
// Code returns: 0...4 best hypothesis
//               -1 no reliable hypothesis
//
// Modifications:
//   May 29, 2000 (DD): calculation moved to RingSearchARTE
//
//
////////////////////////////////////////////////////////////////

#include "arte/arte.hh"
#include "rise/RingSearchARTE.h"

// deviations in sigma^2 -> dd
int rc99idRISE(float ip, float radius, float *dd) {
  if (!RICH_Search) RICH_Search = new RingSearchARTE;
  RingSearchTrack::ip              = ip;
  RingSearchAttrib::id             = true;
  RingSearchAttrib::use_simple_R2P2= true;
  RingSearchAttrib::fillrtra       = false;
  RICH_Search->makeR2P2lines();
  int ret=RICH_Search->R2P2likelihood(radius);
  dd=RICH_Search->GetLr();
  return ret;
}

// RTRA is filled with probabilities normalized to 1
int rc99idRISE(ArtePointer<RTRA> t) {
  if (!RICH_Search) RICH_Search=new RingSearchARTE;
  RingSearchTrack::ip              = t->pf;
  RingSearchAttrib::id             = true;
  RingSearchAttrib::use_simple_R2P2= false;
  RingSearchAttrib::fillrtra       = true;
  RICH_Search->makeR2P2lines();
  int ret=RICH_Search->R2P2likelihood(t);
  return ret;
}
