//********************************************************************
//
//  Routine:      rccreateRTRA
//  Author:       O. Igonkina
//
//  Description:  select and store RTRA tracks based on VDS/main tracker
//
//  Modification:
//  - rcSetRtraHitRelations +  rcCountRtraHits added   27/00 D. Samtleben 
//
//********************************************************************
#include <iostream>
#include <math.h>

#include <arte/arte.hh>
#include <arte/RSEG_cmp.hh>
#include <marple/marple.hh>

extern "C"
{
  void hf1_(int*, float*, float*);
}


int makeRtraBaseVds(ArtePointer<RSEG> rseg, ArtePointer<RTRA>& rtraOut );
int rcCountRtraHits();
int rcSetRtraHitRelations();

int rccreateRTRA(){

  ArteTable<RTRA>::clear();


  float weight = 1;
  //---------------------------------------------------------
  // get first SVD-OTR 
  //---------------------------------------------------------
  for(ArteTable<RSEG>::iterator imatch = ArteTable<RSEG>::begin();
      imatch != ArteTable<RSEG>::end() ; ++imatch)
    {
      if( ((*imatch)->cmp & Rsegc::bitsign) != 0 ) continue;
      if( ( *imatch)->fit != 7 ) continue;
      if( ((*imatch)->cmp & Rsegc::vxd)  == 0 ||
	  ((*imatch)->cmp & Rsegc::patt) == 0 )  continue;

      ArtePointer<RSEG> svd;
      ArtePointer<RSEG> otr;
      for( ArteRelationIterator<RSEG> itr = (*imatch)->rend.begin();
	   itr != (*imatch)->rend.end(); ++itr )
	{
	  if( ((*itr)->cmp & Rsegc::vxd) != 0 )  svd = *itr;
	  if( ((*itr)->cmp & Rsegc::patt) != 0 )  otr = *itr;
	}
      ArtePointer<RTRA> rtra;
      makeRtraBaseVds(svd,rtra);

      // otr
      createRelation( rtra, otr, &RTRA::rseg );
      if( otr->nhit == 0 ) 
	rtra->hou = (int)otr->time;
      else
	rtra->hou = otr->nhit;

      // momentum + chi2
      rtra->pf = (*imatch)->pf;
      rtra->cvf[14] = (*imatch)->cvf[14]; // cov(5,5)
      rtra->chi = (*imatch)->chi;

    }


  //---------------------------------------------------------
  // get SVD - ECAL
  //---------------------------------------------------------
  // int ihistOtrEcalP1  = 100;
  //int ihistOtrEcalP13 = 110;
  //int ihistOtrEcalP3  = 120;

  for(ArteTable<RSEG>::iterator imatch = ArteTable<RSEG>::begin();
      imatch != ArteTable<RSEG>::end() ; ++imatch)
    {
      if( ((*imatch)->cmp & Rsegc::bitsign) != 0 ) continue;
      if( ( *imatch)->fit != 7 ) continue;
      if( ((*imatch)->cmp & Rsegc::vxd)  == 0 ||
	  ((*imatch)->cmp & Rsegc::ecal) == 0 )  continue;

      // find svd
      ArtePointer<RSEG> svd;
      for( ArteRelationIterator<RSEG> itr = (*imatch)->rend.begin();
	   itr != (*imatch)->rend.end(); ++itr )
	{
	  if( ((*itr)->cmp & Rsegc::vxd) != 0 )  svd = *itr;
	}
      ArtePointer<RCCL> ecal = *(*imatch)->rccl.begin();

      int assigned = 0;

      // find RTRA
      for(ArteRelationIterator<RTRA> irtra = svd->rtra.begin();
	  irtra != svd->rtra.end() ; ++irtra )
	{
	  // find corresponding OTR
	  for( ArteRelationIterator<RSEG> itr = (*irtra)->rseg.begin();
	   itr != (*irtra)->rseg.end(); ++itr )
	    {
	      if( ((*itr)->cmp & Rsegc::patt) == 0 ) continue;
	      float dx = (*itr)->xe+(ecal->z-(*itr)->ze)*(*itr)->txe - ecal->x;
	      float dy = (*itr)->ye+(ecal->z-(*itr)->ze)*(*itr)->tye - ecal->y;
	      if( fabs(dx) > 4 || fabs(dy) > 8 ) continue;
	      
	      // good coordinates
	      assigned = 1;
	      (*irtra)->hca = 1;

           //======= take momentum of vds-ecal match!
              // momentum + chi2
              (*irtra)->chi = (*imatch)->chi;
              (*irtra)->pf = (*imatch)->pf;
              (*irtra)->cvf[14] = (*imatch)->cvf[14]; // cov(5,5)
           //================================================= 
	      createRelation( *irtra, ecal, &RTRA::rccl );
	      
	      // float p = 1/(*irtra)->pf;
	      //float dP = p*(*imatch)->pf;
	      //if( fabs(p) < 10 ) 
	      //	hf1_(&ihistOtrEcalP1, &dP, &weight);
	      //else if( fabs(p) < 30 ) 
	      //	hf1_(&ihistOtrEcalP13, &dP, &weight);
	      //else
	      //	hf1_(&ihistOtrEcalP3, &dP, &weight);
	    }
	}
      
      if( !assigned ) // no proper SVD-OTR found
	{
	  ArtePointer<RTRA> rtra;
	  makeRtraBaseVds(svd,rtra);
	  rtra->hca = 1;
	  createRelation( rtra, ecal, &RTRA::rccl );
	  // momentum
	  rtra->pf = (*imatch)->pf;
          rtra->chi = (*imatch)->chi;
	  rtra->cvf[14] = (*imatch)->cvf[14]; // cov(5,5)
	}
    }

  //---------------------------------------------------------
  // get SVD - RICH
  //---------------------------------------------------------
  //  int ihistRichEcalP1  = 101;
  //int ihistRichOtrP1   = 102;
  //int ihistRichEcalP13 = 111;
  //int ihistRichOtrP13  = 112;
  //int ihistRichEcalP3  = 121;
  //int ihistRichOtrP3   = 122;

  for(ArteTable<RSEG>::iterator imatch = ArteTable<RSEG>::begin();
      imatch != ArteTable<RSEG>::end() ; ++imatch)
    {
      if( ((*imatch)->cmp & Rsegc::bitsign) != 0 ) continue;
      if( ( *imatch)->fit != 7 ) continue;
      if( ((*imatch)->cmp & Rsegc::vxd)  == 0 ||
	  ((*imatch)->cmp & Rsegc::rich) == 0 )  continue;

      // find svd
      ArtePointer<RSEG> svd;
      ArtePointer<RSEG> rich;
      for( ArteRelationIterator<RSEG> itr = (*imatch)->rend.begin();
	   itr != (*imatch)->rend.end(); ++itr )
	{
	  if( ((*itr)->cmp & Rsegc::vxd) != 0 )  svd = *itr;
	  if( ((*itr)->cmp & Rsegc::rich) != 0 )  rich = *itr;
	}

      int assigned = 0;

      // find RTRA
      for(ArteRelationIterator<RTRA> irtra = svd->rtra.begin();
	  irtra != svd->rtra.end() ; ++irtra )
	{
	  int otrExist = 0;
	  // find corresponding OTR
	  for( ArteRelationIterator<RSEG> itr = (*irtra)->rseg.begin();
	   itr != (*irtra)->rseg.end(); ++itr )
	    {
	      if( ((*itr)->cmp & Rsegc::patt) == 0 ) continue;
	      otrExist = 1;
	      float dtx = (*itr)->txe - rich->txf;
	      float dty = (*itr)->tye - rich->tyf;
	      if( fabs(dtx) > 0.0036 || fabs(dty) > 0.013 ) continue;
	      
	      // good coordinates
	      assigned = 1;
	      (*irtra)->hri = 1;

           //======= take momentum of vds-rich match!
	      // momentum + chi2
	      (*irtra)->pf = (*imatch)->pf;
              (*irtra)->chi = (*imatch)->chi;
	      (*irtra)->cvf[14] = (*imatch)->cvf[14]; // cov(5,5)
          //===============================================

	      createRelation( *irtra, rich, &RTRA::rseg );
	      
	      //float p = 1/(*irtra)->pf;
	      //float dP = p*(*imatch)->pf;
	      //if( fabs(p) < 10 ) 
	      //	hf1_(&ihistRichOtrP1, &dP, &weight);
	      //else if( fabs(p) < 30 ) 
	      //	hf1_(&ihistRichOtrP13, &dP, &weight);
	      //else
	      //	hf1_(&ihistRichOtrP3, &dP, &weight);
	    }
	  if( ! otrExist )
	    {
	      RcTraPoint point(*irtra);
	      point.prolongate(1000);
	      float dtx = point.tx() - rich->txf;
	      float dty = point.ty() - rich->tyf;
	      if( fabs(dtx) > 0.003 || fabs(dty) > 0.003 ) continue;
	      
	      // good coordinates
	      assigned = 1;
	      (*irtra)->hri = 1;
	      createRelation( *irtra, rich, &RTRA::rseg );
           //======= take momentum of vds-rich match! 
	      // momentum + chi2
	      (*irtra)->pf = (*imatch)->pf;
              (*irtra)->chi = (*imatch)->chi;
	      (*irtra)->cvf[14] = (*imatch)->cvf[14]; // cov(5,5)
          //===============================================
	      
	      //float p = 1/(*irtra)->pf;
	      //float dP = p*1/(*imatch)->pf;
	      //	      if( fabs(p) < 10 ) 
	      //	hf1_(&ihistRichEcalP1, &dP, &weight);
	      //else if( fabs(p) < 30 ) 
	      //	hf1_(&ihistRichEcalP13, &dP, &weight);
	      //else
	      //	hf1_(&ihistRichEcalP3, &dP, &weight);
	    }

	}
      
      if( !assigned ) // no vds-OTR or vds--ECAL found
	{
	  ArtePointer<RTRA> rtra;
	  makeRtraBaseVds(svd,rtra);
	  rtra->hri = 1;
	  createRelation( rtra, rich, &RTRA::rseg );
	  // momentum + chi2
	  rtra->pf = (*imatch)->pf;
          rtra->chi = (*imatch)->chi;
	  rtra->cvf[14] = (*imatch)->cvf[14]; // cov(5,5)
	}
    }

//==================================================================
// now select remaining VDS,OTR segments
//==================================================================

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

  if ((*rseg)->fit == 0 &&
      (*rseg)->rtra.begin() == (*rseg)->rtra.end() && 
       (((*rseg)->cmp & Rsegc::vxd)  != 0 ||
        ((*rseg)->cmp & Rsegc::patt) != 0 )){

	  ArtePointer<RTRA> rtra;
	  makeRtraBaseVds(*rseg,rtra);

    if (((*rseg)->cmp & Rsegc::vxd)  != 0) {
      rtra->fit = 0;
    } else {
      rtra->fit = 0;
    };
      rtra->chi=(*rseg)->chi;

  
    for (int i=0; i<=14; i++)   rtra->cvf[i] = (*rseg)->cvf[i];

  }}

  int flagrel=rcSetRtraHitRelations();
  int flaghits=rcCountRtraHits();


  return 0;
}

int makeRtraBaseVds(ArtePointer<RSEG> svd, ArtePointer<RTRA> & outRtra)
{
  if( ( (svd->cmp & Rsegc::vxd) == 0) &&  (svd->cmp & Rsegc::patt) == 0) return 1;

  //------------------------------------------------------------  
  RTRA rtra;
  
  // empty variabales
  rtra.fit = 0;
  rtra.chi = -10.;
  rtra.pf = 0.;

  rtra.hin = 0;
  rtra.hou = 0;
  rtra.hri = 0;
  rtra.htr = 0;
  rtra.hmu = 0;
  
  rtra.flr = 0;
  rtra.flt = 0;
  rtra.fle = 0;
  rtra.flm = 0;
  
  rtra.lmmu = 0;
  rtra.lmh = 0;

  rtra.lte = 0;
  rtra.lth = 0;

  rtra.lee = 0;
  rtra.leh = 0;
  rtra.lemi= 0;
  
  rtra.lre = 0.;
  rtra.lrmu= 0.;
  rtra.lrpi= 0.;
  rtra.lrk = 0.;
  rtra.lrp = 0.;
  
  // --Track parameters--------------------------------

  rtra.xf  = svd->xf;
  rtra.yf  = svd->yf;
  rtra.zf  = svd->zf;
  rtra.txf = svd->txf;
  rtra.tyf = svd->tyf;
        
  for (int i=0; i<=3; i++){for (int j=i; j<=3; j++)
    rtra.cvf[c_matrix_pack[i][j]] = svd->cvf[c_matrix_pack[i][j]];};

  for (int i=0; i<=4; i++){
    rtra.cvf[c_matrix_pack[4][i]] = 0.;}

  rtra.hsi = int(svd->time+svd->dtim);   // number of vds hits

  outRtra = ArteTable<RTRA>::push_back( rtra );  
  createRelation( outRtra, svd, &RTRA::rseg );
  return 0;
}
