//--*-C++-*--
//-----------------------------------------------------------------------//
//
//  RingSearchARTE: ARTE interface for stand-alone ring reconstruction
//
//  Author: Reinhard Eckmann/Denis Dujmic
//
//  Created: May-98
//
//  Modifications:
//    May 15, 2000 (DD) nhit filled with Ring::numhit_3sigma
//    May 24, 2000 (DD) OTR based reconstruction: ringsearchRSEG()
//    May 25, 2000 (DD) Access to R2P2 calculation
//
//-----------------------------------------------------------------------//
#ifndef __RingSearchARTE_H__
#define __RingSearchARTE_H__

#include "rise/ring.h"
#include "rise/RingSearchDB.h"
#include <vector.h>

extern "C" {
  void rcrings_(float r1=30,float r2=53,float dr=-1,int t=5);
  void rcringsdb_();
  void rcringsid_(ArtePointer<RTRA> t);
  void rcringsclean_();
	   }
float* ringid(ArtePointer<RTRA> t);
float* ringid(float pf, float radius);
void   removerings();

/// Routine for RTRA seed-based reconstruction + cleanup
void   ringsearchRTRA();
/// Routine for RSEG seed-based reconstruction + cleanup
void   ringsearchRSEG();

///////////////////// RingSearchARTE ////////////////////////////////////

/// ARTE interface to stand-alone ringsearch program
class RingSearchARTE : public RingSearchDB {
private:
  // -- hits
  Hits            h;
  int             makeHitMap();

  // -- dualspace
  DualSpace d;

  // -- track/ring
  bool            SetSeedRTRA();
  bool            SetSeedRSEG();
  void            Choose();
  void            FillRSEG();
  void            FillRTRA(ArtePointer<RTRA> t, ArtePointer<RSEG> rich);
  int             n_mass;
  float           iR2P2sigma2[5];
  float           R2P2rad2[5];
  float           lr[5];
  vector<float>   radius_list;
  void            makeRadiusList(float r1, float r2, float dr);
  float           lrsum;


public:
  RingSearchARTE();
  void   getrings(float r1,float r2,float dr,int t);
  void   getrings(int i);
  void   getrings();
  int    swim(const double *source,double *dest,const double zout);
  int    R2P2likelihood(ArtePointer<RTRA> t);
  int    R2P2likelihood(float radius);
  void   makeR2P2lines();

  // -- access to reco data
  inline int    numrings()    const  { return d.numring();  }
  inline int    &numrings()          { return d.numring();  }
  inline Ring   &getRing(int i)      { return d.getRing(i); }
  inline int    imtra(int i)  const  { return d.mctrack(i); }
  inline float* GetLr()              { return &lr[0]; }
  inline float  GetLrsum()    const  { return lrsum; }
  inline int    numhits()     const  { return h.numhit; }
  inline float  GetR2P2rad2(int i)   { return R2P2rad2[i]; }
  inline float  GetR2P2sigma2(int i) { return iR2P2sigma2[i]; }
};


extern RingSearchARTE *RICH_Search;

#endif
