//-----------------------------------------------------------------------//
//
//  ring.C : Program for stand alone RICH reconstruction, 
//           required by RingSearchARTE
//
//  Author: Reinhard Eckmann
//
//  Created: May-98
//
//  All Files: RingSearchARTE.C, RingSearchARTE.h, ring.C, ring.h,
//             phila.C,  phila.h    
//
// Modifications:
//   23 May, 2000 (DD): attributes moved to RingSearchAttrib.C.h
//
//-----------------------------------------------------------------------//

#include "rise/ring.h"
#include "rise/RingSearchTiming.h"


//////////////// scaling /////////////////

float milliradian=1;//1000;
float setdualfac=1;//100;
float scaling=1/(milliradian*setdualfac);




/////////////////////  dualspace //////////////////////////////////////

void DualSpace::define(float rad,int t) {
    threshold=t;

    float sigma=1*sqrt(2)*0.7*(1e-3*milliradian);  // single intersection resolution

    float rr=(dualfac*rad);
    r=rr;
    r2=r*r;

    /*
    nsqrtf=10000;
    sqrtbini=float(nsqrtf)/float(r2);
    for (int i=0;i<=nsqrtf;i++) {
      float x=float(i)/float(nsqrtf);
      sqrtf[i]=r2*sqrt(x-x*x);
    }
    */

    phibin=2*sigma;
    nphi=(2*int(2*M_PI*rad/phibin+0.5))/2;  // require even nphi
    phibin=2*M_PI*rad/nphi;

    ri=1500./float(r);
    for (int i=0;i<3001;i++) {
      phitab[i]=int(rad/phibin*acos(float(i-1500)/1500));
    }
}

// ------------------------------------------------------------------------

int DualSpace::findmax() {
    register int i;
    register int jmax=-1;
    register int max=0;
    register int j=0;
    register int t=threshold;
    for (i=0;i<numabovethresh;i++) {
      register Center& c=*abovethresh[i];    
      if (c.numhit>t) {
        if (c.numhit>max) {
          max=c.numhit;
          jmax=j;
	}
        abovethresh[j++]=abovethresh[i];  // remove center below threshold
      }
    }
    numabovethresh=j;
    return jmax;
}

// ------------------------------------------------------------------------

void DualSpace::findring(Hit* hi,Hits* h) {

  numabovethresh=numassigned;
  memcpy(&abovethresh[0],&assignedcenter[0], sizeof(abovethresh[0])*numassigned);
  register int kmax;

  if ((kmax=findmax())>=0) { 
      
    // --  determine index in center from abovetreshhold pointer
    kmax=(abovethresh[kmax]-&center[0]);
    int k,k0=kmax-6;
    if (k0<0) k0+=nphi;
    float nsig=0;
    float nbg=0;
    for (k=0;k<14;k++) {
      int i=k+k0;
      if (i==nphi) {
	k0=0;
	i=0;
      }
      if (5<=k&&k<=7) {
	nsig+=center[i].numhit;
      }
      else {
	nbg+=center[i].numhit;
      }
    }
    nbg/=10;
    nsig-=3*nbg;
    //if (nbg==0) nbg=1e-3;
    if (nsig*nsig>16*3*nbg) {  // 4 sigma
      
      k0=kmax-1;
      if (k0<0) k0+=nphi;

      register Ring& ri=rings[numrings++];
      ri.ix0=0;
      ri.iy0=0;
      for (k=0;k<3;k++) {
	int i=k+k0;
	if (i==nphi) {
	  i=0;
	  k0=0;
	}
	if (center[i].numhit>0) {
	  ri.ix0+=((center[i].numhit-nbg)*(float(center[i].x0)/center[i].numhit));
	  ri.iy0+=((center[i].numhit-nbg)*(float(center[i].y0)/center[i].numhit));
	}
      }

      ri.ix0=(ri.ix0/nsig+hi->x);
      ri.iy0=(ri.iy0/nsig+hi->y);
      ri.numhit=int(nsig+0.5);
      ri.radius=r;
      switch (RingSearchAttrib::type) {
      case RingSearchType::nobg :
	if (!ri.calcring(*h)) numrings--; break;
      case RingSearchType::lkbg :
	if (!ri.fitring(*h))  numrings--; break;
      default:
	fprintf(stderr, "Wrong Ringsearch type\n");
	return;
      }
    }
  } // end while
}


// ------------------------------------------------------------------------

void DualSpace::ringsearch(Hits& h) {

  register Hit* hi=&h.hit[-1];
  register Hit* hn=&h.hit[h.numhit];
  register Hit* hin=&h.hit[h.numhit];
  while (++hi!=hin) {
    if (hi->assigned) continue;
    register float xi=hi->x;
    register float yi=hi->y;

    register Hit* hj=hi;
    register Hit* hjn=hi+(hn-hi);
    ringfl=0;

    while (++hj!=hn) {
      if (hj==hjn&&!ringfl) break;
      if (hj->assigned) continue;

      register float xdif=(hj->x-xi)/2;
      if (fabs(xdif)<=r) {
	register float ydif=(hj->y-yi)/2;

	if (fabs(ydif)<r) {
	  register float d2=xdif*xdif+ydif*ydif; 
	  if (d2<r2 && d2>0) { 
	    //register float a=sqrtf[int(sqrtbini*float(d2))]/float(d2);
	    register float a=sqrt(r2/d2-1);
	    register float dx=float(a*xdif);
	    register float dy=float(a*ydif);
	    append(xdif+dy,ydif-dx,hj);
	    append(xdif-dy,ydif+dx,hj);
	  }
	  else if (d2==r2){
	    append(xdif,ydif,hj);
	  }

	}
      }
      
    }
    if (ringfl) findring(hi,&h);
    register Center** c=&assignedcenter[-1];
    register Center** cn=&assignedcenter[numassigned];
    while (++c!=cn) (*c)->x0=(*c)->y0=(*c)->numhit=0;
    numassigned=0;
  }
}

int Ring::mctrack() const {
  int i;
  int *id=new  int[numhit];
  int *nid=new int[numhit];
  int n=0;
  int nmax=0; 
  int idmax=0;
  for (i=0;i<numhit;i++) {
    nid[i]=0;
    Hit& h=*hit[i];
    //if (!h.assigned) continue;
    if (!assigned[i]) continue;
    int k;
    for (k=0;k<n;k++) 
      if (id[k]==h.mctrack) break;
    id[k]=h.mctrack;
    nid[k]++;
    if (k==n) n++;
    if (nid[k]>nmax) { 
      idmax=id[k];
      nmax=nid[k];
    }
  }
  delete nid;
  delete id;
  //if (nmax<5) idmax=0;
  return idmax;
}



int invert(double* a,int n) {
  int i,j,k;
  int *p=new int[n];
  for (k=0;k<n;k++) {
    double* ak=&a[k*n];
    double max=0;
    p[k]=0;
    for (i=k;i<n;i++) {
      double s=0;
      double *ai=&a[i*n];
      for (j=k;j<n;j++) {
        s+=fabs(ai[j]);
        }
      double q=fabs(ai[k])/s;
      if (q>max) {
        max=q;
        p[k]=i;
      }
    }
    if (max==0) return 1;
    if (p[k]!=k) {
      double* ap=&a[p[k]*n];
      for (j=0;j<n;j++) {
        double h=ak[j];
        ak[j]=ap[j];
        ap[j]=h;
      }
    }
    double pivot=ak[k];
    for (j=0;j<n;j++) {
      if (j!=k) {
        ak[j]=-ak[j]/pivot;
        for (i=0;i<n;i++) {
          if (i!=k) a[i*n+j]+=a[i*n+k]*ak[j];
	}
      }
    }
    for (i=0;i<n;i++) a[i*n+k]/=pivot;
    ak[k]=1/pivot;
  }
  for (k=n-2;k>=0;k--) {
    if (p[k]!=k) {
      for (i=0;i<n;i++) {
        double h=a[i*n+k];
        a[i*n+k]=a[i*n+p[k]];
        a[i*n+p[k]]=h;
      }
    }
  }
  delete p;
  return 0;
}


///////////////////////////////////  OLD RING FIT ////////////////////////////////////////

int Ring::calcring(Hits& hh) {

#ifdef RISE_TIMING
  RICH_SearchTiming->startFitTiming();
#endif

  // -- set initial parameters
  register float xx0=ix0;
  register float yy0=iy0;
  if (RingSearchTrack::exist && RingSearchAttrib::use_seed_phi) 
    xx0=x0=RingSearchTrack::phi;
  else
    xx0=x0=ix0;

  if (RingSearchTrack::exist && RingSearchAttrib::use_seed_lambda)
    yy0=y0=RingSearchTrack::lambda;
  else 
    yy0=y0=iy0;

  // -- check if close to original track
  if (!IsCloseToSeed()) return 0;


  // -- resel local assignments
  memset(&assigned[0],0,sizeof(bool)*MAXHITSPERRING);

  float *dx=new float[hh.numhit];
  float *dy=new float[hh.numhit];
  float *ri=new float[hh.numhit];

  float *hitx = new float[hh.numhit];
  float *hity = new float[hh.numhit];

  sigma=0.7e-3/scaling;
  register float sigcut=(8*sigma)*(8*sigma);
  register float sigmin=(3*sigma)*(3*sigma);
  float dx0,dy0,rr;
  niter=0;

  // -- setup correction
  if (RingSearchTrack::exist) {
    RICH_LambdaPhi->Correct()->newRing(xx0*scaling, yy0*scaling, radius*scaling,RingSearchTrack::bx,RingSearchTrack::by);
  }
  else
    RICH_LambdaPhi->Correct()->newRing(xx0*scaling, yy0*scaling, radius*scaling);


  // -- fill hit container with ROI hits
  // -- correct hits using present x0,y0,r0
  // -- calculate initial radius
  numhit=0;
  register float r=0;
  int i;
  for (i=0;i<hh.numhit;i++) {
    Hit& h=hh.hit[i];

    //if (h.assigned) continue;////////////////////////////////////

    register float ddx = - RICH_LambdaPhi->Correct()->dPhi   ( h.x*scaling, h.y*scaling, h.plane)/scaling;
    register float ddy = - RICH_LambdaPhi->Correct()->dLambda( h.x*scaling, h.y*scaling, h.plane)/scaling;

    register float rri = sqrt(ddx*ddx+ddy*ddy);
    float a=(rri-radius);

    if (a*a>sigcut) continue;
    dx[numhit]=ddx/rri;
    dy[numhit]=ddy/rri;
    ri[numhit]=rri;
    r+=rri;

    hitx[numhit]=xx0-ddx;
    hity[numhit]=yy0-ddy;

    hit[numhit++]=&h;
  }
  if (!numhit) return 0;
  radius=r/numhit;


  // --- recalculate RADIUS, and CENTER
  chi2=0;
  int n=0;
  do {
         
    // --- better center
    float a11,a12,a22,b1,b2,c;
    a11=a12=a22=b1=b2=c=0;
    int i;
    for (i=0;i<numhit;i++) {
      register float dr=(ri[i]-radius);
      register float a=dr*dr;
      if (a>sigcut) continue;

      register float dfdx0=dx[i];
      register float dfdy0=dy[i];
      
      a11+=dfdx0*dfdx0;
      a22+=dfdy0*dfdy0;
      a12+=dfdx0*dfdy0;

      b1+=dfdx0*dr;
      b2+=dfdy0*dr;
    
      n++;
    }

    float deta=a11*a22-2*a12;
    dx0=-( a22*b1-a12*b2)/deta;
    dy0=-(-a12*b1+a11*b2)/deta;


    xx0+=dx0;  // better center
    yy0+=dy0;  // better center

    if (sigcut>sigmin) sigcut/=4;


    // --- better radius
    r=0;
    n=0;
    for (i=0;i<numhit;i++) {

      ri[i]+=dx0*dx[i]+dy0*dy[i];
      dx[i]=(xx0-hitx[i])/ri[i];
      dy[i]=(yy0-hity[i])/ri[i];

      register float dr=(ri[i]-radius);

      register float a=dr*dr;
      if (a>sigcut) continue;
      r+=ri[i];
      n++;
    }
    rr=radius;
    radius=r/n; // better radius

    niter++;

    if (niter==15) break;
  } while (fabs(dx0)>1e-5/scaling||fabs(dy0)>1e-5/scaling||fabs(rr-radius)>1e-5/scaling);

  x0=xx0;
  y0=yy0;


  // -- check S/sqrt(N)
  if (!checkSN(hh)) {
    delete ri;
    delete dy;
    delete dx;
    delete hitx;
    delete hity;
    return 0;
  }

  // --- calculate COVARIANCE MATRIX
  memset(&cov[0][0],0x0,9*sizeof(float));

  register float isig=1/sigma;
  chi2=0;
  int k=0;
  for (i=0;i<numhit;i++) {

    register double ddx=dx[i];
    register double ddy=dy[i];
    register double r=ri[i];
    double d=(r-radius);
    if (d*d>sigcut) continue;
    d*=isig;

    if (fabs(d)<3) {

      double d2=d*d;
      chi2+=d2;

      double dddx= isig*ddx;
      double dddy= isig*ddy;
      double dddr=-isig;

      cov[0][0]+=(dddx*dddx);
      cov[1][1]+=(dddy*dddy);
      cov[0][1]+=(dddy*dddx);
      cov[0][2]+=(dddx*dddr);
      cov[1][2]+=(dddy*dddr);
      cov[2][2]+=(dddr*dddr);

      if (refit_sn>RingSearchAttrib::SNcut) 
	hit[i]->assigned=1; // global
      assigned[i]=true;   // local
      k++;
    }
  }

  cov[1][0]=cov[0][1];
  cov[2][0]=cov[0][2];
  cov[2][1]=cov[1][2];
  
  invert(&cov[0][0],3);

  //fittype = (RingSearchTrack::exist) ? RingSearchType::seed : RingSearchType::standalone;
  //fittype = (fittype|RingSearchType::nobg);

  //cout<<"final="<<ix0<<"->"<<x0<<" , "<<iy0<<"->"<<y0<<endl;
  //cout<<"final         :  "<<x0*scaling<<" "<<y0*scaling<<" "<<radius*scaling<<"  it="<<niter<<endl;

  numhit=k;
  delete ri;
  delete dy;
  delete dx;
  delete hitx;
  delete hity;

#ifdef RISE_TIMING
  RICH_SearchTiming->stopFitTiming();
#endif

  //////////////////////////fitring(hh);

  return 1;
}



int Ring::checkSN(Hits &hh) {
  int verb=0;

  if (RingSearchTrack::exist) {
    RICH_LambdaPhi->Correct()->newRing(x0*scaling, y0*scaling, radius*scaling,RingSearchTrack::bx,RingSearchTrack::by);
  }
  else
    RICH_LambdaPhi->Correct()->newRing(x0*scaling, y0*scaling, radius*scaling);

  if (verb) cout<<"check SN: "<<x0<<" "<<y0<<" "<<radius<<endl;

  static float sigmaInv=1/(0.7*1e-3); // photon sigma = 0.7 mrad
  static float hiSignal=3;            // signal range
  static float loSignal=-3;           // signal range
  static float hiRoi=10;              // ROI
  static float loRoi=-10;             // ROI

  const int   nbins=24;        // bins in radius histogram
  float histo[nbins];          // radius histogram
  memset(&histo[0],0x0,nbins*sizeof(float));
  float delta_ddr=(hiSignal-loSignal)/float(nbins);
  float inv_delta_ddr=1/delta_ddr;

  const int   max_phot=300;
  static float ddr_list[max_phot];    // list of photons inside signal region
  int   n_ddr_list=0;                 // number of photons inside signal region
  memset(&ddr_list[0],0x0,max_phot*sizeof(float));

  float sSum=0;
  float bgSum=0;
  float low=0;
  for (int i=0;i<hh.numhit;i++) {
    Hit& h=hh.hit[i];

    register float ddx=RICH_LambdaPhi->Correct()->dPhi   (h.x*scaling,h.y*scaling,h.plane);
    register float ddy=RICH_LambdaPhi->Correct()->dLambda(h.y*scaling,h.y*scaling,h.plane);
    register float ddr=(sqrt(ddx*ddx+ddy*ddy)-radius*scaling)*sigmaInv;

    if (ddr<loRoi || ddr>hiRoi) continue;

    if (ddr>loSignal && ddr<hiSignal) {
      int bin=int((ddr-loSignal)*inv_delta_ddr);
      if (bin<0 || bin>=nbins) continue;
      histo[bin]+=1.;
      ddr_list[n_ddr_list]=ddr;
      if (h.plane) low+=1.;
      n_ddr_list++;
      if (n_ddr_list>=max_phot) {
	cout<<" $$$ RICH: Ring::checkSN: number of photons too large"<<endl;
	return 0;
      }
    }
    else bgSum++;
  }
  if (!n_ddr_list) {
    cout<<" $$$ RICH: Ring::checkSN: no photons"<<endl;
    return 0;
  }
  sSum=float(n_ddr_list);

  numhit_3sigma=sSum;

  float bgAverage=bgSum/(hiRoi-loRoi-hiSignal+loSignal);  
  float N=bgAverage*(hiSignal-loSignal);
  float S=sSum-N;
  numhit_refit=S;
  numhit_bgrd=N;
  f=(sSum-2*low)/sSum;
  if (S<=0) {
    //cout<<"$$$ RICH: Ring::checkSN: no signal"<<endl;
    return 0;
  }
  refit_sn=(N>0) ? S/sqrt(N) : 100;

  float dr_sum=0;
  float dr2_sum=0;
  float bgAverage_per_bin=bgAverage*delta_ddr;
  sigma_signal=0;
  for (int i=0; i<n_ddr_list; i++) {
    int bin=int((ddr_list[i]-loSignal)*inv_delta_ddr);
    if (bin<0 || bin>=nbins) continue;
    if (histo[bin]<bgAverage_per_bin) continue; // reject zeros
    float dr=ddr_list[i]*(histo[bin]-bgAverage_per_bin)/histo[bin];
    dr_sum +=dr;
    dr2_sum+=dr*dr;
  }

  sigma_signal=sqrt(dr2_sum/sSum-(dr_sum/sSum)*(dr_sum/sSum))/sigmaInv;

  if (verb) cout<<"S="<<numhit_refit<<"  B="<<numhit_bgrd<<"  f="<<f<<"  sigma_signal="<<sigma_signal<<endl;

  if (!RingSearchAttrib::use_underdogs && refit_sn<RingSearchAttrib::SNcut) return 0;

  return 1;
}
