#include <arte/arte.hh>
#include <riter/RITERMain.hh>
#include <riter/RITERFill.hh>
#include "rise/RICHEventMC.h"  
#include "rich/ricutil.h"
#include "reco/rcflags.hh"
#include "arte/RSEG_cmp.hh"

#include "dq/DQ.h"
#include "dq/DQ_RICH.h"
#include "rhp.h"
#include "riter/RITERhbook.hh"                                                       
extern "C" void atstopcc_ (char *t); 


int RITER_neve=0;
int  riteralltracks    =0;


#ifdef RITER_DEBUG
#include <myhisto.hh>
#endif


float GetGlobalAlignmentCorrection(int i){

  for( ArteTable<ACGA>::iterator acga = ArteTable<ACGA>::begin();
        acga != ArteTable<ACGA>::end(); ++acga ) {
    if ((*acga)->cmp == Rsegc::rich ){
      if (i)
	return (*acga)->dty;
      else
	return (*acga)->dtx;
    };
  };
  return 0;
}

// ---------------------------------------------------------------------------
int  Riter::AddPhoton(ArtePointer<HITB> hitb,int &nhitb){ 
  int type =  ( hitb->chid == 4 );
  if ( hitb->chid == 0 ) {
  
    int id=(*hitb->digb.begin())->id;
    int ch =  id%100; id=id/100;
    int col=  id%100; id=id/100;
    int row=  id%100;
    int  sm=  id/100;
    type    = RITER_geo->Type(sm,row,col,ch);
  }

  float sig2  = RITER_geo->Sigma2(type); 
  RITERVec rf = 10. * RITERVec(hitb->x,hitb->y,hitb->z);

  int dn = ( hitb->y<0 );

  rf  =  virtual_position(rf, RITER_geo->Plr0(dn), RITER_geo->Plvn(dn));
 
  RITERHits hit(rf,sig2,1,type);
  fotons.push_back( hit );
  nhitb++;
  return 0;                
};
// ---------------------------------------------------------------------------
int  Riter::Fillfotons_HITB(){
  int nhitb=0;
  if (recoRICH) {
    int nphot=  RichRun::gede->hitl - RichRun::gede->hit1; 
    if (nphot>0) fotons.reserve( nphot );
    // fill photon structure ****************************************

    for (ArteTable<HITB>::iterator hitbit=RichRun::gede->hit1;
	 hitbit!=RichRun::gede->hitl; ++hitbit) {                              
      AddPhoton((*hitbit),nhitb); 
    }   
  } else {
    for(ArteTable<HITB>::iterator hitbit=ArteTable<HITB>::begin();
	hitbit!=ArteTable<HITB>::end();++hitbit) {
      if((*hitbit)->gede->cmp!=5) continue;     
      AddPhoton((*hitbit),nhitb); 
    }     
  }
  return nhitb;
}  
// -------------------------------------------------------------------------

int  Riter::Filltracks_MIMPdenis(int &nhitb){
  int itra=0;
  nhitb=0;
  // DENIS's function
  RICHrings *rich=new RICHrings;
  // loop over RICH MC particles (= at least one photon is radiated)
  
  
  for ( rich->begin(); rich->valid(); rich->next() ) {
    float tx=rich->tx();
    float ty=rich->ty();    
    float x=rich->x().x();
    float y=rich->x().y();
    float z=rich->x().z();
    cout << "vsi tracki " << RITERVec(x,y,z) <<endl;

    RITERVec r=toRITERVec(0.1*rich->x());
    cout << "*vsi tracki " << r  << endl;

    RITERVec mver=10* RITERVec(rich->vertex()->x,
				   rich->vertex()->y,
				   rich->vertex()->z);
    int lund=rich->topp()->lund;
    //     if (lund !=11) continue; // select only electrons
    if (lund !=211) continue; // select only pions

    if (rich->momentum()<15) continue; // cut  15 Gev
    cout<< "lund:"<< lund<< r<< mver <<  endl;

    if  (mver.z()> RITER_geo->z_rich_min) continue; // particle should be 
    RITERVec dir = RITERVec(tx,ty,1).unit();
    r              = toRICH(r,dir);    
    float momentum = min( float(1000.) ,rich->momentum() ) ; 
    int   itype    = itip_lund(lund);
    float sig      = 0;      

    RITERTracks Track(r,dir,momentum,sig,sig,itype); 
    tracks.push_back(Track);
    itra++;
    RICHphotons *ph=rich->GetPhotons();
      
    for (ph->begin(); ph->valid(); ph->next()) { 
      if (rich->mtra()==ph->mtra()) {
	ArteRelationIterator<HITB>::iterator ihitb= ph->mimp()->hitb.begin();
	ArtePointer<HITB> hitb(ihitb);
	AddPhoton(hitb,nhitb); 
      } 
    }
    
    break; 
  }
  if (rich) delete rich;  
  cout <<    itra << "MIMP  =>" << endl;
  return itra;
}

// -------------------------------------------------------------------------
int Riter::Filltracks_MIMP(ArtePointer<MTRA> aprtra,int &itra)
{
  register float zmin= 860;  
  static float dtyGlobal =  GetGlobalAlignmentCorrection(1);
  static float dtxGlobal =  GetGlobalAlignmentCorrection(0);

  //look for MIMP closer to the begin of RICH
  ArtePointer<MIMP> closerpnt;
  for(ArteRelationIterator<MIMP> rpntit=aprtra->mimp.begin();
      rpntit!=aprtra->mimp.end();rpntit++) {
    if( ((*rpntit)->z > closerpnt->z) && 
	((*rpntit)->z < zmin) )	closerpnt=*rpntit;
  }
  RITERVec point(closerpnt->x,closerpnt->y,closerpnt->z);
  
  float txf= closerpnt->tx - dtxGlobal;
  float tyf= closerpnt->ty - dtyGlobal;
  
  if((closerpnt->z>300)&& IsInside( txf , tyf ) ){
    
    if (closerpnt->p==0)  return 0;
    float momentum = min(float(1000), fabs(closerpnt->p)); 
    if ( momentum < 1  )  return 0;
    float sigma2tx   = 1e-8;
    float sigma2ty   = 1e-8;

    RITERVec  dir = RITERVec(txf,tyf,1).unit();
    point=toRICH(point,dir);

    int lund    =  itip_lund(aprtra->topp->lund);
    RITERTracks Track(point,dir,momentum,sigma2tx,sigma2ty,lund);
    if (Track.stf[1]==0) return 0;
 
    tracks.push_back(Track);
    
    itra++;
    return 1; 
  }
  return 0;
}

// -------------------------------------------------------------------------
int  Riter::Filltracks_MTRA(){
  int itra=0;
  // fill track structure **************************************
  int alltracks=0;
  for(ArteTable<MTRA>::iterator rtra1it=ArteTable<MTRA>::begin();
      rtra1it!=ArteTable<MTRA>::end();++rtra1it) {
if (!(*rtra1it)->topp->chrg) continue;
    Filltracks_MIMP((*rtra1it),itra);
    alltracks++;
    if ( itra >RITERntra ) return itra; 
    // ************************************************************
  }  
  return itra;
}



//-----------------------------------------------------------
int Riter::Filltracks_RPNT2(ArtePointer<RTRA> aprtra,int &itra)
{

  register float zmin= 1001;  // midddle of the RICH  
  static float dtyGlobal =  GetGlobalAlignmentCorrection(1);
  static float dtxGlobal =  GetGlobalAlignmentCorrection(0);

  //look for RPNT closest to the middle of RICH
  ArtePointer<RPNT> closerpnt;

  //int nrpnt=0;
  for(ArteRelationIterator<RPNT> rpntit=aprtra->rpnt.begin();
      rpntit!=aprtra->rpnt.end();rpntit++) {
    //nrpnt++;
    if( ((*rpntit)->zf > closerpnt->zf) && 
	((*rpntit)->zf < zmin) )	closerpnt=*rpntit;
  }

  RITERVec point(closerpnt->xf,closerpnt->yf,closerpnt->zf);

  //  cout << nrpnt <<"rpnt2 " << point << endl;
  
  float txf= closerpnt->txf - dtxGlobal;
  float tyf= closerpnt->tyf - dtyGlobal;

  if((closerpnt->zf>500)&& IsInside( txf , tyf ) ){
    
    if (closerpnt->pf==0)  return 0;
    float momentum =min( float(1000) ,fabs(1/closerpnt->pf)); 

#ifdef RITER_DEBUG
    htrack_sigma_x->Fill(closerpnt->cvf[5],1);
    htrack_sigma_y->Fill(closerpnt->cvf[9],1);
    htrack_sigma_rtra_x->Fill(aprtra->cvf[5],1);
    htrack_sigma_rtra_y->Fill(aprtra->cvf[9],1);
#endif 
    
    float sigma2tx   = closerpnt->cvf[5];
    float sigma2ty   = closerpnt->cvf[9]; 
    //if ( sigma > 0.005 ) return 0;

    RITERVec  dir = RITERVec(txf,tyf,1).unit();
    //#warning zamenjani indeksi v RITERVec  dir
    //RITERVec  dir = RITERVec(tyf,txf,1).unit();
    point=toRICH(point,dir);

    ArtePointer<MTRA> imtra(aprtra->mtra.begin());
    int lund    =  itip_lund(imtra->topp->lund);
    RITERTracks Track(point,dir,momentum,sigma2tx,sigma2ty,lund,aprtra);
    if (Track.stf[1]==0) return 0;
 
    tracks.push_back(Track);
    
    itra++;
    return 1; 
  }
  return 0;
}

// -------------------------------------------------------------------------

int  Riter::Filltracks_RPNT(){

  if (!ArteTable<RPNT>::size()) return 0;

  int itra=0;
  // fill track structure **************************************
  int alltracks=0;
  for(ArteTable<RTRA>::iterator rtra1it=ArteTable<RTRA>::begin();
      rtra1it!=ArteTable<RTRA>::end();++rtra1it) {
    if ( (*rtra1it)->fit ) continue;      
    Filltracks_RPNT2((*rtra1it),itra);
    alltracks++;
    if ( itra >RITERntra ) return itra; 
    // ************************************************************
  }  
  return itra;
}



//-----------------------------------------------------------
int  Riter::Filltracks_RSEG(){
  int itra=0;
  
  // fill track structure **************************************
  for(ArteTable<RSEG>::iterator rtra1it=ArteTable<RSEG>::begin();
      rtra1it!=ArteTable<RSEG>::end();++rtra1it) {
    
    ArtePointer<RSEG> rtra(rtra1it);
      // ************************************************************
      // track selection is done here
      // ************************************************************
      // before RICH

      int pc=(rtra->cmp & ((Rsegc::wch )|(Rsegc::patt)));
      // between RICH and ECAL
      //      int tc=(rtra->cmp & ((Rsegc::wch )|(Rsegc::trit)));
      if (pc) {
	if  (fabs(rtra->txf)> RITER_geo->th_0) continue;
	if  (fabs(rtra->tyf)> RITER_geo->fi_0) continue;
	RITERVec drx = RITERVec(rtra->txf,rtra->tyf,1).unit(); 
	float ycoor=rtra->yf;
        float p = Momentum(rtra); 
	float sigma2tx   = rtra->cvf[5];
	float sigma2ty   = rtra->cvf[9]; 
	
        ArtePointer<RCCL> rccl=Find_ecal(rtra);
	if (rccl->z>100) { 
  	  drx= RITERVec(rtra->txf,rccl->y/rccl->z,1).unit();
  	  ycoor=rccl->y*rtra->zf/rccl->z;
	  p=Momentum(rtra->xf,rtra->zf,rtra->txf,rccl->y/rccl->z);
	  //if (rccl->esum > (0.7*p)) track[itra].itype = 1;  // e- ali hadron
	  sigma2ty= rccl->cve[2]/rccl->z/rccl->z;
	} else continue;       
	RITERVec r    = toRICH(rtra->xf,ycoor,rtra->zf,drx);	
        float momentum=0;
	if (rtra->pf!=0)     momentum=min(1000,fabs(1/rtra->pf)); 

	RITERTracks Track(r,drx,momentum, sigma2tx,sigma2ty,(rccl->esum > (0.7*p))); 
	tracks.push_back(Track);

#ifdef RITER_DEBUG
        hep->Fill(rccl->esum,Track.p,1);  
#endif
	itra++;
      }
      // ************************************************************
  
  }
#ifdef RITER_DEBUG
  cout <<    itra << "segments found  =>" << endl;
#endif
  return itra;
}

// ------------------------------------------------
void Riter::FillRTRA(int mode=0) {

  for (Rvector<RITERTracks>::iterator it=tracks.begin();
       it!=tracks.end();++it) {
    float sum=0;
    float max=(*it).plkh[0];
    int itip=0;
    for (int i=0;i<6;i++) if ( (*it).plkh[i] > max ) {
      max=(*it).plkh[i];
      itip=i;
    }
    for (int i=0;i<6;i++) (*it).plkh[i]=exp((*it).plkh[i]-max);
    for (int i=0;i<6;i++) sum+=(*it).plkh[i];
    if (sum!=0) sum=1/sum; 
    
    if (RITERhisto) {
      float   bgr          =   (*it).Likelihood()->BackgroundPhotons();
      float   sfot         =   (*it).Likelihood()->SignalPhotons();
      rhp_hfill(DQ_ID(COMP_RICH, LVL_REC, 232),bgr,sfot,1);
    }


    switch (mode) {
    case 0:
    case 1:
      (*it).rtra->lre =(*it).plkh[1]*sum;
      (*it).rtra->lrmu=(*it).plkh[2]*sum;
      (*it).rtra->lrpi=(*it).plkh[3]*sum;
      (*it).rtra->lrk =(*it).plkh[4]*sum;
      (*it).rtra->lrp =(*it).plkh[5]*sum;
      (*it).rtra->hri= int( (*it).sfot  );
      (*it).rtra->flr= 1;
      break;
    case 10:
      (*it).rtra->lrpi=(*it).plkh[1]*sum;
      (*it).rtra->lrk =(*it).plkh[3]*sum;
      (*it).rtra->lrp =((*it).plkh[2]+(*it).plkh[4]+(*it).plkh[5])*sum;
      break;

    case 11:
      (*it).rtra->lre =(*it).plkh[1]*sum;
      (*it).rtra->lrmu=(*it).plkh[2]*sum;
      (*it).rtra->lrpi=(*it).plkh[3]*sum;
      (*it).rtra->lrk =(*it).plkh[4]*sum;
      (*it).rtra->lrp =(*it).plkh[5]*sum;
      (*it).rtra->hri= int( (*it).sfot );
      (*it).rtra->flr= 1;
      (*it).rtra->flt= (*it).itype;
      break;

    }
  }
}




// --------------------------------------------------------------------------
void Riter::FillRSEG(int mode) {
  if (mode >1) return;
  for (Rvector<RITERTracks>::iterator it=tracks.begin();
       it!=tracks.end();++it) {


    //    if ( mode ) createrelation;

  }
}




// --------------------------------------------------------------------------
int  Riter::Riter_evnt(){

  int itra=0;
  int nhitb=0;

  tracks.reserve(1000);
  fotons.reserve(15000);
 
  switch (mode) {
  case 0: itra=Filltracks_RPNT();
    break;
  case 2: itra=Filltracks_RSEG();
    break;
  case 3: itra=Filltracks_MIMPdenis(nhitb);
    break;
  case 4: itra=Filltracks_MTRA();
    break;
  }

  if (!itra) return RITER_OK;

  if (itra<RITERntra) {
    if (mode!=3) nhitb=Fillfotons_HITB();
    
    riteralltracks+=itra;

    //fprintf(stderr, "RITER :%d  Nphotons :%d  Nalltracks :%d\n",
    //itra,nhitb , riteralltracks); 
    
    
    if (itra) {
      main();   
      FillRTRA(mode);
      //FillRTRA(11);
    }
  }


  Reset();


#ifdef RITER_DEBUG
  FillEvent();
#endif 

  // printf("RITER CLEARED tracks %d photons %d\n" ,
  //	 tracks.capacity(), fotons.capacity());
  if (!tracks.empty()) 
    atstopcc_("RITERTrack:: not all memory is deallocated"); 
  if (!fotons.empty())  
    atstopcc_("RITERHit  :: not all memory is deallocated"); 
  return RITER_OK;
};
// ------------------------------------------------
// adds track to the track structure, 
// on input : rtra + 
//            direction dir at point point after the magnet + 
//            momentum  should be supplied 
//            err sigma^2 dirtrack
int  Riter::AddTrack(ArtePointer<RTRA> rtra,
		     RITERVec &dir  ,
		     float err        ,
		     RITERVec &point, 
		     float p           ){  
  if((point.z()>300)&& IsInside(dir.x(),dir.y() ) )
{
    RITERVec r =toRICH(point,dir);
    RITERTracks Track(r,dir,p,err,err,0,rtra);
    tracks.push_back(Track);
    return 1; 
  }
  return 0;
}
// ------------------------------------------------

int  Riter::ResetTracks(){
  Reset();
  return RITER_OK;
}
// ------------------------------------------------
int  Riter::Iterate(int mode){

  int nhitb = Fillfotons_HITB();
  if ( nhitb && tracks.size() ) main();
  return RITER_OK;
}
// ------------------------------------------------

int  Riter::CherenkovAngle(int nbins, float min, float max, float *xarray){
  if (!nbins   ) return 1;
  if (max < min) return 0;

 
  if ( !Filltracks_RPNT() ) return 0;

  if ( !Fillfotons_HITB() ) return 0;

  int bin=0;
  float th_cer=0,fi_cer,sig;
  int sf=0,pl=0,flag=0;
  float dx=(max - min) /nbins;

  for(Rvector<RITERTracks>::iterator it=tracks.begin();
      it!=tracks.end();++it){    
   
    for(Rvector<RITERHits>::iterator hit=fotons.begin();
	hit!=fotons.end();++hit){
      
      if (ThetaObj->CalculateFull((*hit),(*it),th_cer,fi_cer,flag,sig)!=RITER_OK) 	continue;
     
      if ((th_cer>=max) || (th_cer<min)) continue;
      bin= (int)((th_cer-min) / dx);
      xarray[bin]++;
    }
  }
  
  Reset();
  return nbins;
}

// ------------------------------------------------

Riter *reco_Riter;

int RITERhbook();

extern "C" {
  void Riter_init_(){

  }
 
 
  void Riter_evnt_(){
#ifdef RITER_DEBUG
    cout << "Riter_evnt_ \n" ;
#endif
     int nphot =  RichRun::gede->hitl - RichRun::gede->hit1;
     if (!nphot) return;
     if  (!reco_Riter )  {

       static int firsttime=1;
       if (firsttime) {         
	 printf("############################################\n") ;
	 printf("RITER Initialization: Calling  initrich_(); \n") ;
	 initrich_();
       }	 

       reco_Riter=new Riter();

       if (firsttime) {
         if (!reco_Riter) atstopcc_("RITER not initialized"); 
	 printf("RITER Initialization: Finished \n") ;
	 printf("############################################\n") ;
         firsttime = 0;
       }

     } 
     reco_Riter->Riter_evnt();
     delete reco_Riter;
     reco_Riter = NULL; 

     RITER_neve++;

  }
  
  void Riter_stop_(){

   if  ( reco_Riter )     delete reco_Riter;
   if  ( RITER_Geometry ) delete RITER_Geometry; 
   reco_Riter    = NULL;
   RITER_Geometry= NULL; 

  }


}
  
	   




