#ifndef _RVECTOR_H_
#define _RVECTOR_H_

#include <stdlib.h>
#include <string.h>
#include <iostream>

#define BlockMove( a, b, c) memcpy( b , a , c) 
#define memFullErr 1
#define memAdrErr  2
#define  DEFAULT_BLOCK_SIZE 100

typedef int Rint; 

template <class T>
class Rvector {
  private:
  Rint fsize     ;
  Rint fmax_size ;
  Rint fblock    ;

  public:


  T *  fbuf; 
  typedef T* iterator;  
  inline iterator& operator++() {  return (*++this);};		// get item reference by index

  inline int        max_size(){ return fmax_size ;     };
  inline iterator   begin()   { return fbuf;           };
  inline iterator   end()     { return (fbuf + fsize); };
  inline bool       empty()   { return (fsize==0);     };
  inline int        size()    { return  fsize;         };
  inline int        capacity(){ return  fmax_size;     };
  inline int        stepsize(){ return  fblock;        };
  inline void       reserve(const int  n){ if ( fmax_size < n ) resize(n) ; }; 
  inline void       clear(){
    delete [] fbuf; 
    fbuf  = NULL; 
    fsize = 0 ;
    fmax_size = 0;
  };
  inline void       setincr(const int i){    fblock=i; };
	
  // assignment-op
  inline Rvector& operator=(const Rvector<T>&);	// (copies data)
  
  inline Rvector();			 // sets size to 0
  inline Rvector(const int n);		 // allocates n slots
  inline Rvector(const Rvector<T>&);	 // copy-constructor (copies data)


  inline ~Rvector();
  inline void push_back(const T& item);	    // push onto end of the vector
  inline T pop_back();			    // get last item, remove from  
  
  // treating it like an array
  inline T& operator[](int idx);	    // get item reference by index
  //  inline T& operator[](const int idx);  // get item reference by index
  inline T& operator[](const int idx) const;
  inline void resize(const int n);
  
};


template <class T>
inline Rvector<T>::Rvector()
: fsize(0), fmax_size(0), fblock( DEFAULT_BLOCK_SIZE ), fbuf(0)
{
}

template <class T>
inline Rvector<T>::Rvector(const int n)
:  fsize(0), fmax_size(n), fblock(DEFAULT_BLOCK_SIZE ), fbuf(0)
{
	fbuf = new T[fsize];
	if (!fbuf) {
	  throw memFullErr;
	}
}

template <class T>
inline Rvector<T>::Rvector(const Rvector<T>& vec)
: fsize(0), fbuf(0)
{
	*this = vec;
}
	
template <class T>
inline Rvector<T>& Rvector<T>::operator=(const Rvector<T>& vec)
{
	if (fbuf) delete[] fbuf;
	fsize     = vec.size();
	fmax_size = vec.capacity();
	fblock    = vec.stepsize();
	fbuf      = new T[fmax_size];
	if (!fbuf) {
		throw memFullErr;
	}
	BlockMove( vec.fbuf, fbuf, fmax_size*sizeof(T) );
	return *this;
}

template <class T>
inline Rvector<T>::~Rvector()
{
	if (fbuf) delete[] fbuf;
        fbuf = NULL; 
}

template <class T>
inline T&  Rvector<T>::operator[](int idx) const
{
	if (idx < 0 || idx >= fsize) throw memAdrErr;
	return fbuf[idx];
}

template <class T>
inline T&  Rvector<T>::operator[](int idx) 
{
	if (idx < 0 || idx >= fsize) throw memAdrErr;
	return fbuf[idx];
}



template <class T>
inline void Rvector<T>::push_back(const T& item)
{
	// do we need to increase the buffer size?
	while (fsize >= fmax_size) {
		// yes -- expand it by one block 
		resize(fmax_size +  fblock );
	}
	// stuff the item
	fbuf[fsize] = item;
	fsize++;
}

template <class T>
inline T Rvector<T>::pop_back()
{
	if ( fsize > fmax_size || fsize <= 0) throw memAdrErr;
	return fbuf[--fsize];
}

template <class T>
inline void Rvector<T>::resize(const int n)
{
	if (n == fmax_size) return;

	T *newbuf = new T[n];
	if (!newbuf) throw memFullErr;

	if (fbuf) {

	  BlockMove(	fbuf,  newbuf, sizeof(T) * fmax_size );
	  delete[] fbuf;
	}

	fbuf = newbuf;
	fmax_size = n;
}


template <class T>
inline Rvector<T>::iterator operator++ ( Rvector<T>::iterator &x) {
  return Rvector<T>::iterator (x++);
};   

template <class T>
inline bool operator!= ( Rvector<T>::iterator &a, Rvector<T>::iterator &b) {
  return (a!=b);
};



#ifdef RVECTOR_MAIN

/*
Test program: 
Copy the header file in .C file and compile with 

 g++ -DRVECTOR_MAIN  Rvector.C -o Rvector

*/

 
template <class T>
inline std::ostream & operator << (std::ostream &out ,T& x) {
  return out << "[" << x[0] << "]";
};     


class mycl {
public:
  int a;
  int b[5];


  mycl(){};
  mycl(const int i){ a=i; };
  ~mycl(){};
};

inline std::ostream & operator << (std::ostream &out ,mycl& x) {
  return out << x.a ;
};     

int main(){

  mycl  mychar;
  Rvector<mycl> vec;               // create an empty vector
  vec.setincr(1000);               // allocate in blocks of 5 items

 for (int i=0;i<11000;i++) {
   vec.push_back(mycl(i+10));
 }

 mychar = vec[0];                    // get some items
 cout << mychar << endl;
 mychar = vec.pop_back();            // get last item and remove it
 cout << mychar << endl;

 int j=0;
 for ( Rvector<mycl>::iterator i=vec.begin(); 
       i!=vec.end(); 
       i++) { 
   // cout << i << "   :"  << vec.begin() << " " << vec.size()  << endl;
   if ( (j++%1000)==0 ) cout << i << (*i)<< ":" << " " << vec.size()  << endl;
 }
 vec.clear();             // empty the list
 
 return 0;
}



#endif
#endif






