// *****************************************************************************
//
// source:    L3_State.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include "L3_State.hh"

L3_State::L3_State() {
  TotalNumberOfReconstructedSpacepoints = 0;
  TotalNumberOfReconstructedTracks = 0;
  NumberOfEventsProcessed =0;
  NumberOfEventsAccepted =0;
  dt_PointReco = 0.0;
  dt_TrackReco = 0.0;
  dt_VertexReco = 0.0;
  dt_Trigger = 0.0;
}

L3_State::L3_State(const L3_State& a) { operator=(a); }

L3_State& L3_State::operator = (const L3_State& a) {
  TotalNumberOfReconstructedSpacepoints = a.TotalNumberOfReconstructedSpacepoints;
  TotalNumberOfReconstructedTracks = a.TotalNumberOfReconstructedTracks;
  NumberOfEventsProcessed = a.NumberOfEventsProcessed;
  NumberOfEventsAccepted = a.NumberOfEventsAccepted;
  dt_PointReco = a.dt_PointReco;
  dt_TrackReco = a.dt_TrackReco;
  dt_VertexReco = a.dt_VertexReco;
  dt_Trigger = a.dt_Trigger;
  return *this;
}

void L3_State::update(const L3_Event& Ev) {
  NumberOfEventsProcessed++;
  NumberOfEventsAccepted++;
  TotalNumberOfReconstructedSpacepoints  +=  Ev.silicon().nPoints();
  TotalNumberOfReconstructedTracks       +=  Ev.silicon().nTracks();
}

int L3_State::nEvents() const {return NumberOfEventsProcessed;}
int L3_State::nEventsAccepted() const {return NumberOfEventsAccepted;}
int L3_State::nSpacepoints() const {return TotalNumberOfReconstructedSpacepoints;}
int L3_State::nTracks() const {return TotalNumberOfReconstructedTracks;}

float L3_State::dtPointReco() const {return dt_PointReco;}
float L3_State::dtTrackReco() const {return dt_TrackReco;}
float L3_State::dtVertexReco() const {return dt_VertexReco;}
float L3_State::dtTrigger()   const {return dt_Trigger;}

void L3_State::read(const L3_Timer& Timer) {
  dt_PointReco += Timer.lastPointTime();
  dt_TrackReco += Timer.lastTrackTime();
  dt_VertexReco += Timer.lastVertexTime();
  dt_Trigger   += Timer.lastTriggerTime();
}

#include <iostream>

void L3_State::print() const {
  cout << "TLT_Event_Nr: " << nEvents();
}

#ifdef L3_ARTE_FRAME
#include "arte/arte.hh"
#endif

void L3_State::printEVHD() const {
#ifdef L3_ARTE_FRAME
  ArtePointer<EVHD> evhd;
  cout << "     ||EVHD:  Exp: " << evhd->exp
       << "   Run: " << evhd->run
       << "   Evt: " << evhd->evt;
#endif
  cout  << endl;
  cout << "*******************************************************************"
       << endl;
}

void L3_State::printSummary() const {
  cout << endl;
  cout << ">>>>>>>>>>>>>>> Summary of L3_Trigger <<<<<<<<<<<<<<<" << endl;
  cout << " Processed events          = " <<  nEvents() << endl;
  cout << endl;
  cout << " Reconstructed Spacepoints = " <<  nSpacepoints() << endl;
  cout << " Reconstructed Tracks      = " <<  nTracks() << endl;
  cout << endl;
  cout << " Total time: Spacepoints = "  <<  dtPointReco() << endl;
  cout << " Total time: Tracking    = "  <<  dtTrackReco() << endl;
  cout << " Total time: Vertexing   = "  <<  dtVertexReco() << endl;
  cout << " Total time: L3_Trigger  = "  <<  dtTrigger() << endl;
  cout << ">>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<" << endl;
}


