#ifndef _L3_VERTEX_HH_
#define _L3_VERTEX_HH_
// *****************************************************************************
//
// source:    L3_Vertex.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************

#include "L3_Defines.hh"

#include <iostream>
#include <cmath>
#include <vector>

#include <CLHEP/Vector/ThreeVector.h>

#include "L3_Track.hh"
#include "L3_Vtx.hh"
#include "L3_CovMatrix.hh"

class L3_Vertex {
public:
  inline L3_Vertex();
  L3_Vertex(const L3_Vtx&);
  L3_Vertex(const L3_Vertex&);
  L3_Vertex(const L3_Track&, const L3_Track&);
  L3_Vertex& operator = (const L3_Vertex&);
  inline int operator == (const L3_Vertex&) const;
  inline int operator != (const L3_Vertex&) const;
  inline void add(const vector<L3_Track>&);
  inline Hep3Vector vertex() const;
  inline float x() const;
  inline float y() const;
  inline float z() const;
  inline float dx() const;
  inline float dy() const;
  inline float dz() const;
  inline float chisqr() const;  
  inline vector<L3_Track> OutgoingTracks() const;
  inline int nOutgoingTracks() const;
  inline vector<int> OutgoingTracksNr() const;
  inline L3_CovMatrix<3> getCov() const;
  inline double Cov(int,int) const;
private:
  vector<L3_Track> AssociatedTracks;
  Hep3Vector Vertex;
  L3_CovMatrix<3> cov;
  float chiSquare;
  int Status;
  inline void add(const L3_Track&);
  void fit2(const L3_Track&, const L3_Track&);
};

ostream& operator<<(ostream& out,const L3_Vertex&);

#include "L3_Vertex.icc"

#endif
