*CMZ :          11/11/96  18.05.25  by  T. Oest
*-- Author :    T. Oest   28/05/96

      SUBROUTINE RKMAI2(ICOVAR, ZOUT, FSTEP,
     +                  PIN, PREC, POUT, RKD, IERROR)

************************************************************************
*
* Description: main routine for Runge Kuta tracking to z value ZOUT
*
*
* Called from:
*
* External references:
*
* Argument in/out description
* ICOVAR   in     = 0   without covariance matrix
*                 = 1   normal calculation of covariance matrix
* ZOUT     in     z value for output parameters
* FSTEP    in     first step size
*          out    last step size
* PIN(6)   in     input track parameter (z,x,y,tx,ty,Q/p*sign(cos g))
* PREC(4)  in     precision for POUT(2-5) / cm in z
* POUT(6)  out    output track parameters
* RKD(6,6) out    if COVAR=.TRUE., derivatives
*                 d POUT(z,x,y,tx,ty,rho) / d PIN(z,x,y,tx,ty,rho)
* IERROR   out    = 0 ok
*                 = 1 track curls
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"

      INTEGER          ICOVAR, IERROR
      DOUBLE PRECISION PIN(6), POUT(6), PREC(4)
      DOUBLE PRECISION RKD(6,6)
      DOUBLE PRECISION ZOUT, FSTEP, LENGTH, DZ, Z
      LOGICAL          UPDATE
      DOUBLE PRECISION PWIN(6), DELTA(4)
      INTEGER          I, J
      DOUBLE PRECISION FAC, SLEFT, H, HNEW
      LOGICAL          OK
      DOUBLE PRECISION EC
      DOUBLE PRECISION TANMAX, HMIN

#include "arte/Arte.ff"

      DATA   EC     / 2.99792458D-4 /

C TANMAX   : maximum value for Tx, Ty
C HMIN     : minimal step size
      DATA   TANMAX / 1.D10 /, HMIN /1.0D0 /

************************************************************************
* initialization
************************************************************************
      IERROR  = 0
      UPDATE  = .FALSE.
      POUT(6) = PIN(6)

      DO I=1,6
       DO J=1,6
        RKD(I,J) = 0.0D0
       ENDDO
       RKD(I,I) = 1.0D0
       PWIN(I) = PIN(I)
      ENDDO

      DZ      = ZOUT - PIN(1)
      FSTEP = SIGN(MIN(ABS(DZ),ABS(FSTEP)),DZ)
c      print*,' dz, fstep ',dz,fstep

      IF (FSTEP.EQ.0.0) THEN
       DO I=1,6
        POUT(I) = PIN(I)
       ENDDO
       GOTO 900
      ENDIF



      FAC   = PIN(6) * EC


      H      = FSTEP

      SLEFT = DZ

c      print*,''
c      PRINT*,' H  ',H
c      PRINT*,' 1.STEP ZXY',SNGL(PWIN(1)),SNGL(PWIN(2)),SNGL(pwin(3))
c      PRINT*,'     TX TY ',SNGL(PWIN(4)),SNGL(PWIN(5))
************************************************************************
* loop
************************************************************************

 100  CONTINUE


************************************************************************
* check if ZOUT can be reached
************************************************************************

      IF (ABS(PWIN(4)) .GT. TANMAX .OR. ABS(PWIN(5)) .GT. TANMAX) THEN
       IERROR = 1
       GOTO 900
      ENDIF

************************************************************************
* do one step
************************************************************************
      CALL RKSTEP(H, PWIN, FAC, POUT, DELTA)

************************************************************************
* check precision
************************************************************************

      CALL RKNEWH(H, DELTA, PREC, OK, HNEW)
      IF (ABS(HNEW).LT.HMIN) THEN
c       PRINT*,' HMIN', HNEW
       OK   = .TRUE.
       HNEW = SIGN(HMIN, H)
c       PRINT*,'     ', HNEW
      ENDIF

c      if (ok) then
c       PRINT*,' new  STEP ',SNGL(POUT(1)),SNGL(POUT(2)),SNGL(POUT(3))
c       print*,' h,hnew ',h,hnew
c      else
c       PRINT*,' same STEP ',SNGL(POUT(1)),SNGL(POUT(2)),SNGL(POUT(3))
c       print*,' h,hnew ',h,hnew
c      endif



      IF (OK) THEN
       DO I=1,5
        PWIN(I) = POUT(I)
       ENDDO
       IF (ICOVAR.EQ.1) THEN
        CALL RKDV0(UPDATE, H, EC, RKD)
       ELSEIF (ICOVAR.EQ.2) THEN
        CALL RKDV02(UPDATE, H, EC, RKD)
       ENDIF
       SLEFT = ZOUT - PWIN(1)
       UPDATE  = .TRUE.
      ENDIF


      H     = SIGN(MIN(ABS(HNEW),ABS(SLEFT)),SLEFT)
c       print*,' hmin/max ',h
      IF (SLEFT .NE. 0.0) GOTO 100

      FSTEP = HNEW

 900  END

************************************************************************
* End of RKMAIN
************************************************************************
