#ifndef __WAX_RTMANAGER_HH__
#define __WAX_RTMANAGER_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __STRING__
#include <string>
#endif

class waxRTRelation ;

class waxRTManager
{
public:
  //typedefs
  typedef std::vector<waxRTRelation*> RTContainer ;
  typedef std::vector<unsigned char>  GeoMapContainer ;
  typedef unsigned int RTIndex_t ;
  typedef unsigned int GedeIndex_t ;
 
  // constructors, destructor
  static waxRTManager* GetMe() ;
  static void DeleteMe() ;
  ~waxRTManager() ;

  // setters and getters
  static const waxRTManager* GetUncheckedMe() { return fgMe ; }
  static void SetSwitch(int i) { fgSwitch = i ; }
  static void SetConfigDirName(const string& dir) { fgConfigDirName = dir ; }
  RTContainer& GetRTRelations() { return fRTRelations ; }
  const RTContainer& GetRTRelations() const { return fRTRelations ; }
  RTIndex_t GetRTIndexMin() const { return fRTIndexMin ; }

  // other inlines
  waxRTRelation* GetRTRelation(RTIndex_t rti) {
    return fRTRelations[rti-fRTIndexMin] ; }
  
  const waxRTRelation* GetRTRelationFromGedeIndex(GedeIndex_t gedeid) const {
    return fRTRelations[fGeoToRTMap[gedeid-fMinGedeId]-fRTIndexMin] ; }
  
  RTIndex_t GetRTIndexFromGedeIndex(GedeIndex_t gedeid) const {
    return fGeoToRTMap[gedeid-fMinGedeId] ; }
  
  // IO
  void ReadDB() ;
  void WriteDB() ;
  void ReadGeoToRtMap(istream &) ;
  void WriteAll(ostream& os, int format=4) const ;
  void ReadAll(istream& is) ;
    
  // others
  void Dump() const ;
  
protected:
  waxRTManager() ;
private:
  // data
  RTIndex_t fRTIndexMin ;
  RTContainer fRTRelations ; // the rtrelations
  waxRTRelation* fMCRTRelation05mm ;  // rtrelation 05mm cells for MC
  waxRTRelation* fMCRTRelation10mm ;  // rtrelation 10mm cells for MC
  
  GedeIndex_t fMinGedeId ;
  GeoMapContainer fGeoToRTMap ;
  
  static std::string fgConfigDirName ;
  static waxRTManager* fgMe ;
  static int fgSwitch ;
} ;

#endif
